(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    206654,       6456]
NotebookOptionsPosition[    203336,       6334]
NotebookOutlinePosition[    204104,       6366]
CellTagsIndexPosition[    203972,       6359]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["TARCER examples", "Section",
 CellTags->"T1.11.1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$LoadTARCER", "=", "True"}], ";"}]], "Input",
 CellChangeTimes->{{3.522783742124017*^9, 3.522783748194449*^9}}],

Cell[BoxData[
 RowBox[{"<<", "HighEnergyPhysics`FeynCalc`"}]], "Input"],

Cell["Loading FeynCalc from /home/rolfm/HighEnergyPhysics", "Text",
 CellMargins->{{Inherited, Inherited}, {1, 0}},
 GeneratedCell->True,
 CellAutoOverwrite->True],

Cell["\<\
Loading TARCER /home/rolfm/HighEnergyPhysics/Tarcer/tarcerLinuxx8664bit25.mx\
\>", "Text",
 CellMargins->{{Inherited, Inherited}, {1, 0}},
 GeneratedCell->True,
 CellAutoOverwrite->True],

Cell[TextData[{
 StyleBox["FeynCalc",
  FontWeight->"Bold"],
 " ",
 "8.1.0",
 " For help, type ?FeynCalc, open ",
 ButtonBox["FeynCalcRef8.nb",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookOpen[
   "/home/rolfm/HighEnergyPhysics/Documentation/English/FeynCalcRef8.nb"],
  ButtonData:>{"Short Overview", "intro"},
  ButtonNote->"Open FeynCalcRef8.nb"],
 " or visit ",
 ButtonBox["www.feyncalc.org",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.feyncalc.org/"], None},
  ButtonNote->"http://www.feyncalc.org/"]
}], "Text",
 CellMargins->{{Inherited, Inherited}, {1, 0}},
 GeneratedCell->True,
 CellAutoOverwrite->True],

Cell[TextData[{
 "Loading FeynArts, see ",
 ButtonBox["www.feynarts.de",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.feynarts.de/"], None},
  ButtonNote->"http://www.feynarts.de/"],
 " for documentation"
}], "Text",
 CellMargins->{{Inherited, Inherited}, {1, 0}},
 GeneratedCell->True,
 CellAutoOverwrite->True],

Cell["FeynArts 3.4 patched for use with FeynCalc", "Text",
 CellMargins->{{Inherited, Inherited}, {1, 0}},
 GeneratedCell->True,
 CellAutoOverwrite->True],

Cell["\<\
With $Comment one can switch the comment mechanism on. Changing True to False \
will suppress the comment notebook.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"$Comment", "=", "True"}], ";"}], "   "}]], "Input"],

Cell[CellGroupData[{

Cell["Simple Examples", "Subsection",
 CellTags->"T1.11.1.1"],

Cell[TextData[{
 "This reduces (for ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["p", "2"], "=", 
    SuperscriptBox["M", "2"]}], TraditionalForm]]],
 ") \n",
 Cell[BoxData[
  InterpretationBox[
   SubsuperscriptBox[
    StyleBox["F",
     SingleLetterItalics->False,
     FontWeight->"Bold"], 
    RowBox[{
     FormBox[
      RowBox[{"{", 
       RowBox[{"1", ",", "0"}], "}"}],
      TraditionalForm], 
     FormBox[
      RowBox[{"{", 
       RowBox[{"1", ",", "M"}], "}"}],
      TraditionalForm], 
     FormBox[
      RowBox[{"{", 
       RowBox[{"1", ",", "M"}], "}"}],
      TraditionalForm], 
     FormBox[
      RowBox[{"{", 
       RowBox[{"1", ",", "0"}], "}"}],
      TraditionalForm], 
     FormBox[
      RowBox[{"{", 
       RowBox[{"1", ",", "M"}], "}"}],
      TraditionalForm]}], 
    RowBox[{"(", 
     FormBox["d",
      TraditionalForm], ")"}]],
   TFI[d, M^2, {{1, 0}, {1, M}, {1, M}, {1, 0}, {1, M}}],
   Editable->True]]],
 " =\n\[Integral]\[Integral]  ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    FractionBox[
     RowBox[{
      SuperscriptBox["d", "d"], " ", 
      SubscriptBox["q", "1"], 
      SuperscriptBox["d", "d"], 
      SubscriptBox["q", "2"]}], 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"[", 
          SubsuperscriptBox["q", "1", "2"], "]"}], " ", "[", 
         RowBox[{
          SubsuperscriptBox["q", "2", "2"], "-", 
          SuperscriptBox["M", "2"]}], "]"}], " ", "[", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            SubscriptBox["q", "1"], "-", "p"}], ")"}], "2"], "-", 
         SuperscriptBox["M", "2"]}], "]"}], " ", "[", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["q", "2"], "-", "p"}], ")"}], "2"], "]"}], " ", "[", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          SubscriptBox["q", "1"], "-", 
          SubscriptBox["q", "2"]}], ")"}], "2"], "-", 
       SuperscriptBox["M", "2"]}], "]"}]]}], TraditionalForm]]]
}], "Text"],

Cell[CellGroupData[{

Cell["\<\
ex1 = 
   TarcerRecurse[TFI[d, M^2, 
     {{1, 0}, {1, M}, {1, M}, {1, 0}, 
      {1, M}}]]\
\>", "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"3", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"d", "-", "2"}], ")"}], "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"5", " ", "d"}], "-", "18"}], ")"}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        InterpretationBox[
         SubsuperscriptBox[
          StyleBox["A",
           SingleLetterItalics->False,
           FontWeight->"Bold"], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", "M"}], "}"}],
           TraditionalForm], 
          RowBox[{"(", 
           FormBox["d",
            TraditionalForm], ")"}]],
         HighEnergyPhysics`Tarcer`TAI[$CellContext`d, 
          0, {{1, $CellContext`M}}],
         Editable->True], ")"}], "2"]}], 
     RowBox[{"32", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"d", "-", "4"}], ")"}], "2"], " ", 
      RowBox[{"(", 
       RowBox[{"d", "-", "3"}], ")"}], " ", 
      SuperscriptBox["M", "6"]}]]}], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"3", " ", "d"}], "-", "10"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"3", " ", "d"}], "-", "8"}], ")"}], " ", 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "M"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      HighEnergyPhysics`Tarcer`TJI[$CellContext`d, $CellContext`M^2, {{
        1, $CellContext`M}, {1, 0}, {1, 0}}],
      Editable->True]}], 
    RowBox[{"8", " ", 
     RowBox[{"(", 
      RowBox[{"d", "-", "4"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", " ", "d"}], "-", "7"}], ")"}], " ", 
     SuperscriptBox["M", "4"]}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"3", " ", "d"}], "-", "10"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"3", " ", "d"}], "-", "8"}], ")"}], " ", 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "M"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "M"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "M"}], "}"}],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      HighEnergyPhysics`Tarcer`TJI[$CellContext`d, $CellContext`M^2, {{
        1, $CellContext`M}, {1, $CellContext`M}, {1, $CellContext`M}}],
      Editable->True]}], 
    RowBox[{"16", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"d", "-", "4"}], ")"}], "2"], " ", 
     SuperscriptBox["M", "4"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["This expands up to the constant term.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TarcerExpand", "[", 
  RowBox[{"ex1", ",", " ", 
   RowBox[{"d", "->", 
    RowBox[{"4", "+", "\[CurlyEpsilon]"}]}], ",", "0"}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       SuperscriptBox["M", "2"], ")"}], "\[CurlyEpsilon]"], " ", 
     SuperscriptBox[
      InterpretationBox[
       StyleBox[
        SubscriptBox["S", 
         FormBox["\[CurlyEpsilon]",
          TraditionalForm]],
        FontWeight->"Bold"],
       HighEnergyPhysics`Tarcer`SEpsilon[4 + $CellContext`\[CurlyEpsilon]],
       Editable->False], "2"]}], 
    SuperscriptBox["M", "2"]], ".", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"6", " ", 
      RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
      RowBox[{"log", "(", "2", ")"}]}], "-", 
     FractionBox[
      RowBox[{"3", " ", 
       TemplateBox[{"3"},
        "Zeta"]}], "2"]}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "InputForm"}]], "Input"],

Cell["\<\
(((M^2)^\[CurlyEpsilon]*SEpsilon[4 + \[CurlyEpsilon]]^2)/M^2) . 
 (6*Zeta2*Log[2] - (3*Zeta[3])/2)\
\>", "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Moment Examples", "Subsection",
 CellTags->"T1.11.2.1"],

Cell["\<\
eg2[m_]:=TFI[d, M^2,\[CapitalDelta]p,{m,0}, 
     {{2, M}, 1, 1, {1,M}, 
      {1, M}}]\
\>", "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TarcerRecurse", "[", 
  RowBox[{"eg2", "[", "0", "]"}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"d", "-", "2"}], ")"}], "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"49", " ", 
         SuperscriptBox["d", "3"]}], "-", 
        RowBox[{"652", " ", 
         SuperscriptBox["d", "2"]}], "+", 
        RowBox[{"2877", " ", "d"}], "-", "4194"}], ")"}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        InterpretationBox[
         SubsuperscriptBox[
          StyleBox["A",
           SingleLetterItalics->False,
           FontWeight->"Bold"], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", "M"}], "}"}],
           TraditionalForm], 
          RowBox[{"(", 
           FormBox["d",
            TraditionalForm], ")"}]],
         HighEnergyPhysics`Tarcer`TAI[$CellContext`d, 
          0, {{1, $CellContext`M}}],
         Editable->True], ")"}], "2"]}], 
     RowBox[{"256", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"d", "-", "5"}], ")"}], "2"], " ", 
      RowBox[{"(", 
       RowBox[{"d", "-", "4"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"d", "-", "3"}], ")"}], " ", 
      SuperscriptBox["M", "8"]}]]}], "-", 
   FractionBox[
    RowBox[{"3", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"3", " ", "d"}], "-", "10"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"3", " ", "d"}], "-", "8"}], ")"}], " ", 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "M"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      HighEnergyPhysics`Tarcer`TJI[$CellContext`d, $CellContext`M^2, {{
        1, $CellContext`M}, {1, 0}, {1, 0}}],
      Editable->True]}], 
    RowBox[{"32", " ", 
     RowBox[{"(", 
      RowBox[{"d", "-", "5"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", " ", "d"}], "-", "7"}], ")"}], " ", 
     SuperscriptBox["M", "6"]}]], "+", 
   FractionBox[
    RowBox[{"3", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"3", " ", "d"}], "-", "10"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"3", " ", "d"}], "-", "8"}], ")"}], " ", 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "M"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "M"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "M"}], "}"}],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      HighEnergyPhysics`Tarcer`TJI[$CellContext`d, $CellContext`M^2, {{
        1, $CellContext`M}, {1, $CellContext`M}, {1, $CellContext`M}}],
      Editable->True]}], 
    RowBox[{"128", " ", 
     RowBox[{"(", 
      RowBox[{"d", "-", "4"}], ")"}], " ", 
     SuperscriptBox["M", "6"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TarcerExpand", "[", 
  RowBox[{"%", ",", 
   RowBox[{"d", "->", 
    RowBox[{"4", "+", "\[CurlyEpsilon]"}]}], ",", "0"}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       SuperscriptBox["M", "2"], ")"}], "\[CurlyEpsilon]"], " ", 
     SuperscriptBox[
      InterpretationBox[
       StyleBox[
        SubscriptBox["S", 
         FormBox["\[CurlyEpsilon]",
          TraditionalForm]],
        FontWeight->"Bold"],
       HighEnergyPhysics`Tarcer`SEpsilon[4 + $CellContext`\[CurlyEpsilon]],
       Editable->False], "2"]}], 
    SuperscriptBox["M", "4"]], ".", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       RowBox[{"2", " ", 
        SuperscriptBox["\[CurlyEpsilon]", "2"]}]]}], "-", 
     FractionBox[
      RowBox[{"7", " ", 
       RowBox[{"\[Zeta]", "(", "2", ")"}]}], "8"], "+", 
     FractionBox["1", "2"]}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TarcerRecurse", "[", 
  RowBox[{"eg2", "[", "1", "]"}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"d", "-", "2"}], ")"}], "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"27", " ", 
         SuperscriptBox["d", "4"]}], "-", 
        RowBox[{"404", " ", 
         SuperscriptBox["d", "3"]}], "+", 
        RowBox[{"2175", " ", 
         SuperscriptBox["d", "2"]}], "-", 
        RowBox[{"4902", " ", "d"}], "+", "3776"}], ")"}], " ", 
      "\[CapitalDelta]p", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        InterpretationBox[
         SubsuperscriptBox[
          StyleBox["A",
           SingleLetterItalics->False,
           FontWeight->"Bold"], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", "M"}], "}"}],
           TraditionalForm], 
          RowBox[{"(", 
           FormBox["d",
            TraditionalForm], ")"}]],
         HighEnergyPhysics`Tarcer`TAI[$CellContext`d, 
          0, {{1, $CellContext`M}}],
         Editable->True], ")"}], "2"]}], 
     RowBox[{"256", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"d", "-", "5"}], ")"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"d", "-", "4"}], ")"}], "2"], " ", 
      RowBox[{"(", 
       RowBox[{"d", "-", "3"}], ")"}], " ", 
      SuperscriptBox["M", "8"]}]]}], "-", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{"d", "-", "2"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"3", " ", "d"}], "-", "10"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"3", " ", "d"}], "-", "8"}], ")"}], " ", "\[CapitalDelta]p", 
     " ", 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "M"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      HighEnergyPhysics`Tarcer`TJI[$CellContext`d, $CellContext`M^2, {{
        1, $CellContext`M}, {1, 0}, {1, 0}}],
      Editable->True]}], 
    RowBox[{"32", " ", 
     RowBox[{"(", 
      RowBox[{"d", "-", "5"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"d", "-", "4"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", " ", "d"}], "-", "7"}], ")"}], " ", 
     SuperscriptBox["M", "6"]}]], "+", 
   FractionBox[
    RowBox[{"d", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"3", " ", "d"}], "-", "10"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"3", " ", "d"}], "-", "8"}], ")"}], " ", "\[CapitalDelta]p", 
     " ", 
     InterpretationBox[
      SubsuperscriptBox[
       StyleBox["J",
        SingleLetterItalics->False,
        FontWeight->"Bold"], 
       RowBox[{
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "M"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "M"}], "}"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", "M"}], "}"}],
         TraditionalForm]}], 
       RowBox[{"(", 
        FormBox["d",
         TraditionalForm], ")"}]],
      HighEnergyPhysics`Tarcer`TJI[$CellContext`d, $CellContext`M^2, {{
        1, $CellContext`M}, {1, $CellContext`M}, {1, $CellContext`M}}],
      Editable->True]}], 
    RowBox[{"128", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"d", "-", "4"}], ")"}], "2"], " ", 
     SuperscriptBox["M", "6"]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TarcerExpand", "[", 
  RowBox[{"%", ",", 
   RowBox[{"d", "->", 
    RowBox[{"4", "+", "\[CurlyEpsilon]"}]}], ",", "0"}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{"\[CapitalDelta]p", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       SuperscriptBox["M", "2"], ")"}], "\[CurlyEpsilon]"], " ", 
     SuperscriptBox[
      InterpretationBox[
       StyleBox[
        SubscriptBox["S", 
         FormBox["\[CurlyEpsilon]",
          TraditionalForm]],
        FontWeight->"Bold"],
       HighEnergyPhysics`Tarcer`SEpsilon[4 + $CellContext`\[CurlyEpsilon]],
       Editable->False], "2"]}], 
    SuperscriptBox["M", "4"]], ".", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", 
       RowBox[{"2", " ", 
        SuperscriptBox["\[CurlyEpsilon]", "2"]}]]}], "-", 
     FractionBox[
      RowBox[{"11", " ", 
       RowBox[{"\[Zeta]", "(", "2", ")"}]}], "8"], "+", 
     RowBox[{"3", " ", 
      RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
      RowBox[{"log", "(", "2", ")"}]}], "-", 
     FractionBox[
      RowBox[{"3", " ", 
       TemplateBox[{"3"},
        "Zeta"]}], "4"], "+", 
     FractionBox["1", "2"]}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["General massive examples", "Subsection",
 CellTags->"T1.11.2.1"],

Cell["Typesetting rules:", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"pp", ",", "fmt_"}], "]"}], ":=", 
   RowBox[{"InterpretationBox", "[", 
    RowBox[{
     RowBox[{"SuperscriptBox", "[", 
      RowBox[{"\"\<p\>\"", ",", "2"}], "]"}], ",", "pp"}], "]"}]}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"m2", ",", "fmt_"}], "]"}], ":=", 
   RowBox[{"InterpretationBox", "[", 
    RowBox[{
     RowBox[{"SubscriptBox", "[", 
      RowBox[{"\"\<m\>\"", ",", "2"}], "]"}], ",", "m2"}], "]"}]}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"m3", ",", "fmt_"}], "]"}], ":=", 
   RowBox[{"InterpretationBox", "[", 
    RowBox[{
     RowBox[{"SubscriptBox", "[", 
      RowBox[{"\"\<m\>\"", ",", "3"}], "]"}], ",", "m3"}], "]"}]}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"m4", ",", "fmt_"}], "]"}], ":=", 
   RowBox[{"InterpretationBox", "[", 
    RowBox[{
     RowBox[{"SubscriptBox", "[", 
      RowBox[{"\"\<m\>\"", ",", "4"}], "]"}], ",", "m4"}], "]"}]}], 
  ";"}]}], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 FormBox[
  SubsuperscriptBox["Y", "234", "1"], TraditionalForm]]]], "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", "\n", 
  RowBox[{"y1234", " ", "=", " ", 
   RowBox[{"TarcerRecurse", "[", " ", 
    RowBox[{"TFI", "[", 
     RowBox[{"D", ",", "pp", ",", 
      RowBox[{"{", 
       RowBox[{"1", ",", "0", ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", " ", 
        RowBox[{"{", 
         RowBox[{"1", ",", "m4"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"1", ",", "m2"}], "}"}], ",", "0", ",", 
        RowBox[{"{", 
         RowBox[{"1", ",", "m3"}], "}"}]}], "}"}]}], "]"}], " ", "]"}]}], 
  "\n", "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"0.048002999999999574`", ",", 
    RowBox[{
     RowBox[{
      FractionBox["1", "3"], " ", 
      InterpretationBox[
       SubsuperscriptBox[
        StyleBox["A",
         SingleLetterItalics->False,
         FontWeight->"Bold"], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", 
           InterpretationBox[
            SubscriptBox["m", "2"],
            $CellContext`m2]}], "}"}],
         TraditionalForm], 
        RowBox[{"(", 
         FormBox["D",
          TraditionalForm], ")"}]],
       HighEnergyPhysics`Tarcer`TAI[D, 0, {{1, $CellContext`m2}}],
       Editable->True], " ", 
      InterpretationBox[
       SubsuperscriptBox[
        StyleBox["A",
         SingleLetterItalics->False,
         FontWeight->"Bold"], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", 
           InterpretationBox[
            SubscriptBox["m", "3"],
            $CellContext`m3]}], "}"}],
         TraditionalForm], 
        RowBox[{"(", 
         FormBox["D",
          TraditionalForm], ")"}]],
       HighEnergyPhysics`Tarcer`TAI[D, 0, {{1, $CellContext`m3}}],
       Editable->True]}], "+", 
     RowBox[{
      FractionBox["1", "3"], " ", 
      InterpretationBox[
       SubsuperscriptBox[
        StyleBox["A",
         SingleLetterItalics->False,
         FontWeight->"Bold"], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", 
           InterpretationBox[
            SubscriptBox["m", "2"],
            $CellContext`m2]}], "}"}],
         TraditionalForm], 
        RowBox[{"(", 
         FormBox["D",
          TraditionalForm], ")"}]],
       HighEnergyPhysics`Tarcer`TAI[D, 0, {{1, $CellContext`m2}}],
       Editable->True], " ", 
      InterpretationBox[
       SubsuperscriptBox[
        StyleBox["A",
         SingleLetterItalics->False,
         FontWeight->"Bold"], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", 
           InterpretationBox[
            SubscriptBox["m", "4"],
            $CellContext`m4]}], "}"}],
         TraditionalForm], 
        RowBox[{"(", 
         FormBox["D",
          TraditionalForm], ")"}]],
       HighEnergyPhysics`Tarcer`TAI[D, 0, {{1, $CellContext`m4}}],
       Editable->True]}], "+", 
     RowBox[{
      FractionBox["1", "3"], " ", 
      InterpretationBox[
       SubsuperscriptBox[
        StyleBox["A",
         SingleLetterItalics->False,
         FontWeight->"Bold"], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", 
           InterpretationBox[
            SubscriptBox["m", "3"],
            $CellContext`m3]}], "}"}],
         TraditionalForm], 
        RowBox[{"(", 
         FormBox["D",
          TraditionalForm], ")"}]],
       HighEnergyPhysics`Tarcer`TAI[D, 0, {{1, $CellContext`m3}}],
       Editable->True], " ", 
      InterpretationBox[
       SubsuperscriptBox[
        StyleBox["A",
         SingleLetterItalics->False,
         FontWeight->"Bold"], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", 
           InterpretationBox[
            SubscriptBox["m", "4"],
            $CellContext`m4]}], "}"}],
         TraditionalForm], 
        RowBox[{"(", 
         FormBox["D",
          TraditionalForm], ")"}]],
       HighEnergyPhysics`Tarcer`TAI[D, 0, {{1, $CellContext`m4}}],
       Editable->True]}], "-", 
     FractionBox[
      RowBox[{"4", " ", 
       SuperscriptBox[
        RowBox[{"(", 
         InterpretationBox[
          SubscriptBox["m", "2"],
          $CellContext`m2], ")"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           InterpretationBox[
            SubscriptBox["m", "2"],
            $CellContext`m2], ")"}], "2"], "-", 
         InterpretationBox[
          SuperscriptBox["p", "2"],
          $CellContext`pp]}], ")"}], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["J",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         RowBox[{
          FormBox[
           RowBox[{"{", 
            RowBox[{"2", ",", 
             InterpretationBox[
              SubscriptBox["m", "2"],
              $CellContext`m2]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "4"],
              $CellContext`m4]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "3"],
              $CellContext`m3]}], "}"}],
           TraditionalForm]}], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TJI[
        D, $CellContext`pp, {{2, $CellContext`m2}, {1, $CellContext`m4}, {
          1, $CellContext`m3}}],
        Editable->True]}], 
      RowBox[{"3", " ", 
       RowBox[{"(", 
        RowBox[{"D", "-", "2"}], ")"}]}]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"7", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "-", 
         RowBox[{"2", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"]}], "-", 
         RowBox[{"2", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"]}], "+", 
         RowBox[{"D", " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}], "-", 
         RowBox[{"18", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "+", 
         RowBox[{"6", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"]}], "+", 
         RowBox[{"6", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"]}], "-", 
         RowBox[{"2", " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}]}], ")"}], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["J",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         RowBox[{
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "4"],
              $CellContext`m4]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "3"],
              $CellContext`m3]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "2"],
              $CellContext`m2]}], "}"}],
           TraditionalForm]}], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TJI[
        D, $CellContext`pp, {{1, $CellContext`m4}, {1, $CellContext`m3}, {
          1, $CellContext`m2}}],
        Editable->True]}], 
      RowBox[{"3", " ", 
       RowBox[{"(", 
        RowBox[{"D", "-", "2"}], ")"}]}]], "+", 
     FractionBox[
      RowBox[{"2", " ", 
       SuperscriptBox[
        RowBox[{"(", 
         InterpretationBox[
          SubscriptBox["m", "3"],
          $CellContext`m3], ")"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"3", " ", 
           SuperscriptBox[
            RowBox[{"(", 
             InterpretationBox[
              SubscriptBox["m", "2"],
              $CellContext`m2], ")"}], "2"]}]}], "+", 
         SuperscriptBox[
          RowBox[{"(", 
           InterpretationBox[
            SubscriptBox["m", "3"],
            $CellContext`m3], ")"}], "2"], "+", 
         RowBox[{"3", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"]}], "-", 
         InterpretationBox[
          SuperscriptBox["p", "2"],
          $CellContext`pp]}], ")"}], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["J",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         RowBox[{
          FormBox[
           RowBox[{"{", 
            RowBox[{"2", ",", 
             InterpretationBox[
              SubscriptBox["m", "3"],
              $CellContext`m3]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "4"],
              $CellContext`m4]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "2"],
              $CellContext`m2]}], "}"}],
           TraditionalForm]}], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TJI[
        D, $CellContext`pp, {{2, $CellContext`m3}, {1, $CellContext`m4}, {
          1, $CellContext`m2}}],
        Editable->True]}], 
      RowBox[{"3", " ", 
       RowBox[{"(", 
        RowBox[{"D", "-", "2"}], ")"}]}]], "+", 
     FractionBox[
      RowBox[{"2", " ", 
       SuperscriptBox[
        RowBox[{"(", 
         InterpretationBox[
          SubscriptBox["m", "4"],
          $CellContext`m4], ")"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"3", " ", 
           SuperscriptBox[
            RowBox[{"(", 
             InterpretationBox[
              SubscriptBox["m", "2"],
              $CellContext`m2], ")"}], "2"]}]}], "+", 
         RowBox[{"3", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"]}], "+", 
         SuperscriptBox[
          RowBox[{"(", 
           InterpretationBox[
            SubscriptBox["m", "4"],
            $CellContext`m4], ")"}], "2"], "-", 
         InterpretationBox[
          SuperscriptBox["p", "2"],
          $CellContext`pp]}], ")"}], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["J",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         RowBox[{
          FormBox[
           RowBox[{"{", 
            RowBox[{"2", ",", 
             InterpretationBox[
              SubscriptBox["m", "4"],
              $CellContext`m4]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "3"],
              $CellContext`m3]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "2"],
              $CellContext`m2]}], "}"}],
           TraditionalForm]}], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TJI[
        D, $CellContext`pp, {{2, $CellContext`m4}, {1, $CellContext`m3}, {
          1, $CellContext`m2}}],
        Editable->True]}], 
      RowBox[{"3", " ", 
       RowBox[{"(", 
        RowBox[{"D", "-", "2"}], ")"}]}]]}]}], "}"}], 
  TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 FormBox[
  SubsuperscriptBox["Y", "2345", "1"], TraditionalForm]]]], "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", "\n", 
  RowBox[{"y12345", " ", "=", " ", 
   RowBox[{"TarcerRecurse", "[", " ", 
    RowBox[{"TFI", "[", 
     RowBox[{"D", ",", "pp", ",", 
      RowBox[{"{", 
       RowBox[{"1", ",", "0", ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", " ", 
        RowBox[{"{", 
         RowBox[{"1", ",", "m4"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"1", ",", "m2"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"1", ",", "m4"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"1", ",", "m3"}], "}"}]}], "}"}]}], "]"}], " ", "]"}]}], 
  "\n", "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"1.0720670000000005`", ",", 
    RowBox[{
     FractionBox[
      RowBox[{
       InterpretationBox[
        SuperscriptBox["p", "2"],
        $CellContext`pp], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["A",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         FormBox[
          RowBox[{"{", 
           RowBox[{"1", ",", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2]}], "}"}],
          TraditionalForm], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TAI[D, 0, {{1, $CellContext`m2}}],
        Editable->True], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["B",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         RowBox[{
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "4"],
              $CellContext`m4]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "4"],
              $CellContext`m4]}], "}"}],
           TraditionalForm]}], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TBI[
        D, $CellContext`pp, {{1, $CellContext`m4}, {1, $CellContext`m4}}],
        Editable->True]}], 
      RowBox[{"2", " ", 
       SuperscriptBox[
        RowBox[{"(", 
         InterpretationBox[
          SubscriptBox["m", "4"],
          $CellContext`m4], ")"}], "2"]}]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"2", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"]}], "-", 
         InterpretationBox[
          SuperscriptBox["p", "2"],
          $CellContext`pp]}], ")"}], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["A",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         FormBox[
          RowBox[{"{", 
           RowBox[{"1", ",", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3]}], "}"}],
          TraditionalForm], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TAI[D, 0, {{1, $CellContext`m3}}],
        Editable->True], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["B",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         RowBox[{
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "4"],
              $CellContext`m4]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "4"],
              $CellContext`m4]}], "}"}],
           TraditionalForm]}], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TBI[
        D, $CellContext`pp, {{1, $CellContext`m4}, {1, $CellContext`m4}}],
        Editable->True]}], 
      RowBox[{"2", " ", 
       SuperscriptBox[
        RowBox[{"(", 
         InterpretationBox[
          SubscriptBox["m", "4"],
          $CellContext`m4], ")"}], "2"]}]], "-", 
     FractionBox[
      RowBox[{
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["A",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         FormBox[
          RowBox[{"{", 
           RowBox[{"1", ",", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2]}], "}"}],
          TraditionalForm], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TAI[D, 0, {{1, $CellContext`m2}}],
        Editable->True], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["A",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         FormBox[
          RowBox[{"{", 
           RowBox[{"1", ",", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4]}], "}"}],
          TraditionalForm], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TAI[D, 0, {{1, $CellContext`m4}}],
        Editable->True]}], 
      RowBox[{"2", " ", 
       SuperscriptBox[
        RowBox[{"(", 
         InterpretationBox[
          SubscriptBox["m", "4"],
          $CellContext`m4], ")"}], "2"]}]], "+", 
     FractionBox[
      RowBox[{
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["A",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         FormBox[
          RowBox[{"{", 
           RowBox[{"1", ",", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3]}], "}"}],
          TraditionalForm], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TAI[D, 0, {{1, $CellContext`m3}}],
        Editable->True], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["A",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         FormBox[
          RowBox[{"{", 
           RowBox[{"1", ",", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4]}], "}"}],
          TraditionalForm], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TAI[D, 0, {{1, $CellContext`m4}}],
        Editable->True]}], 
      RowBox[{"2", " ", 
       SuperscriptBox[
        RowBox[{"(", 
         InterpretationBox[
          SubscriptBox["m", "4"],
          $CellContext`m4], ")"}], "2"]}]], "+", 
     FractionBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         InterpretationBox[
          SubscriptBox["m", "2"],
          $CellContext`m2], ")"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           InterpretationBox[
            SubscriptBox["m", "2"],
            $CellContext`m2], ")"}], "2"], "-", 
         SuperscriptBox[
          RowBox[{"(", 
           InterpretationBox[
            SubscriptBox["m", "3"],
            $CellContext`m3], ")"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{"(", 
           InterpretationBox[
            SubscriptBox["m", "4"],
            $CellContext`m4], ")"}], "2"], "-", 
         InterpretationBox[
          SuperscriptBox["p", "2"],
          $CellContext`pp]}], ")"}], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["J",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         RowBox[{
          FormBox[
           RowBox[{"{", 
            RowBox[{"2", ",", 
             InterpretationBox[
              SubscriptBox["m", "2"],
              $CellContext`m2]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "4"],
              $CellContext`m4]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "3"],
              $CellContext`m3]}], "}"}],
           TraditionalForm]}], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TJI[
        D, $CellContext`pp, {{2, $CellContext`m2}, {1, $CellContext`m4}, {
          1, $CellContext`m3}}],
        Editable->True]}], 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"D", "-", "2"}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         InterpretationBox[
          SubscriptBox["m", "4"],
          $CellContext`m4], ")"}], "2"]}]], "-", 
     FractionBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         InterpretationBox[
          SubscriptBox["m", "3"],
          $CellContext`m3], ")"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "+", 
         SuperscriptBox[
          RowBox[{"(", 
           InterpretationBox[
            SubscriptBox["m", "3"],
            $CellContext`m3], ")"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{"(", 
           InterpretationBox[
            SubscriptBox["m", "4"],
            $CellContext`m4], ")"}], "2"], "-", 
         InterpretationBox[
          SuperscriptBox["p", "2"],
          $CellContext`pp]}], ")"}], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["J",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         RowBox[{
          FormBox[
           RowBox[{"{", 
            RowBox[{"2", ",", 
             InterpretationBox[
              SubscriptBox["m", "3"],
              $CellContext`m3]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "4"],
              $CellContext`m4]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "2"],
              $CellContext`m2]}], "}"}],
           TraditionalForm]}], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TJI[
        D, $CellContext`pp, {{2, $CellContext`m3}, {1, $CellContext`m4}, {
          1, $CellContext`m2}}],
        Editable->True]}], 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"D", "-", "2"}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         InterpretationBox[
          SubscriptBox["m", "4"],
          $CellContext`m4], ")"}], "2"]}]], "-", 
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"3", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "-", 
         RowBox[{"3", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"]}], "+", 
         RowBox[{"D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"]}], "-", 
         RowBox[{"8", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "+", 
         RowBox[{"8", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"]}], "-", 
         RowBox[{"2", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"]}]}], ")"}], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["J",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         RowBox[{
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "4"],
              $CellContext`m4]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "3"],
              $CellContext`m3]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "2"],
              $CellContext`m2]}], "}"}],
           TraditionalForm]}], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TJI[
        D, $CellContext`pp, {{1, $CellContext`m4}, {1, $CellContext`m3}, {
          1, $CellContext`m2}}],
        Editable->True]}], 
      RowBox[{"2", " ", 
       RowBox[{"(", 
        RowBox[{"D", "-", "2"}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         InterpretationBox[
          SubscriptBox["m", "4"],
          $CellContext`m4], ")"}], "2"]}]], "+", 
     FractionBox[
      RowBox[{"2", " ", 
       RowBox[{"(", 
        RowBox[{
         InterpretationBox[
          SubscriptBox["m", "2"],
          $CellContext`m2], "-", 
         InterpretationBox[
          SubscriptBox["m", "3"],
          $CellContext`m3]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         InterpretationBox[
          SubscriptBox["m", "2"],
          $CellContext`m2], "+", 
         InterpretationBox[
          SubscriptBox["m", "3"],
          $CellContext`m3]}], ")"}], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["J",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         RowBox[{
          FormBox[
           RowBox[{"{", 
            RowBox[{"2", ",", 
             InterpretationBox[
              SubscriptBox["m", "4"],
              $CellContext`m4]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "3"],
              $CellContext`m3]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "2"],
              $CellContext`m2]}], "}"}],
           TraditionalForm]}], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TJI[
        D, $CellContext`pp, {{2, $CellContext`m4}, {1, $CellContext`m3}, {
          1, $CellContext`m2}}],
        Editable->True]}], 
      RowBox[{"D", "-", "2"}]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           InterpretationBox[
            SubscriptBox["m", "2"],
            $CellContext`m2], ")"}], "2"], "-", 
         SuperscriptBox[
          RowBox[{"(", 
           InterpretationBox[
            SubscriptBox["m", "3"],
            $CellContext`m3], ")"}], "2"], "+", 
         SuperscriptBox[
          RowBox[{"(", 
           InterpretationBox[
            SubscriptBox["m", "4"],
            $CellContext`m4], ")"}], "2"]}], ")"}], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["K",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         RowBox[{
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "4"],
              $CellContext`m4]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "3"],
              $CellContext`m3]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "2"],
              $CellContext`m2]}], "}"}],
           TraditionalForm]}], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TJI[
        D, 0, {{1, $CellContext`m4}, {1, $CellContext`m3}, {
          1, $CellContext`m2}}],
        Editable->True]}], 
      RowBox[{"2", " ", 
       SuperscriptBox[
        RowBox[{"(", 
         InterpretationBox[
          SubscriptBox["m", "4"],
          $CellContext`m4], ")"}], "2"]}]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             InterpretationBox[
              SubscriptBox["m", "2"],
              $CellContext`m2], ")"}], "2"], " ", 
           InterpretationBox[
            SuperscriptBox["p", "2"],
            $CellContext`pp]}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}], "+", 
         RowBox[{"2", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}]}], ")"}], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["V",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         RowBox[{
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "3"],
              $CellContext`m3]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "4"],
              $CellContext`m4]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "2"],
              $CellContext`m2]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "4"],
              $CellContext`m4]}], "}"}],
           TraditionalForm]}], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TVI[
        D, $CellContext`pp, {{1, $CellContext`m3}, {1, $CellContext`m4}, {
          1, $CellContext`m2}, {1, $CellContext`m4}}],
        Editable->True]}], 
      RowBox[{"2", " ", 
       SuperscriptBox[
        RowBox[{"(", 
         InterpretationBox[
          SubscriptBox["m", "4"],
          $CellContext`m4], ")"}], "2"]}]]}]}], "}"}], 
  TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 FormBox[
  SubsuperscriptBox["Y", "234", "11"], TraditionalForm]]]], "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", "\n", 
  RowBox[{"y11234", " ", "=", " ", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"TarcerRecurse", "[", " ", 
       RowBox[{"TFI", "[", 
        RowBox[{"D", ",", "pp", ",", 
         RowBox[{"{", 
          RowBox[{"2", ",", "0", ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", " ", 
           RowBox[{"{", 
            RowBox[{"1", ",", "m4"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"1", ",", "m2"}], "}"}], ",", "0", ",", 
           RowBox[{"{", 
            RowBox[{"1", ",", "m3"}], "}"}]}], "}"}]}], "]"}], "]"}], "/.", 
      RowBox[{"Plus", "\[Rule]", 
       RowBox[{"Hold", "[", "Plus", "]"}]}]}], "/.", 
     RowBox[{
      RowBox[{
       RowBox[{"Hold", "[", "Plus", "]"}], "[", "a__", "]"}], 
      "\[RuleDelayed]", 
      RowBox[{
       RowBox[{"FullSimplify", "[", 
        RowBox[{"Plus", "[", "a", "]"}], "]"}], "/;", 
       RowBox[{"FreeQ2", "[", 
        RowBox[{
         RowBox[{"{", "a", "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"TAI", ",", "TBI", ",", "TVI"}], "}"}]}], "]"}]}]}]}], "//",
     "ReleaseHold"}]}], "\n", "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"2.4721550000000008`", ",", 
    RowBox[{
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"D", " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"(", 
              InterpretationBox[
               SubscriptBox["m", "2"],
               $CellContext`m2], ")"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{"(", 
              InterpretationBox[
               SubscriptBox["m", "3"],
               $CellContext`m3], ")"}], "2"], "+", 
            RowBox[{"7", " ", 
             SuperscriptBox[
              RowBox[{"(", 
               InterpretationBox[
                SubscriptBox["m", "4"],
                $CellContext`m4], ")"}], "2"]}], "+", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             $CellContext`pp]}], ")"}]}], "-", 
         RowBox[{"12", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"]}]}], ")"}], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["A",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         FormBox[
          RowBox[{"{", 
           RowBox[{"1", ",", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2]}], "}"}],
          TraditionalForm], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TAI[D, 0, {{1, $CellContext`m2}}],
        Editable->True], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["A",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         FormBox[
          RowBox[{"{", 
           RowBox[{"1", ",", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3]}], "}"}],
          TraditionalForm], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TAI[D, 0, {{1, $CellContext`m3}}],
        Editable->True]}], 
      RowBox[{"3", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"3", " ", "D"}], "-", "4"}], ")"}]}]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"D", " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"(", 
              InterpretationBox[
               SubscriptBox["m", "2"],
               $CellContext`m2], ")"}], "2"], "+", 
            RowBox[{"7", " ", 
             SuperscriptBox[
              RowBox[{"(", 
               InterpretationBox[
                SubscriptBox["m", "3"],
                $CellContext`m3], ")"}], "2"]}], "+", 
            SuperscriptBox[
             RowBox[{"(", 
              InterpretationBox[
               SubscriptBox["m", "4"],
               $CellContext`m4], ")"}], "2"], "+", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             $CellContext`pp]}], ")"}]}], "-", 
         RowBox[{"12", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"]}]}], ")"}], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["A",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         FormBox[
          RowBox[{"{", 
           RowBox[{"1", ",", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2]}], "}"}],
          TraditionalForm], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TAI[D, 0, {{1, $CellContext`m2}}],
        Editable->True], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["A",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         FormBox[
          RowBox[{"{", 
           RowBox[{"1", ",", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4]}], "}"}],
          TraditionalForm], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TAI[D, 0, {{1, $CellContext`m4}}],
        Editable->True]}], 
      RowBox[{"3", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"3", " ", "D"}], "-", "4"}], ")"}]}]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"7", " ", "D"}], "-", "12"}], ")"}], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "-", 
         RowBox[{"4", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", "D"}], "-", "3"}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"(", 
              InterpretationBox[
               SubscriptBox["m", "3"],
               $CellContext`m3], ")"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{"(", 
              InterpretationBox[
               SubscriptBox["m", "4"],
               $CellContext`m4], ")"}], "2"]}], ")"}]}], "+", 
         RowBox[{"D", " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}]}], ")"}], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["A",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         FormBox[
          RowBox[{"{", 
           RowBox[{"1", ",", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3]}], "}"}],
          TraditionalForm], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TAI[D, 0, {{1, $CellContext`m3}}],
        Editable->True], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["A",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         FormBox[
          RowBox[{"{", 
           RowBox[{"1", ",", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4]}], "}"}],
          TraditionalForm], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TAI[D, 0, {{1, $CellContext`m4}}],
        Editable->True]}], 
      RowBox[{"3", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"3", " ", "D"}], "-", "4"}], ")"}]}]], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          InterpretationBox[
           SubscriptBox["m", "3"],
           $CellContext`m3], ")"}], "2"], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          InterpretationBox[
           SubscriptBox["m", "4"],
           $CellContext`m4], ")"}], "2"]}], ")"}], " ", 
      InterpretationBox[
       SubsuperscriptBox[
        StyleBox["A",
         SingleLetterItalics->False,
         FontWeight->"Bold"], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", 
           InterpretationBox[
            SubscriptBox["m", "4"],
            $CellContext`m4]}], "}"}],
         TraditionalForm], 
        RowBox[{"(", 
         FormBox["D",
          TraditionalForm], ")"}]],
       HighEnergyPhysics`Tarcer`TAI[D, 0, {{1, $CellContext`m4}}],
       Editable->True], " ", 
      InterpretationBox[
       SubsuperscriptBox[
        StyleBox["A",
         SingleLetterItalics->False,
         FontWeight->"Bold"], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", 
           InterpretationBox[
            SubscriptBox["m", "3"],
            $CellContext`m3]}], "}"}],
         TraditionalForm], 
        RowBox[{"(", 
         FormBox["D",
          TraditionalForm], ")"}]],
       HighEnergyPhysics`Tarcer`TAI[D, 0, {{1, $CellContext`m3}}],
       Editable->True]}], "-", 
     FractionBox[
      RowBox[{"8", " ", 
       SuperscriptBox[
        RowBox[{"(", 
         InterpretationBox[
          SubscriptBox["m", "2"],
          $CellContext`m2], ")"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           InterpretationBox[
            SubscriptBox["m", "2"],
            $CellContext`m2], ")"}], "2"], "-", 
         InterpretationBox[
          SuperscriptBox["p", "2"],
          $CellContext`pp]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", "D"}], "-", "3"}], ")"}], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "-", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"D", "-", "3"}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"(", 
              InterpretationBox[
               SubscriptBox["m", "3"],
               $CellContext`m3], ")"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{"(", 
              InterpretationBox[
               SubscriptBox["m", "4"],
               $CellContext`m4], ")"}], "2"]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", "D"}], "-", "3"}], ")"}], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}]}], ")"}], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["J",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         RowBox[{
          FormBox[
           RowBox[{"{", 
            RowBox[{"2", ",", 
             InterpretationBox[
              SubscriptBox["m", "2"],
              $CellContext`m2]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "4"],
              $CellContext`m4]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "3"],
              $CellContext`m3]}], "}"}],
           TraditionalForm]}], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TJI[
        D, $CellContext`pp, {{2, $CellContext`m2}, {1, $CellContext`m4}, {
          1, $CellContext`m3}}],
        Editable->True]}], 
      RowBox[{"3", " ", 
       RowBox[{"(", 
        RowBox[{"D", "-", "2"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"3", " ", "D"}], "-", "4"}], ")"}]}]], "+", 
     RowBox[{
      FractionBox["1", 
       RowBox[{"3", " ", 
        RowBox[{"(", 
         RowBox[{"D", "-", "2"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"3", " ", "D"}], "-", "4"}], ")"}]}]], 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"2", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{"19", " ", 
                RowBox[{"(", 
                 RowBox[{"D", "-", "4"}], ")"}], " ", "D"}], "+", "72"}], 
              ")"}], " ", 
             InterpretationBox[
              SuperscriptBox["p", "2"],
              $CellContext`pp]}], "-", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"D", "-", "3"}], ")"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"5", " ", "D"}], "-", "12"}], ")"}], " ", 
             RowBox[{"(", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 InterpretationBox[
                  SubscriptBox["m", "3"],
                  $CellContext`m3], ")"}], "2"], "+", 
               SuperscriptBox[
                RowBox[{"(", 
                 InterpretationBox[
                  SubscriptBox["m", "4"],
                  $CellContext`m4], ")"}], "2"]}], ")"}]}]}], ")"}]}], "+", 
         RowBox[{"2", " ", 
          RowBox[{"(", 
           RowBox[{"D", "-", "4"}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{"D", "-", "3"}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"(", 
              InterpretationBox[
               SubscriptBox["m", "3"],
               $CellContext`m3], ")"}], "2"], "+", 
            SuperscriptBox[
             RowBox[{"(", 
              InterpretationBox[
               SubscriptBox["m", "4"],
               $CellContext`m4], ")"}], "2"]}], ")"}], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}], "+", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"D", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"25", " ", "D"}], "-", "102"}], ")"}]}], "+", "96"}], 
           ")"}], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "4"]}], "-", 
         RowBox[{"2", " ", 
          RowBox[{"(", 
           RowBox[{"D", "-", "3"}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"D", " ", 
             SuperscriptBox[
              RowBox[{"(", 
               InterpretationBox[
                SubscriptBox["m", "3"],
                $CellContext`m3], ")"}], "4"]}], "+", 
            RowBox[{"2", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"7", " ", "D"}], "-", "12"}], ")"}], " ", 
             SuperscriptBox[
              RowBox[{"(", 
               InterpretationBox[
                SubscriptBox["m", "4"],
                $CellContext`m4], ")"}], "2"], " ", 
             SuperscriptBox[
              RowBox[{"(", 
               InterpretationBox[
                SubscriptBox["m", "3"],
                $CellContext`m3], ")"}], "2"]}], "+", 
            RowBox[{"D", " ", 
             SuperscriptBox[
              RowBox[{"(", 
               InterpretationBox[
                SubscriptBox["m", "4"],
                $CellContext`m4], ")"}], "4"]}]}], ")"}]}], "+", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"D", "-", "2"}], ")"}], " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             $CellContext`pp], ")"}], "2"]}]}], ")"}], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["J",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         RowBox[{
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "4"],
              $CellContext`m4]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "3"],
              $CellContext`m3]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "2"],
              $CellContext`m2]}], "}"}],
           TraditionalForm]}], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TJI[
        D, $CellContext`pp, {{1, $CellContext`m4}, {1, $CellContext`m3}, {
          1, $CellContext`m2}}],
        Editable->True]}]}], "-", 
     RowBox[{
      FractionBox["1", 
       RowBox[{"3", " ", 
        RowBox[{"(", 
         RowBox[{"D", "-", "2"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"3", " ", "D"}], "-", "4"}], ")"}]}]], 
      RowBox[{"2", " ", 
       SuperscriptBox[
        RowBox[{"(", 
         InterpretationBox[
          SubscriptBox["m", "3"],
          $CellContext`m3], ")"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", 
             RowBox[{"4", " ", 
              RowBox[{"(", 
               RowBox[{"D", "-", "3"}], ")"}], " ", 
              SuperscriptBox[
               RowBox[{"(", 
                InterpretationBox[
                 SubscriptBox["m", "3"],
                 $CellContext`m3], ")"}], "2"]}]}], "+", 
            RowBox[{"22", " ", "D", " ", 
             InterpretationBox[
              SuperscriptBox["p", "2"],
              $CellContext`pp]}], "-", 
            RowBox[{"36", " ", 
             InterpretationBox[
              SuperscriptBox["p", "2"],
              $CellContext`pp]}]}], ")"}]}], "+", 
         RowBox[{"D", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"4", " ", 
             SuperscriptBox[
              RowBox[{"(", 
               InterpretationBox[
                SubscriptBox["m", "4"],
                $CellContext`m4], ")"}], "2"], " ", 
             InterpretationBox[
              SuperscriptBox["p", "2"],
              $CellContext`pp]}], "-", 
            SuperscriptBox[
             RowBox[{"(", 
              InterpretationBox[
               SubscriptBox["m", "3"],
               $CellContext`m3], ")"}], "4"], "-", 
            RowBox[{"22", " ", 
             SuperscriptBox[
              RowBox[{"(", 
               InterpretationBox[
                SubscriptBox["m", "4"],
                $CellContext`m4], ")"}], "2"], " ", 
             SuperscriptBox[
              RowBox[{"(", 
               InterpretationBox[
                SubscriptBox["m", "3"],
                $CellContext`m3], ")"}], "2"]}], "-", 
            RowBox[{"9", " ", 
             SuperscriptBox[
              RowBox[{"(", 
               InterpretationBox[
                SubscriptBox["m", "4"],
                $CellContext`m4], ")"}], "4"]}], "+", 
            SuperscriptBox[
             RowBox[{"(", 
              InterpretationBox[
               SuperscriptBox["p", "2"],
               $CellContext`pp], ")"}], "2"]}], ")"}]}], "+", 
         RowBox[{"3", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"3", " ", "D"}], "-", "4"}], ")"}], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "4"]}], "+", 
         RowBox[{"12", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"3", " ", 
             SuperscriptBox[
              RowBox[{"(", 
               InterpretationBox[
                SubscriptBox["m", "3"],
                $CellContext`m3], ")"}], "2"]}], "+", 
            SuperscriptBox[
             RowBox[{"(", 
              InterpretationBox[
               SubscriptBox["m", "4"],
               $CellContext`m4], ")"}], "2"], "-", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             $CellContext`pp]}], ")"}]}]}], ")"}], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["J",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         RowBox[{
          FormBox[
           RowBox[{"{", 
            RowBox[{"2", ",", 
             InterpretationBox[
              SubscriptBox["m", "3"],
              $CellContext`m3]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "4"],
              $CellContext`m4]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "2"],
              $CellContext`m2]}], "}"}],
           TraditionalForm]}], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TJI[
        D, $CellContext`pp, {{2, $CellContext`m3}, {1, $CellContext`m4}, {
          1, $CellContext`m2}}],
        Editable->True]}]}], "-", 
     RowBox[{
      FractionBox["1", 
       RowBox[{"3", " ", 
        RowBox[{"(", 
         RowBox[{"D", "-", "2"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"3", " ", "D"}], "-", "4"}], ")"}]}]], 
      RowBox[{"2", " ", 
       SuperscriptBox[
        RowBox[{"(", 
         InterpretationBox[
          SubscriptBox["m", "4"],
          $CellContext`m4], ")"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", 
             RowBox[{"4", " ", 
              RowBox[{"(", 
               RowBox[{"D", "-", "3"}], ")"}], " ", 
              SuperscriptBox[
               RowBox[{"(", 
                InterpretationBox[
                 SubscriptBox["m", "4"],
                 $CellContext`m4], ")"}], "2"]}]}], "+", 
            RowBox[{"22", " ", "D", " ", 
             InterpretationBox[
              SuperscriptBox["p", "2"],
              $CellContext`pp]}], "-", 
            RowBox[{"36", " ", 
             InterpretationBox[
              SuperscriptBox["p", "2"],
              $CellContext`pp]}]}], ")"}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"(", 
              RowBox[{"36", "-", 
               RowBox[{"22", " ", "D"}]}], ")"}], " ", 
             SuperscriptBox[
              RowBox[{"(", 
               InterpretationBox[
                SubscriptBox["m", "4"],
                $CellContext`m4], ")"}], "2"]}], "+", 
            RowBox[{"4", " ", 
             RowBox[{"(", 
              RowBox[{"D", "-", "3"}], ")"}], " ", 
             InterpretationBox[
              SuperscriptBox["p", "2"],
              $CellContext`pp]}]}], ")"}]}], "+", 
         RowBox[{"D", " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"(", 
              InterpretationBox[
               SuperscriptBox["p", "2"],
               $CellContext`pp], ")"}], "2"], "-", 
            SuperscriptBox[
             RowBox[{"(", 
              InterpretationBox[
               SubscriptBox["m", "4"],
               $CellContext`m4], ")"}], "4"]}], ")"}]}], "+", 
         RowBox[{"3", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"3", " ", "D"}], "-", "4"}], ")"}], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "4"]}], "+", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"12", "-", 
            RowBox[{"9", " ", "D"}]}], ")"}], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "4"]}]}], ")"}], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["J",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         RowBox[{
          FormBox[
           RowBox[{"{", 
            RowBox[{"2", ",", 
             InterpretationBox[
              SubscriptBox["m", "4"],
              $CellContext`m4]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "3"],
              $CellContext`m3]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "2"],
              $CellContext`m2]}], "}"}],
           TraditionalForm]}], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TJI[
        D, $CellContext`pp, {{2, $CellContext`m4}, {1, $CellContext`m3}, {
          1, $CellContext`m2}}],
        Editable->True]}]}]}]}], "}"}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 FormBox[
  SubsuperscriptBox["Y", "234", "111"], TraditionalForm]]]], "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", "\n", 
  RowBox[{"y111234", " ", "=", " ", 
   RowBox[{"TarcerRecurse", "[", " ", 
    RowBox[{"TFI", "[", 
     RowBox[{"D", ",", "pp", ",", 
      RowBox[{"{", 
       RowBox[{"3", ",", "0", ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", " ", 
        RowBox[{"{", 
         RowBox[{"1", ",", "m4"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"1", ",", "m2"}], "}"}], ",", "0", ",", 
        RowBox[{"{", 
         RowBox[{"1", ",", "m3"}], "}"}]}], "}"}]}], "]"}], " ", "]"}]}], 
  "\n", "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"9.748609`", ",", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       FractionBox["1", 
        RowBox[{"3", " ", 
         RowBox[{"(", 
          RowBox[{"D", "-", "2"}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"3", " ", "D"}], "-", "4"}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"3", " ", "D"}], "-", "2"}], ")"}]}]], 
       RowBox[{"4", " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"], "-", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"13", " ", 
           SuperscriptBox["D", "2"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             InterpretationBox[
              SubscriptBox["m", "2"],
              $CellContext`m2], ")"}], "4"]}], "-", 
          RowBox[{"28", " ", "D", " ", 
           SuperscriptBox[
            RowBox[{"(", 
             InterpretationBox[
              SubscriptBox["m", "2"],
              $CellContext`m2], ")"}], "4"]}], "+", 
          RowBox[{"12", " ", 
           SuperscriptBox[
            RowBox[{"(", 
             InterpretationBox[
              SubscriptBox["m", "2"],
              $CellContext`m2], ")"}], "4"]}], "-", 
          RowBox[{"4", " ", 
           SuperscriptBox["D", "2"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             InterpretationBox[
              SubscriptBox["m", "3"],
              $CellContext`m3], ")"}], "2"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             InterpretationBox[
              SubscriptBox["m", "2"],
              $CellContext`m2], ")"}], "2"]}], "+", 
          RowBox[{"12", " ", "D", " ", 
           SuperscriptBox[
            RowBox[{"(", 
             InterpretationBox[
              SubscriptBox["m", "3"],
              $CellContext`m3], ")"}], "2"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             InterpretationBox[
              SubscriptBox["m", "2"],
              $CellContext`m2], ")"}], "2"]}], "-", 
          RowBox[{"4", " ", 
           SuperscriptBox["D", "2"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             InterpretationBox[
              SubscriptBox["m", "4"],
              $CellContext`m4], ")"}], "2"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             InterpretationBox[
              SubscriptBox["m", "2"],
              $CellContext`m2], ")"}], "2"]}], "+", 
          RowBox[{"12", " ", "D", " ", 
           SuperscriptBox[
            RowBox[{"(", 
             InterpretationBox[
              SubscriptBox["m", "4"],
              $CellContext`m4], ")"}], "2"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             InterpretationBox[
              SubscriptBox["m", "2"],
              $CellContext`m2], ")"}], "2"]}], "+", 
          RowBox[{"38", " ", 
           SuperscriptBox["D", "2"], " ", 
           InterpretationBox[
            SuperscriptBox["p", "2"],
            $CellContext`pp], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             InterpretationBox[
              SubscriptBox["m", "2"],
              $CellContext`m2], ")"}], "2"]}], "-", 
          RowBox[{"72", " ", "D", " ", 
           InterpretationBox[
            SuperscriptBox["p", "2"],
            $CellContext`pp], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             InterpretationBox[
              SubscriptBox["m", "2"],
              $CellContext`m2], ")"}], "2"]}], "+", 
          RowBox[{"24", " ", 
           InterpretationBox[
            SuperscriptBox["p", "2"],
            $CellContext`pp], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             InterpretationBox[
              SubscriptBox["m", "2"],
              $CellContext`m2], ")"}], "2"]}], "-", 
          RowBox[{"2", " ", 
           SuperscriptBox["D", "2"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             InterpretationBox[
              SubscriptBox["m", "3"],
              $CellContext`m3], ")"}], "4"]}], "+", 
          RowBox[{"2", " ", "D", " ", 
           SuperscriptBox[
            RowBox[{"(", 
             InterpretationBox[
              SubscriptBox["m", "3"],
              $CellContext`m3], ")"}], "4"]}], "+", 
          RowBox[{"12", " ", 
           SuperscriptBox[
            RowBox[{"(", 
             InterpretationBox[
              SubscriptBox["m", "3"],
              $CellContext`m3], ")"}], "4"]}], "-", 
          RowBox[{"2", " ", 
           SuperscriptBox["D", "2"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             InterpretationBox[
              SubscriptBox["m", "4"],
              $CellContext`m4], ")"}], "4"]}], "+", 
          RowBox[{"2", " ", "D", " ", 
           SuperscriptBox[
            RowBox[{"(", 
             InterpretationBox[
              SubscriptBox["m", "4"],
              $CellContext`m4], ")"}], "4"]}], "+", 
          RowBox[{"12", " ", 
           SuperscriptBox[
            RowBox[{"(", 
             InterpretationBox[
              SubscriptBox["m", "4"],
              $CellContext`m4], ")"}], "4"]}], "-", 
          RowBox[{"28", " ", 
           SuperscriptBox["D", "2"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             InterpretationBox[
              SubscriptBox["m", "3"],
              $CellContext`m3], ")"}], "2"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             InterpretationBox[
              SubscriptBox["m", "4"],
              $CellContext`m4], ")"}], "2"]}], "+", 
          RowBox[{"108", " ", "D", " ", 
           SuperscriptBox[
            RowBox[{"(", 
             InterpretationBox[
              SubscriptBox["m", "3"],
              $CellContext`m3], ")"}], "2"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             InterpretationBox[
              SubscriptBox["m", "4"],
              $CellContext`m4], ")"}], "2"]}], "-", 
          RowBox[{"72", " ", 
           SuperscriptBox[
            RowBox[{"(", 
             InterpretationBox[
              SubscriptBox["m", "3"],
              $CellContext`m3], ")"}], "2"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             InterpretationBox[
              SubscriptBox["m", "4"],
              $CellContext`m4], ")"}], "2"]}], "+", 
          RowBox[{"13", " ", 
           SuperscriptBox["D", "2"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             InterpretationBox[
              SuperscriptBox["p", "2"],
              $CellContext`pp], ")"}], "2"]}], "-", 
          RowBox[{"28", " ", "D", " ", 
           SuperscriptBox[
            RowBox[{"(", 
             InterpretationBox[
              SuperscriptBox["p", "2"],
              $CellContext`pp], ")"}], "2"]}], "+", 
          RowBox[{"12", " ", 
           SuperscriptBox[
            RowBox[{"(", 
             InterpretationBox[
              SuperscriptBox["p", "2"],
              $CellContext`pp], ")"}], "2"]}], "-", 
          RowBox[{"4", " ", 
           SuperscriptBox["D", "2"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             InterpretationBox[
              SubscriptBox["m", "3"],
              $CellContext`m3], ")"}], "2"], " ", 
           InterpretationBox[
            SuperscriptBox["p", "2"],
            $CellContext`pp]}], "+", 
          RowBox[{"12", " ", "D", " ", 
           SuperscriptBox[
            RowBox[{"(", 
             InterpretationBox[
              SubscriptBox["m", "3"],
              $CellContext`m3], ")"}], "2"], " ", 
           InterpretationBox[
            SuperscriptBox["p", "2"],
            $CellContext`pp]}], "-", 
          RowBox[{"4", " ", 
           SuperscriptBox["D", "2"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             InterpretationBox[
              SubscriptBox["m", "4"],
              $CellContext`m4], ")"}], "2"], " ", 
           InterpretationBox[
            SuperscriptBox["p", "2"],
            $CellContext`pp]}], "+", 
          RowBox[{"12", " ", "D", " ", 
           SuperscriptBox[
            RowBox[{"(", 
             InterpretationBox[
              SubscriptBox["m", "4"],
              $CellContext`m4], ")"}], "2"], " ", 
           InterpretationBox[
            SuperscriptBox["p", "2"],
            $CellContext`pp]}]}], ")"}], " ", 
        InterpretationBox[
         SubsuperscriptBox[
          StyleBox["J",
           SingleLetterItalics->False,
           FontWeight->"Bold"], 
          RowBox[{
           FormBox[
            RowBox[{"{", 
             RowBox[{"2", ",", 
              InterpretationBox[
               SubscriptBox["m", "2"],
               $CellContext`m2]}], "}"}],
            TraditionalForm], 
           FormBox[
            RowBox[{"{", 
             RowBox[{"1", ",", 
              InterpretationBox[
               SubscriptBox["m", "4"],
               $CellContext`m4]}], "}"}],
            TraditionalForm], 
           FormBox[
            RowBox[{"{", 
             RowBox[{"1", ",", 
              InterpretationBox[
               SubscriptBox["m", "3"],
               $CellContext`m3]}], "}"}],
            TraditionalForm]}], 
          RowBox[{"(", 
           FormBox["D",
            TraditionalForm], ")"}]],
         HighEnergyPhysics`Tarcer`TJI[
         D, $CellContext`pp, {{2, $CellContext`m2}, {1, $CellContext`m4}, {
           1, $CellContext`m3}}],
         Editable->True], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          InterpretationBox[
           SubscriptBox["m", "2"],
           $CellContext`m2], ")"}], "2"]}]}]}], "+", 
     RowBox[{
      FractionBox["1", 
       RowBox[{"3", " ", "D", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"3", " ", "D"}], "-", "4"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"3", " ", "D"}], "-", "2"}], ")"}]}]], 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox["D", "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "4"]}], "+", 
         RowBox[{"2", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "4"]}], "+", 
         RowBox[{"5", " ", 
          SuperscriptBox["D", "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "-", 
         RowBox[{"18", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "+", 
         RowBox[{"24", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "+", 
         RowBox[{"11", " ", 
          SuperscriptBox["D", "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "-", 
         RowBox[{"18", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "-", 
         RowBox[{"22", " ", 
          SuperscriptBox["D", "3"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "+", 
         RowBox[{"144", " ", 
          SuperscriptBox["D", "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "-", 
         RowBox[{"216", " ", "D", " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "+", 
         RowBox[{"96", " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "+", 
         RowBox[{
          SuperscriptBox["D", "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "4"]}], "+", 
         RowBox[{"2", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "4"]}], "+", 
         RowBox[{"25", " ", 
          SuperscriptBox["D", "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "4"]}], "-", 
         RowBox[{"58", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "4"]}], "+", 
         RowBox[{"24", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "4"]}], "+", 
         RowBox[{"38", " ", 
          SuperscriptBox["D", "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"]}], "-", 
         RowBox[{"72", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"]}], "+", 
         RowBox[{"24", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"]}], "+", 
         RowBox[{
          SuperscriptBox["D", "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             $CellContext`pp], ")"}], "2"]}], "+", 
         RowBox[{"2", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             $CellContext`pp], ")"}], "2"]}], "+", 
         RowBox[{"5", " ", 
          SuperscriptBox["D", "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}], "-", 
         RowBox[{"18", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}], "+", 
         RowBox[{"24", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}], "+", 
         RowBox[{"11", " ", 
          SuperscriptBox["D", "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}], "-", 
         RowBox[{"18", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}]}], ")"}], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["A",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         FormBox[
          RowBox[{"{", 
           RowBox[{"1", ",", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2]}], "}"}],
          TraditionalForm], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TAI[D, 0, {{1, $CellContext`m2}}],
        Editable->True], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["A",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         FormBox[
          RowBox[{"{", 
           RowBox[{"1", ",", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3]}], "}"}],
          TraditionalForm], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TAI[D, 0, {{1, $CellContext`m3}}],
        Editable->True]}]}], "+", 
     RowBox[{
      FractionBox["1", 
       RowBox[{"3", " ", "D", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"3", " ", "D"}], "-", "4"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"3", " ", "D"}], "-", "2"}], ")"}]}]], 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox["D", "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "4"]}], "+", 
         RowBox[{"2", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "4"]}], "+", 
         RowBox[{"11", " ", 
          SuperscriptBox["D", "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "-", 
         RowBox[{"18", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "+", 
         RowBox[{"5", " ", 
          SuperscriptBox["D", "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "-", 
         RowBox[{"18", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "+", 
         RowBox[{"24", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "-", 
         RowBox[{"22", " ", 
          SuperscriptBox["D", "3"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "+", 
         RowBox[{"144", " ", 
          SuperscriptBox["D", "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "-", 
         RowBox[{"216", " ", "D", " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "+", 
         RowBox[{"96", " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "+", 
         RowBox[{"25", " ", 
          SuperscriptBox["D", "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "4"]}], "-", 
         RowBox[{"58", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "4"]}], "+", 
         RowBox[{"24", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "4"]}], "+", 
         RowBox[{
          SuperscriptBox["D", "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "4"]}], "+", 
         RowBox[{"2", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "4"]}], "+", 
         RowBox[{"38", " ", 
          SuperscriptBox["D", "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"]}], "-", 
         RowBox[{"72", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"]}], "+", 
         RowBox[{"24", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"]}], "+", 
         RowBox[{
          SuperscriptBox["D", "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             $CellContext`pp], ")"}], "2"]}], "+", 
         RowBox[{"2", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             $CellContext`pp], ")"}], "2"]}], "+", 
         RowBox[{"11", " ", 
          SuperscriptBox["D", "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}], "-", 
         RowBox[{"18", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}], "+", 
         RowBox[{"5", " ", 
          SuperscriptBox["D", "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}], "-", 
         RowBox[{"18", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}], "+", 
         RowBox[{"24", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}]}], ")"}], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["A",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         FormBox[
          RowBox[{"{", 
           RowBox[{"1", ",", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2]}], "}"}],
          TraditionalForm], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TAI[D, 0, {{1, $CellContext`m2}}],
        Editable->True], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["A",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         FormBox[
          RowBox[{"{", 
           RowBox[{"1", ",", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4]}], "}"}],
          TraditionalForm], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TAI[D, 0, {{1, $CellContext`m4}}],
        Editable->True]}]}], "+", 
     RowBox[{
      FractionBox["1", 
       RowBox[{"3", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"3", " ", "D"}], "-", "4"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"3", " ", "D"}], "-", "2"}], ")"}]}]], 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"25", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "4"]}], "-", 
         RowBox[{"58", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "4"]}], "+", 
         RowBox[{"24", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "4"]}], "-", 
         RowBox[{"16", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "+", 
         RowBox[{"36", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "-", 
         RowBox[{"24", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "-", 
         RowBox[{"16", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "+", 
         RowBox[{"36", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "-", 
         RowBox[{"24", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "+", 
         RowBox[{"38", " ", 
          SuperscriptBox["D", "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "-", 
         RowBox[{"72", " ", "D", " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "+", 
         RowBox[{"24", " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "-", 
         RowBox[{"26", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "4"]}], "+", 
         RowBox[{"56", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "4"]}], "-", 
         RowBox[{"24", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "4"]}], "-", 
         RowBox[{"26", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "4"]}], "+", 
         RowBox[{"56", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "4"]}], "-", 
         RowBox[{"24", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "4"]}], "-", 
         RowBox[{"76", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"]}], "+", 
         RowBox[{"144", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"]}], "-", 
         RowBox[{"48", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"]}], "+", 
         RowBox[{
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             $CellContext`pp], ")"}], "2"]}], "+", 
         RowBox[{"2", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             $CellContext`pp], ")"}], "2"]}], "+", 
         RowBox[{"32", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}], "-", 
         RowBox[{"72", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}], "+", 
         RowBox[{"48", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}], "+", 
         RowBox[{"32", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}], "-", 
         RowBox[{"72", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}], "+", 
         RowBox[{"48", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}]}], ")"}], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["A",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         FormBox[
          RowBox[{"{", 
           RowBox[{"1", ",", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3]}], "}"}],
          TraditionalForm], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TAI[D, 0, {{1, $CellContext`m3}}],
        Editable->True], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["A",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         FormBox[
          RowBox[{"{", 
           RowBox[{"1", ",", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4]}], "}"}],
          TraditionalForm], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TAI[D, 0, {{1, $CellContext`m4}}],
        Editable->True]}]}], "+", 
     RowBox[{
      FractionBox["1", 
       RowBox[{"D", "-", "1"}]], 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "4"]}], "-", 
         SuperscriptBox[
          RowBox[{"(", 
           InterpretationBox[
            SubscriptBox["m", "3"],
            $CellContext`m3], ")"}], "4"], "+", 
         RowBox[{"D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"]}], "-", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"]}], "+", 
         RowBox[{"2", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"]}], "+", 
         RowBox[{"2", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"]}], "-", 
         RowBox[{"D", " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"]}], "+", 
         RowBox[{
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"]}], "+", 
         RowBox[{"D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "4"]}], "-", 
         SuperscriptBox[
          RowBox[{"(", 
           InterpretationBox[
            SubscriptBox["m", "4"],
            $CellContext`m4], ")"}], "4"], "+", 
         RowBox[{"D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"]}], "-", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"]}], "-", 
         RowBox[{"D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}]}], ")"}], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["A",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         FormBox[
          RowBox[{"{", 
           RowBox[{"1", ",", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4]}], "}"}],
          TraditionalForm], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TAI[D, 0, {{1, $CellContext`m4}}],
        Editable->True], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["A",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         FormBox[
          RowBox[{"{", 
           RowBox[{"1", ",", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3]}], "}"}],
          TraditionalForm], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TAI[D, 0, {{1, $CellContext`m3}}],
        Editable->True]}]}], "-", 
     FractionBox[
      RowBox[{"4", " ", 
       SuperscriptBox[
        RowBox[{"(", 
         InterpretationBox[
          SubscriptBox["m", "3"],
          $CellContext`m3], ")"}], "2"], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         InterpretationBox[
          SubscriptBox["m", "4"],
          $CellContext`m4], ")"}], "2"], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["A",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         FormBox[
          RowBox[{"{", 
           RowBox[{"1", ",", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3]}], "}"}],
          TraditionalForm], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TAI[D, 0, {{1, $CellContext`m3}}],
        Editable->True], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["A",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         FormBox[
          RowBox[{"{", 
           RowBox[{"1", ",", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4]}], "}"}],
          TraditionalForm], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TAI[D, 0, {{1, $CellContext`m4}}],
        Editable->True]}], 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"D", "-", "1"}], ")"}], " ", "D"}]], "+", 
     RowBox[{
      FractionBox["1", 
       RowBox[{"3", " ", 
        RowBox[{"(", 
         RowBox[{"D", "-", "2"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"3", " ", "D"}], "-", "4"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"3", " ", "D"}], "-", "2"}], ")"}]}]], 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"79", " ", 
          SuperscriptBox["D", "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "6"]}], "-", 
         RowBox[{"376", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "6"]}], "+", 
         RowBox[{"516", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "6"]}], "-", 
         RowBox[{"192", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "6"]}], "-", 
         RowBox[{"18", " ", 
          SuperscriptBox["D", "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "4"]}], "+", 
         RowBox[{"98", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "4"]}], "-", 
         RowBox[{"132", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "4"]}], "-", 
         RowBox[{"18", " ", 
          SuperscriptBox["D", "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "4"]}], "+", 
         RowBox[{"98", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "4"]}], "-", 
         RowBox[{"132", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "4"]}], "+", 
         RowBox[{"291", " ", 
          SuperscriptBox["D", "3"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "4"]}], "-", 
         RowBox[{"1296", " ", 
          SuperscriptBox["D", "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "4"]}], "+", 
         RowBox[{"1604", " ", "D", " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "4"]}], "-", 
         RowBox[{"480", " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "4"]}], "-", 
         RowBox[{"18", " ", 
          SuperscriptBox["D", "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "4"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "+", 
         RowBox[{"98", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "4"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "-", 
         RowBox[{"132", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "4"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "-", 
         RowBox[{"18", " ", 
          SuperscriptBox["D", "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "4"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "+", 
         RowBox[{"98", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "4"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "-", 
         RowBox[{"132", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "4"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "-", 
         RowBox[{"156", " ", 
          SuperscriptBox["D", "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "+", 
         RowBox[{"972", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "-", 
         RowBox[{"1800", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "+", 
         RowBox[{"864", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "+", 
         RowBox[{"141", " ", 
          SuperscriptBox["D", "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             $CellContext`pp], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "-", 
         RowBox[{"632", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             $CellContext`pp], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "+", 
         RowBox[{"828", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             $CellContext`pp], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "-", 
         RowBox[{"288", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             $CellContext`pp], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "+", 
         RowBox[{"12", " ", 
          SuperscriptBox["D", "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "-", 
         RowBox[{"124", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "+", 
         RowBox[{"360", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "-", 
         RowBox[{"288", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "+", 
         RowBox[{"12", " ", 
          SuperscriptBox["D", "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "-", 
         RowBox[{"124", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "+", 
         RowBox[{"360", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "-", 
         RowBox[{"288", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "-", 
         RowBox[{"2", " ", 
          SuperscriptBox["D", "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "6"]}], "+", 
         RowBox[{"2", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "6"]}], "+", 
         RowBox[{"12", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "6"]}], "-", 
         RowBox[{"2", " ", 
          SuperscriptBox["D", "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "6"]}], "+", 
         RowBox[{"2", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "6"]}], "+", 
         RowBox[{"12", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "6"]}], "-", 
         RowBox[{"126", " ", 
          SuperscriptBox["D", "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "4"]}], "+", 
         RowBox[{"638", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "4"]}], "-", 
         RowBox[{"876", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "4"]}], "+", 
         RowBox[{"288", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "4"]}], "+", 
         RowBox[{
          SuperscriptBox["D", "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             $CellContext`pp], ")"}], "3"]}], "-", 
         RowBox[{"4", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             $CellContext`pp], ")"}], "3"]}], "-", 
         RowBox[{"126", " ", 
          SuperscriptBox["D", "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "4"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"]}], "+", 
         RowBox[{"638", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "4"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"]}], "-", 
         RowBox[{"876", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "4"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"]}], "+", 
         RowBox[{"288", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "4"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"]}], "+", 
         RowBox[{"6", " ", 
          SuperscriptBox["D", "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             $CellContext`pp], ")"}], "2"]}], "-", 
         RowBox[{"38", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             $CellContext`pp], ")"}], "2"]}], "+", 
         RowBox[{"60", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             $CellContext`pp], ")"}], "2"]}], "+", 
         RowBox[{"6", " ", 
          SuperscriptBox["D", "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             $CellContext`pp], ")"}], "2"]}], "-", 
         RowBox[{"38", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             $CellContext`pp], ")"}], "2"]}], "+", 
         RowBox[{"60", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             $CellContext`pp], ")"}], "2"]}], "-", 
         RowBox[{"6", " ", 
          SuperscriptBox["D", "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "4"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}], "+", 
         RowBox[{"54", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "4"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}], "-", 
         RowBox[{"172", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "4"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}], "+", 
         RowBox[{"192", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "4"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}], "-", 
         RowBox[{"6", " ", 
          SuperscriptBox["D", "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "4"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}], "+", 
         RowBox[{"54", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "4"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}], "-", 
         RowBox[{"172", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "4"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}], "+", 
         RowBox[{"192", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "4"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}], "+", 
         RowBox[{"12", " ", 
          SuperscriptBox["D", "3"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}], "-", 
         RowBox[{"124", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}], "+", 
         RowBox[{"360", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}], "-", 
         RowBox[{"288", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}]}], ")"}], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["J",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         RowBox[{
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "4"],
              $CellContext`m4]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "3"],
              $CellContext`m3]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "2"],
              $CellContext`m2]}], "}"}],
           TraditionalForm]}], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TJI[
        D, $CellContext`pp, {{1, $CellContext`m4}, {1, $CellContext`m3}, {
          1, $CellContext`m2}}],
        Editable->True]}]}], "-", 
     RowBox[{
      FractionBox["1", 
       RowBox[{"3", " ", 
        RowBox[{"(", 
         RowBox[{"D", "-", "2"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"3", " ", "D"}], "-", "4"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"3", " ", "D"}], "-", "2"}], ")"}]}]], 
      RowBox[{"2", " ", 
       SuperscriptBox[
        RowBox[{"(", 
         InterpretationBox[
          SubscriptBox["m", "3"],
          $CellContext`m3], ")"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"27", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "6"]}], "-", 
         RowBox[{"54", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "6"]}], "+", 
         RowBox[{"24", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "6"]}], "-", 
         RowBox[{"4", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "4"]}], "+", 
         RowBox[{"4", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "4"]}], "+", 
         RowBox[{"24", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "4"]}], "+", 
         RowBox[{"139", " ", 
          SuperscriptBox["D", "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "4"]}], "-", 
         RowBox[{"274", " ", "D", " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "4"]}], "+", 
         RowBox[{"96", " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "4"]}], "-", 
         RowBox[{"8", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "4"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "+", 
         RowBox[{"24", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "4"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "-", 
         RowBox[{"56", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "+", 
         RowBox[{"216", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "-", 
         RowBox[{"144", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "+", 
         RowBox[{"89", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             $CellContext`pp], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "-", 
         RowBox[{"186", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             $CellContext`pp], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "+", 
         RowBox[{"72", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             $CellContext`pp], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "+", 
         RowBox[{"56", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "-", 
         RowBox[{"216", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "+", 
         RowBox[{"144", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "-", 
         RowBox[{
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "6"]}], "-", 
         RowBox[{"2", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "6"]}], "-", 
         RowBox[{"27", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "6"]}], "+", 
         RowBox[{"54", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "6"]}], "-", 
         RowBox[{"24", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "6"]}], "-", 
         RowBox[{"139", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "4"]}], "+", 
         RowBox[{"274", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "4"]}], "-", 
         RowBox[{"96", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "4"]}], "+", 
         RowBox[{
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             $CellContext`pp], ")"}], "3"]}], "+", 
         RowBox[{"2", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             $CellContext`pp], ")"}], "3"]}], "-", 
         RowBox[{"89", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "4"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"]}], "+", 
         RowBox[{"186", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "4"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"]}], "-", 
         RowBox[{"72", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "4"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"]}], "+", 
         RowBox[{"4", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             $CellContext`pp], ")"}], "2"]}], "-", 
         RowBox[{"20", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             $CellContext`pp], ")"}], "2"]}], "+", 
         RowBox[{"24", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             $CellContext`pp], ")"}], "2"]}], "+", 
         RowBox[{"8", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             $CellContext`pp], ")"}], "2"]}], "-", 
         RowBox[{"24", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             $CellContext`pp], ")"}], "2"]}], "-", 
         RowBox[{"4", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "4"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}], "+", 
         RowBox[{"20", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "4"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}], "-", 
         RowBox[{"24", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "4"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}], "+", 
         RowBox[{"4", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "4"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}], "-", 
         RowBox[{"4", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "4"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}], "-", 
         RowBox[{"24", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "4"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}]}], ")"}], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["J",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         RowBox[{
          FormBox[
           RowBox[{"{", 
            RowBox[{"2", ",", 
             InterpretationBox[
              SubscriptBox["m", "3"],
              $CellContext`m3]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "4"],
              $CellContext`m4]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "2"],
              $CellContext`m2]}], "}"}],
           TraditionalForm]}], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TJI[
        D, $CellContext`pp, {{2, $CellContext`m3}, {1, $CellContext`m4}, {
          1, $CellContext`m2}}],
        Editable->True]}]}], "-", 
     RowBox[{
      FractionBox["1", 
       RowBox[{"3", " ", 
        RowBox[{"(", 
         RowBox[{"D", "-", "2"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"3", " ", "D"}], "-", "4"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"3", " ", "D"}], "-", "2"}], ")"}]}]], 
      RowBox[{"2", " ", 
       SuperscriptBox[
        RowBox[{"(", 
         InterpretationBox[
          SubscriptBox["m", "4"],
          $CellContext`m4], ")"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"27", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "6"]}], "-", 
         RowBox[{"54", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "6"]}], "+", 
         RowBox[{"24", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "6"]}], "-", 
         RowBox[{"4", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "4"]}], "+", 
         RowBox[{"4", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "4"]}], "+", 
         RowBox[{"24", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "4"]}], "+", 
         RowBox[{"139", " ", 
          SuperscriptBox["D", "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "4"]}], "-", 
         RowBox[{"274", " ", "D", " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "4"]}], "+", 
         RowBox[{"96", " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "4"]}], "-", 
         RowBox[{"8", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "4"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "+", 
         RowBox[{"24", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "4"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "-", 
         RowBox[{"56", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "+", 
         RowBox[{"216", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "-", 
         RowBox[{"144", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "+", 
         RowBox[{"89", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             $CellContext`pp], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "-", 
         RowBox[{"186", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             $CellContext`pp], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "+", 
         RowBox[{"72", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             $CellContext`pp], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "+", 
         RowBox[{"56", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "-", 
         RowBox[{"216", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "+", 
         RowBox[{"144", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "-", 
         RowBox[{"27", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "6"]}], "+", 
         RowBox[{"54", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "6"]}], "-", 
         RowBox[{"24", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "6"]}], "-", 
         RowBox[{
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "6"]}], "-", 
         RowBox[{"2", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "6"]}], "-", 
         RowBox[{"89", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "4"]}], "+", 
         RowBox[{"186", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "4"]}], "-", 
         RowBox[{"72", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "4"]}], "+", 
         RowBox[{
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             $CellContext`pp], ")"}], "3"]}], "+", 
         RowBox[{"2", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             $CellContext`pp], ")"}], "3"]}], "-", 
         RowBox[{"139", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "4"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"]}], "+", 
         RowBox[{"274", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "4"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"]}], "-", 
         RowBox[{"96", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "4"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"]}], "+", 
         RowBox[{"8", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             $CellContext`pp], ")"}], "2"]}], "-", 
         RowBox[{"24", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             $CellContext`pp], ")"}], "2"]}], "+", 
         RowBox[{"4", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             $CellContext`pp], ")"}], "2"]}], "-", 
         RowBox[{"20", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             $CellContext`pp], ")"}], "2"]}], "+", 
         RowBox[{"24", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SuperscriptBox["p", "2"],
             $CellContext`pp], ")"}], "2"]}], "+", 
         RowBox[{"4", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "4"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}], "-", 
         RowBox[{"4", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "4"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}], "-", 
         RowBox[{"24", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "4"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}], "-", 
         RowBox[{"4", " ", 
          SuperscriptBox["D", "2"], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "4"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}], "+", 
         RowBox[{"20", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "4"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}], "-", 
         RowBox[{"24", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "4"], " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}]}], ")"}], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["J",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         RowBox[{
          FormBox[
           RowBox[{"{", 
            RowBox[{"2", ",", 
             InterpretationBox[
              SubscriptBox["m", "4"],
              $CellContext`m4]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "3"],
              $CellContext`m3]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "2"],
              $CellContext`m2]}], "}"}],
           TraditionalForm]}], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TJI[
        D, $CellContext`pp, {{2, $CellContext`m4}, {1, $CellContext`m3}, {
          1, $CellContext`m2}}],
        Editable->True]}]}]}]}], "}"}], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 FormBox[
  SubsuperscriptBox["Y", "234", "5"], TraditionalForm]]]], "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", "\n", 
  RowBox[{"y5234", " ", "=", " ", 
   RowBox[{"TarcerRecurse", "[", " ", 
    RowBox[{"TFI", "[", 
     RowBox[{"D", ",", "pp", ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "0", ",", "1", ",", "0", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", " ", 
        RowBox[{"{", 
         RowBox[{"1", ",", "m4"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"1", ",", "m2"}], "}"}], ",", "0", ",", 
        RowBox[{"{", 
         RowBox[{"1", ",", "m3"}], "}"}]}], "}"}]}], "]"}], " ", "]"}]}], 
  "\n", "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"0.03600199999999987`", ",", 
    RowBox[{
     RowBox[{
      FractionBox["1", "6"], " ", 
      InterpretationBox[
       SubsuperscriptBox[
        StyleBox["A",
         SingleLetterItalics->False,
         FontWeight->"Bold"], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", 
           InterpretationBox[
            SubscriptBox["m", "2"],
            $CellContext`m2]}], "}"}],
         TraditionalForm], 
        RowBox[{"(", 
         FormBox["D",
          TraditionalForm], ")"}]],
       HighEnergyPhysics`Tarcer`TAI[D, 0, {{1, $CellContext`m2}}],
       Editable->True], " ", 
      InterpretationBox[
       SubsuperscriptBox[
        StyleBox["A",
         SingleLetterItalics->False,
         FontWeight->"Bold"], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", 
           InterpretationBox[
            SubscriptBox["m", "3"],
            $CellContext`m3]}], "}"}],
         TraditionalForm], 
        RowBox[{"(", 
         FormBox["D",
          TraditionalForm], ")"}]],
       HighEnergyPhysics`Tarcer`TAI[D, 0, {{1, $CellContext`m3}}],
       Editable->True]}], "+", 
     RowBox[{
      FractionBox["1", "6"], " ", 
      InterpretationBox[
       SubsuperscriptBox[
        StyleBox["A",
         SingleLetterItalics->False,
         FontWeight->"Bold"], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", 
           InterpretationBox[
            SubscriptBox["m", "2"],
            $CellContext`m2]}], "}"}],
         TraditionalForm], 
        RowBox[{"(", 
         FormBox["D",
          TraditionalForm], ")"}]],
       HighEnergyPhysics`Tarcer`TAI[D, 0, {{1, $CellContext`m2}}],
       Editable->True], " ", 
      InterpretationBox[
       SubsuperscriptBox[
        StyleBox["A",
         SingleLetterItalics->False,
         FontWeight->"Bold"], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", 
           InterpretationBox[
            SubscriptBox["m", "4"],
            $CellContext`m4]}], "}"}],
         TraditionalForm], 
        RowBox[{"(", 
         FormBox["D",
          TraditionalForm], ")"}]],
       HighEnergyPhysics`Tarcer`TAI[D, 0, {{1, $CellContext`m4}}],
       Editable->True]}], "-", 
     RowBox[{
      FractionBox["1", "3"], " ", 
      InterpretationBox[
       SubsuperscriptBox[
        StyleBox["A",
         SingleLetterItalics->False,
         FontWeight->"Bold"], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", 
           InterpretationBox[
            SubscriptBox["m", "3"],
            $CellContext`m3]}], "}"}],
         TraditionalForm], 
        RowBox[{"(", 
         FormBox["D",
          TraditionalForm], ")"}]],
       HighEnergyPhysics`Tarcer`TAI[D, 0, {{1, $CellContext`m3}}],
       Editable->True], " ", 
      InterpretationBox[
       SubsuperscriptBox[
        StyleBox["A",
         SingleLetterItalics->False,
         FontWeight->"Bold"], 
        FormBox[
         RowBox[{"{", 
          RowBox[{"1", ",", 
           InterpretationBox[
            SubscriptBox["m", "4"],
            $CellContext`m4]}], "}"}],
         TraditionalForm], 
        RowBox[{"(", 
         FormBox["D",
          TraditionalForm], ")"}]],
       HighEnergyPhysics`Tarcer`TAI[D, 0, {{1, $CellContext`m4}}],
       Editable->True]}], "-", 
     FractionBox[
      RowBox[{"2", " ", 
       SuperscriptBox[
        RowBox[{"(", 
         InterpretationBox[
          SubscriptBox["m", "2"],
          $CellContext`m2], ")"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           InterpretationBox[
            SubscriptBox["m", "2"],
            $CellContext`m2], ")"}], "2"], "-", 
         InterpretationBox[
          SuperscriptBox["p", "2"],
          $CellContext`pp]}], ")"}], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["J",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         RowBox[{
          FormBox[
           RowBox[{"{", 
            RowBox[{"2", ",", 
             InterpretationBox[
              SubscriptBox["m", "2"],
              $CellContext`m2]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "4"],
              $CellContext`m4]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "3"],
              $CellContext`m3]}], "}"}],
           TraditionalForm]}], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TJI[
        D, $CellContext`pp, {{2, $CellContext`m2}, {1, $CellContext`m4}, {
          1, $CellContext`m3}}],
        Editable->True]}], 
      RowBox[{"3", " ", 
       RowBox[{"(", 
        RowBox[{"D", "-", "2"}], ")"}]}]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"2", " ", "D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "-", 
         RowBox[{"D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"]}], "-", 
         RowBox[{"D", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"]}], "+", 
         RowBox[{"2", " ", "D", " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}], "-", 
         RowBox[{"6", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "2"],
             $CellContext`m2], ")"}], "2"]}], "+", 
         RowBox[{"3", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"]}], "+", 
         RowBox[{"3", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"]}], "-", 
         RowBox[{"4", " ", 
          InterpretationBox[
           SuperscriptBox["p", "2"],
           $CellContext`pp]}]}], ")"}], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["J",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         RowBox[{
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "4"],
              $CellContext`m4]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "3"],
              $CellContext`m3]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "2"],
              $CellContext`m2]}], "}"}],
           TraditionalForm]}], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TJI[
        D, $CellContext`pp, {{1, $CellContext`m4}, {1, $CellContext`m3}, {
          1, $CellContext`m2}}],
        Editable->True]}], 
      RowBox[{"3", " ", 
       RowBox[{"(", 
        RowBox[{"D", "-", "2"}], ")"}]}]], "+", 
     FractionBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         InterpretationBox[
          SubscriptBox["m", "3"],
          $CellContext`m3], ")"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"3", " ", 
           SuperscriptBox[
            RowBox[{"(", 
             InterpretationBox[
              SubscriptBox["m", "2"],
              $CellContext`m2], ")"}], "2"]}]}], "+", 
         SuperscriptBox[
          RowBox[{"(", 
           InterpretationBox[
            SubscriptBox["m", "3"],
            $CellContext`m3], ")"}], "2"], "+", 
         RowBox[{"3", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "4"],
             $CellContext`m4], ")"}], "2"]}], "-", 
         InterpretationBox[
          SuperscriptBox["p", "2"],
          $CellContext`pp]}], ")"}], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["J",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         RowBox[{
          FormBox[
           RowBox[{"{", 
            RowBox[{"2", ",", 
             InterpretationBox[
              SubscriptBox["m", "3"],
              $CellContext`m3]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "4"],
              $CellContext`m4]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "2"],
              $CellContext`m2]}], "}"}],
           TraditionalForm]}], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TJI[
        D, $CellContext`pp, {{2, $CellContext`m3}, {1, $CellContext`m4}, {
          1, $CellContext`m2}}],
        Editable->True]}], 
      RowBox[{"3", " ", 
       RowBox[{"(", 
        RowBox[{"D", "-", "2"}], ")"}]}]], "+", 
     FractionBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         InterpretationBox[
          SubscriptBox["m", "4"],
          $CellContext`m4], ")"}], "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"3", " ", 
           SuperscriptBox[
            RowBox[{"(", 
             InterpretationBox[
              SubscriptBox["m", "2"],
              $CellContext`m2], ")"}], "2"]}]}], "+", 
         RowBox[{"3", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "3"],
             $CellContext`m3], ")"}], "2"]}], "+", 
         SuperscriptBox[
          RowBox[{"(", 
           InterpretationBox[
            SubscriptBox["m", "4"],
            $CellContext`m4], ")"}], "2"], "-", 
         InterpretationBox[
          SuperscriptBox["p", "2"],
          $CellContext`pp]}], ")"}], " ", 
       InterpretationBox[
        SubsuperscriptBox[
         StyleBox["J",
          SingleLetterItalics->False,
          FontWeight->"Bold"], 
         RowBox[{
          FormBox[
           RowBox[{"{", 
            RowBox[{"2", ",", 
             InterpretationBox[
              SubscriptBox["m", "4"],
              $CellContext`m4]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "3"],
              $CellContext`m3]}], "}"}],
           TraditionalForm], 
          FormBox[
           RowBox[{"{", 
            RowBox[{"1", ",", 
             InterpretationBox[
              SubscriptBox["m", "2"],
              $CellContext`m2]}], "}"}],
           TraditionalForm]}], 
         RowBox[{"(", 
          FormBox["D",
           TraditionalForm], ")"}]],
        HighEnergyPhysics`Tarcer`TJI[
        D, $CellContext`pp, {{2, $CellContext`m4}, {1, $CellContext`m3}, {
          1, $CellContext`m2}}],
        Editable->True]}], 
      RowBox[{"3", " ", 
       RowBox[{"(", 
        RowBox[{"D", "-", "2"}], ")"}]}]]}]}], "}"}], 
  TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
WindowSize->{1253, 681},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
Magnification->1.5,
FrontEndVersion->"8.0 for Linux x86 (64-bit) (October 10, 2011)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "T1.11.1"->{
  Cell[579, 22, 56, 1, 110, "Section",
   CellTags->"T1.11.1"]},
 "T1.11.1.1"->{
  Cell[2624, 98, 61, 1, 57, "Subsection",
   CellTags->"T1.11.1.1"]},
 "T1.11.2.1"->{
  Cell[9523, 355, 61, 1, 57, "Subsection",
   CellTags->"T1.11.2.1"],
  Cell[19646, 711, 70, 1, 57, "Subsection",
   CellTags->"T1.11.2.1"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"T1.11.1", 203640, 6345},
 {"T1.11.1.1", 203721, 6348},
 {"T1.11.2.1", 203807, 6351}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 56, 1, 110, "Section",
 CellTags->"T1.11.1"],
Cell[638, 25, 146, 3, 43, "Input"],
Cell[787, 30, 71, 1, 43, "Input"],
Cell[861, 33, 163, 3, 22, "Text"],
Cell[1027, 38, 196, 5, 22, "Text"],
Cell[1226, 45, 641, 21, 24, "Text"],
Cell[1870, 68, 329, 11, 22, "Text"],
Cell[2202, 81, 154, 3, 22, "Text"],
Cell[2359, 86, 139, 3, 42, "Text"],
Cell[2501, 91, 98, 3, 43, "Input"],
Cell[CellGroupData[{
Cell[2624, 98, 61, 1, 57, "Subsection",
 CellTags->"T1.11.1.1"],
Cell[2688, 101, 2072, 76, 133, "Text"],
Cell[CellGroupData[{
Cell[4785, 181, 116, 5, 111, "Input"],
Cell[4904, 188, 3300, 111, 92, "Output"]
}, Open  ]],
Cell[8219, 302, 53, 0, 42, "Text"],
Cell[CellGroupData[{
Cell[8297, 306, 169, 4, 43, "Input"],
Cell[8469, 312, 783, 26, 79, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[9289, 343, 58, 1, 43, "Input"],
Cell[9350, 346, 124, 3, 85, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[9523, 355, 61, 1, 57, "Subsection",
 CellTags->"T1.11.2.1"],
Cell[9587, 358, 112, 4, 88, "Input"],
Cell[CellGroupData[{
Cell[9724, 366, 96, 2, 43, "Input"],
Cell[9823, 370, 3497, 116, 92, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[13357, 491, 162, 4, 43, "Input"],
Cell[13522, 497, 827, 27, 79, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[14386, 529, 96, 2, 43, "Input"],
Cell[14485, 533, 3839, 127, 175, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[18361, 665, 162, 4, 43, "Input"],
Cell[18526, 671, 1071, 34, 79, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[19646, 711, 70, 1, 57, "Subsection",
 CellTags->"T1.11.2.1"],
Cell[19719, 714, 34, 0, 42, "Text"],
Cell[19756, 716, 1096, 36, 125, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[20889, 757, 110, 2, 57, "Subsection"],
Cell[CellGroupData[{
Cell[21024, 763, 596, 16, 98, "Input"],
Cell[21623, 781, 12011, 386, 245, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[33683, 1173, 111, 2, 57, "Subsection"],
Cell[CellGroupData[{
Cell[33819, 1179, 657, 18, 98, "Input"],
Cell[34479, 1199, 19481, 628, 335, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[54009, 1833, 111, 2, 57, "Subsection"],
Cell[CellGroupData[{
Cell[54145, 1839, 1198, 33, 152, "Input"],
Cell[55346, 1874, 25338, 779, 555, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[80733, 2659, 112, 2, 57, "Subsection"],
Cell[CellGroupData[{
Cell[80870, 2665, 598, 16, 98, "Input"],
Cell[81471, 2683, 109063, 3230, 1792, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[190583, 5919, 110, 2, 57, "Subsection"],
Cell[CellGroupData[{
Cell[190718, 5925, 596, 16, 98, "Input"],
Cell[191317, 5943, 11979, 386, 245, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
