(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     99687,       3201]
NotebookOptionsPosition[     94581,       3027]
NotebookOutlinePosition[     95701,       3064]
CellTagsIndexPosition[     95531,       3057]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell[TextData[{
 "Doing a difficult with ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " and FeynCalc tools"
}], "Section"],

Cell[CellGroupData[{

Cell[BoxData["$Version"], "Input",
 CellChangeTimes->{{3.501077397865684*^9, 3.501077398597706*^9}}],

Cell[BoxData[
 FormBox["\<\"8.0 for Linux x86 (64-bit) (November 7, 2010)\"\>", 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5010774013690777`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"<<", "HighEnergyPhysics`FeynCalc`"}]], "Input"],

Cell["Loading FeynCalc from /home/rolfm/HighEnergyPhysics", "Text",
 CellMargins->{{Inherited, Inherited}, {1, 0}},
 GeneratedCell->True,
 CellAutoOverwrite->True,
 CellChangeTimes->{3.5010774016875677`*^9}],

Cell[TextData[{
 StyleBox["FeynCalc",
  FontWeight->"Bold"],
 " ",
 "8.0.0.beta1",
 " For help, type ?FeynCalc, use the built-in ",
 ButtonBox["help system",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>(FrontEndExecute[
    FrontEnd`HelpBrowserLookup["AddOns", #]]& ),
  ButtonData:>{"Short Overview", "intro"},
  ButtonNote->"Open the help browser"],
 " or visit ",
 ButtonBox["www.feyncalc.org",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.feyncalc.org/"], None},
  ButtonNote->"http://www.feyncalc.org/"]
}], "Text",
 CellMargins->{{Inherited, Inherited}, {1, 0}},
 GeneratedCell->True,
 CellAutoOverwrite->True,
 CellChangeTimes->{3.501077402844861*^9}],

Cell[TextData[{
 "Loading FeynArts, see ",
 ButtonBox["www.feynarts.de",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.feynarts.de/"], None},
  ButtonNote->"http://www.feynarts.de/"],
 " for documentation"
}], "Text",
 CellMargins->{{Inherited, Inherited}, {1, 0}},
 GeneratedCell->True,
 CellAutoOverwrite->True,
 CellChangeTimes->{3.5010774048812923`*^9}],

Cell["FeynArts 3.4 patched for use with FeynCalc", "Text",
 CellMargins->{{Inherited, Inherited}, {1, 0}},
 GeneratedCell->True,
 CellAutoOverwrite->True,
 CellChangeTimes->{3.501077405104105*^9}],

Cell["\<\
Performing multidimensional Feynman paramater integrals is sometimes a \
nontrivial task. This is augmented by the absence of algorithms for certain \
integrals with end point singularities.
The following integral shows up in research related to massless 4-partice \
phase space and was solved together with Prof. T. Gehrmann from the \
University of Z\[UDoubleDot]rich.
Integration from 0 to 1 is assumed for the parameters u,z and t.
What one is after is the expansion up to order Epsilon.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"integral", "=", 
  RowBox[{"HoldForm", "[", 
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", "1"], 
    RowBox[{
     SubsuperscriptBox["\[Integral]", "0", "1"], 
     RowBox[{
      SubsuperscriptBox["\[Integral]", "0", "1"], 
      RowBox[{
       FractionBox[
        RowBox[{
         SuperscriptBox["z", 
          RowBox[{
           RowBox[{"-", "1"}], "-", "Epsilon"}]], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"1", "-", "z"}], ")"}], 
          RowBox[{
           RowBox[{"-", "2"}], " ", "Epsilon"}]], " ", 
         SuperscriptBox["u", 
          RowBox[{
           RowBox[{"-", "Epsilon"}], "-", "1"}]], " ", 
         SuperscriptBox["t", 
          RowBox[{
           RowBox[{"-", "1"}], "-", "Epsilon"}]], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"1", "-", "t"}], ")"}], 
          RowBox[{
           RowBox[{"-", "1"}], "-", "Epsilon"}]]}], 
        RowBox[{"1", "+", 
         FractionBox[
          RowBox[{"u", " ", "z", " ", "t"}], 
          RowBox[{"1", "-", "t"}]]}]], 
       RowBox[{"\[DifferentialD]", "z"}], 
       RowBox[{"\[DifferentialD]", "u"}], 
       RowBox[{"\[DifferentialD]", "t"}]}]}]}]}], "]"}]}]], "Input"],

Cell[BoxData[
 FormBox[
  TagBox[
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", "1"], 
    RowBox[{
     SubsuperscriptBox["\[Integral]", "0", "1"], 
     RowBox[{
      SubsuperscriptBox["\[Integral]", "0", "1"], 
      RowBox[{
       FractionBox[
        RowBox[{
         SuperscriptBox["z", 
          RowBox[{
           RowBox[{"-", "1"}], "-", 
           TagBox["\[CurlyEpsilon]",
            TraditionalForm]}]], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"1", "-", "z"}], ")"}], 
          RowBox[{
           RowBox[{"-", "2"}], " ", 
           TagBox["\[CurlyEpsilon]",
            TraditionalForm]}]], " ", 
         SuperscriptBox["u", 
          RowBox[{
           RowBox[{"-", 
            TagBox["\[CurlyEpsilon]",
             TraditionalForm]}], "-", "1"}]], " ", 
         SuperscriptBox["t", 
          RowBox[{
           RowBox[{"-", "1"}], "-", 
           TagBox["\[CurlyEpsilon]",
            TraditionalForm]}]], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"1", "-", "t"}], ")"}], 
          RowBox[{
           RowBox[{"-", "1"}], "-", 
           TagBox["\[CurlyEpsilon]",
            TraditionalForm]}]]}], 
        RowBox[{"1", "+", 
         FractionBox[
          RowBox[{"u", " ", "z", " ", "t"}], 
          RowBox[{"1", "-", "t"}]]}]], 
       RowBox[{"\[DifferentialD]", "z"}], 
       RowBox[{"\[DifferentialD]", "u"}], 
       RowBox[{"\[DifferentialD]", "t"}]}]}]}]}],
   HoldForm], TraditionalForm]], "Output",
 CellChangeTimes->{3.4729012563772497`*^9, 3.501051687534317*^9, 
  3.5010524841433887`*^9, 3.501073436972423*^9, 3.501077405684579*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"integrand", "=", 
  RowBox[{"integral", "[", 
   RowBox[{"[", 
    RowBox[{"1", ",", "1"}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "t"}], ")"}], 
     RowBox[{
      RowBox[{"-", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}], "-", "1"}]], " ", 
    SuperscriptBox["t", 
     RowBox[{
      RowBox[{"-", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}], "-", "1"}]], " ", 
    SuperscriptBox["u", 
     RowBox[{
      RowBox[{"-", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}], "-", "1"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", "2"}], " ", 
      TagBox["\[CurlyEpsilon]",
       TraditionalForm]}]], " ", 
    SuperscriptBox["z", 
     RowBox[{
      RowBox[{"-", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}], "-", "1"}]]}], 
   RowBox[{
    FractionBox[
     RowBox[{"t", " ", "u", " ", "z"}], 
     RowBox[{"1", "-", "t"}]], "+", "1"}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.472901259565872*^9, 3.5010516879258633`*^9, 
  3.501052484503002*^9, 3.501073437024909*^9, 3.501077405743972*^9}]
}, Open  ]],

Cell[TextData[{
 "We set various assumptions which will help ",
 StyleBox["Mathematica's ",
  FontSlant->"Italic"],
 "Integrate",
 StyleBox[" ",
  FontSlant->"Italic"],
 "function",
 StyleBox[".",
  FontSlant->"Italic"]
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"Integrate", ",", 
    RowBox[{"Assumptions", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"Re", "[", "Epsilon", "]"}], "<", "0"}], "&&", " ", 
       RowBox[{"0", "\[LessEqual]", "u", "\[LessEqual]", "1"}], "&&", " ", 
       RowBox[{"0", "\[LessEqual]", "z", "\[LessEqual]", "1"}], "&&", 
       RowBox[{
        RowBox[{"Im", "[", "u", "]"}], "\[Equal]", "0"}], "&&", " ", 
       RowBox[{
        RowBox[{
         RowBox[{"Re", "[", 
          RowBox[{"1", "/", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"u", " ", "z"}], "-", "1"}], ")"}]}], "]"}], "+", "1"}], 
        "<", "0"}]}], "}"}]}]}], "]"}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"tint", "=", 
  RowBox[{
   SubsuperscriptBox["\[Integral]", "0", "1"], 
   RowBox[{"integrand", 
    RowBox[{"\[DifferentialD]", "t"}]}]}]}]], "Input"],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"\[Pi]", " ", 
    SuperscriptBox["16", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]], " ", 
    TemplateBox[{RowBox[{
        RowBox[{"-", "2"}], " ", 
        TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
     "Gamma"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", "2"}], " ", 
      TagBox["\[CurlyEpsilon]",
       TraditionalForm]}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"u", " ", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}], "-", "1"}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"2", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}]}], "+", 
      RowBox[{
       TagBox["\[CurlyEpsilon]",
        TraditionalForm], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"u", " ", "z"}], "-", "1"}], ")"}], " ", 
       TemplateBox[{"1",RowBox[{"1", "-", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}],RowBox[{"2", "-", 
           RowBox[{"2", " ", 
             TagBox["\[CurlyEpsilon]", TraditionalForm]}]}],
         RowBox[{"1", "-", 
           RowBox[{"u", " ", "z"}]}]},
        "Hypergeometric2F1"]}], "+", "1"}], ")"}]}], 
   RowBox[{
    TemplateBox[{RowBox[{
        FractionBox["1", "2"], "-", 
        TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{
        FractionBox["3", "2"], "-", 
        TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
     "Gamma"]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.472901279188428*^9, 3.501051705364524*^9, 
  3.501052501867178*^9, 3.5010734542816553`*^9, 3.501077422761548*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"tint2", "=", 
  RowBox[{"PowerSimplify", "[", "tint", "]"}]}]], "Input"],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"\[Pi]", " ", 
    SuperscriptBox["16", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]], " ", 
    TemplateBox[{RowBox[{
        RowBox[{"-", "2"}], " ", 
        TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
     "Gamma"], " ", 
    SuperscriptBox["u", 
     RowBox[{
      RowBox[{"-", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}], "-", "1"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", "2"}], " ", 
      TagBox["\[CurlyEpsilon]",
       TraditionalForm]}]], " ", 
    SuperscriptBox["z", 
     RowBox[{
      RowBox[{"-", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}], "-", "1"}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"2", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}]}], "+", 
      RowBox[{
       TagBox["\[CurlyEpsilon]",
        TraditionalForm], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"u", " ", "z"}], "-", "1"}], ")"}], " ", 
       TemplateBox[{"1",RowBox[{"1", "-", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}],RowBox[{"2", "-", 
           RowBox[{"2", " ", 
             TagBox["\[CurlyEpsilon]", TraditionalForm]}]}],
         RowBox[{"1", "-", 
           RowBox[{"u", " ", "z"}]}]},
        "Hypergeometric2F1"]}], "+", "1"}], ")"}]}], 
   RowBox[{
    TemplateBox[{RowBox[{
        FractionBox["1", "2"], "-", 
        TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{
        FractionBox["3", "2"], "-", 
        TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
     "Gamma"]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.472901290744051*^9, 3.501051705819482*^9, 
  3.50105250245513*^9, 3.501073454645144*^9, 3.501077423162826*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "PowerSimplify"}]], "Input"],

Cell[BoxData[
 StyleBox["\<\"PowerSimplify[exp]  simplifies (-x)^a to (-1)^a x^a \
and\\n(y-x)^n to (-1)^n (x-y)^n; thus assuming that the exponent is\\nan \
integer (even if it is symbolic). Furthermore\\n(-1)^(a+n) and I^(a+n) are \
expanded and (I)^(2 m) -> (-1)^m and\\n(-1)^(n_Integer?EvenQ m) -> 1 \
and\\n(-1)^(n_Integer?OddQ m) -> (-1)^m and\\n(-1)^(-n) -> (-1)^n and Exp[I m \
Pi] -> (-1)^m.\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.501077423452754*^9},
 CellTags->"Info3501081023-3542981"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"uint", "=", 
  RowBox[{"PowerSimplify", "@", 
   RowBox[{"Collect2", "[", 
    RowBox[{
     RowBox[{
      SubsuperscriptBox["\[Integral]", "0", "1"], 
      RowBox[{"tint2", 
       RowBox[{"\[DifferentialD]", "u"}]}]}], " ", ",", " ", "z"}], 
    "]"}]}]}]], "Input",
 CellChangeTimes->{{3.47290137110581*^9, 3.472901407159111*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"\[Pi]", " ", 
      SuperscriptBox["16", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]], " ", 
      TemplateBox[{RowBox[{"2", "-", 
          RowBox[{"2", " ", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{
          RowBox[{"-", "2"}], " ", 
          TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
       "Gamma"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"1", "-", "z"}], ")"}], 
       RowBox[{
        RowBox[{"-", "2"}], " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}]], " ", 
      SuperscriptBox["z", 
       RowBox[{"-", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}]], " ", 
      TagBox[
       TagBox[
        RowBox[{
         RowBox[{
          SubscriptBox["\[InvisiblePrefixScriptBase]", "3"], 
          SubscriptBox["F", "2"]}], "\[InvisibleApplication]", 
         RowBox[{"(", 
          RowBox[{
           TagBox[
            TagBox[
             RowBox[{
              TagBox["1",
               HypergeometricPFQ,
               Editable->True,
               Selectable->True], ",", 
              TagBox[
               RowBox[{"1", "-", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}],
               HypergeometricPFQ,
               Editable->True,
               Selectable->True], ",", 
              TagBox[
               RowBox[{"1", "-", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}],
               HypergeometricPFQ,
               Editable->True,
               Selectable->True]}],
             InterpretTemplate[{
               SlotSequence[1]}& ]],
            HypergeometricPFQ,
            Editable->False,
            Selectable->False], ";", 
           TagBox[
            TagBox[
             RowBox[{
              TagBox[
               RowBox[{"2", "-", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}],
               HypergeometricPFQ,
               Editable->True,
               Selectable->True], ",", 
              TagBox[
               RowBox[{
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm], "+", "1"}],
               HypergeometricPFQ,
               Editable->True,
               Selectable->True]}],
             InterpretTemplate[{
               SlotSequence[1]}& ]],
            HypergeometricPFQ,
            Editable->False,
            Selectable->False], ";", 
           TagBox["z",
            HypergeometricPFQ,
            Editable->True,
            Selectable->True]}], ")"}]}],
        InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
        Editable->False,
        Selectable->False],
       HypergeometricPFQ]}], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "-", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}], ")"}], " ", 
      TemplateBox[{RowBox[{"1", "-", 
          RowBox[{"2", " ", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{
          FractionBox["1", "2"], "-", 
          TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{
          FractionBox["3", "2"], "-", 
          TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
       "Gamma"]}]]}], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["\[Pi]", "2"], " ", 
     SuperscriptBox["16", 
      TagBox["\[CurlyEpsilon]",
       TraditionalForm]], " ", 
     RowBox[{"csc", "(", 
      RowBox[{"\[Pi]", " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}], ")"}], " ", 
     TemplateBox[{RowBox[{"2", "-", 
         RowBox[{"2", " ", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
      "Gamma"], " ", 
     TemplateBox[{RowBox[{
         RowBox[{"-", "2"}], " ", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "z"}], ")"}], 
      RowBox[{
       RowBox[{"-", "2"}], " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}]], " ", 
     SuperscriptBox["z", 
      RowBox[{
       RowBox[{"-", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}], "-", "1"}]], " ", 
     TemplateBox[{"1",RowBox[{"-", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}],RowBox[{
         TagBox["\[CurlyEpsilon]", TraditionalForm], "+", "1"}],"z"},
      "Hypergeometric2F1"]}], 
    RowBox[{
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], " ", 
     TemplateBox[{RowBox[{"1", "-", 
         RowBox[{"2", " ", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
      "Gamma"], " ", 
     TemplateBox[{RowBox[{
         FractionBox["1", "2"], "-", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"], " ", 
     TemplateBox[{RowBox[{"1", "-", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"], " ", 
     TemplateBox[{RowBox[{
         FractionBox["3", "2"], "-", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"], " ", 
     TemplateBox[{TagBox["\[CurlyEpsilon]", TraditionalForm]},
      "Gamma"]}]], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox["\[Pi]", "2"], " ", 
     SuperscriptBox["16", 
      TagBox["\[CurlyEpsilon]",
       TraditionalForm]], " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], " ", 
     RowBox[{"csc", "(", 
      RowBox[{"\[Pi]", " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}], ")"}], " ", 
     TemplateBox[{RowBox[{"1", "-", 
         RowBox[{"2", " ", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
      "Gamma"], " ", 
     TemplateBox[{RowBox[{
         RowBox[{"-", "2"}], " ", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "z"}], ")"}], 
      RowBox[{
       RowBox[{"-", "2"}], " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}]], " ", 
     SuperscriptBox["z", 
      RowBox[{
       RowBox[{"-", "2"}], " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}]], " ", 
     TemplateBox[{RowBox[{"1", "-", 
         RowBox[{"2", " ", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}]}],RowBox[{"1", "-", 
         RowBox[{"2", " ", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}]}],RowBox[{"2", "-", 
         RowBox[{"2", " ", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}]}],"z"},
      "Hypergeometric2F1"]}], 
    RowBox[{
     TemplateBox[{RowBox[{
         FractionBox["1", "2"], "-", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"], " ", 
     SuperscriptBox[
      TemplateBox[{RowBox[{"1", "-", 
          TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
       "Gamma"], "2"], " ", 
     TemplateBox[{RowBox[{
         FractionBox["3", "2"], "-", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"]}]], "+", 
   FractionBox[
    RowBox[{"\[Pi]", " ", 
     SuperscriptBox["16", 
      TagBox["\[CurlyEpsilon]",
       TraditionalForm]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}], "-", "1"}], ")"}], " ", 
     TemplateBox[{RowBox[{
         RowBox[{"-", "2"}], " ", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "z"}], ")"}], 
      RowBox[{
       RowBox[{"-", "2"}], " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}]], " ", 
     SuperscriptBox["z", 
      RowBox[{
       RowBox[{"-", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}], "-", "1"}]]}], 
    RowBox[{
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], " ", 
     TemplateBox[{RowBox[{
         FractionBox["1", "2"], "-", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"], " ", 
     TemplateBox[{RowBox[{
         FractionBox["3", "2"], "-", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"]}]], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["\[Pi]", "2"], " ", 
     SuperscriptBox["16", 
      TagBox["\[CurlyEpsilon]",
       TraditionalForm]], " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], " ", 
     RowBox[{"csc", "(", 
      RowBox[{"\[Pi]", " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}], ")"}], " ", 
     TemplateBox[{RowBox[{"2", "-", 
         RowBox[{"2", " ", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
      "Gamma"], " ", 
     SuperscriptBox[
      TemplateBox[{RowBox[{
          RowBox[{"-", "2"}], " ", 
          TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
       "Gamma"], "2"], " ", 
     SuperscriptBox["z", 
      RowBox[{
       RowBox[{"-", 
        RowBox[{"2", " ", 
         TagBox["\[CurlyEpsilon]",
          TraditionalForm]}]}], "-", "1"}]]}], 
    RowBox[{
     TemplateBox[{RowBox[{"1", "-", 
         RowBox[{"2", " ", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
      "Gamma"], " ", 
     TemplateBox[{RowBox[{
         FractionBox["1", "2"], "-", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"], " ", 
     SuperscriptBox[
      TemplateBox[{RowBox[{"1", "-", 
          TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
       "Gamma"], "2"], " ", 
     TemplateBox[{RowBox[{
         FractionBox["3", "2"], "-", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{
  3.47290133017855*^9, {3.472901387426004*^9, 3.472901415333036*^9}, 
   3.5010517371577177`*^9, 3.501052533465047*^9, 3.501073485877775*^9, 
   3.5010774541227922`*^9}]
}, Open  ]],

Cell["Perform the z - integration:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"rint", "=", 
  RowBox[{"Collect2", "[", 
   RowBox[{
    RowBox[{"FunctionExpand", "@", 
     RowBox[{
      SubsuperscriptBox["\[Integral]", "0", "1"], 
      RowBox[{"uint", 
       RowBox[{"\[DifferentialD]", "z"}]}]}]}], " ", ",", " ", 
    "HypergeometricPFQ"}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.472901462939465*^9, {3.4729015170057487`*^9, 3.472901518302103*^9}, {
   3.472901625298542*^9, 3.4729016484911137`*^9}, {3.472901681866604*^9, 
   3.472901693902978*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     TemplateBox[{RowBox[{"-", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"], " ", 
     SuperscriptBox[
      TemplateBox[{RowBox[{"1", "-", 
          TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
       "Gamma"], "2"], " ", 
     TagBox[
      TagBox[
       RowBox[{
        RowBox[{
         SubscriptBox["\[InvisiblePrefixScriptBase]", "4"], 
         SubscriptBox["F", "3"]}], "\[InvisibleApplication]", 
        RowBox[{"(", 
         RowBox[{
          TagBox[
           TagBox[
            RowBox[{
             TagBox["1",
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"1", "-", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"1", "-", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"1", "-", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True]}],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           HypergeometricPFQ,
           Editable->False,
           Selectable->False], ";", 
          TagBox[
           TagBox[
            RowBox[{
             TagBox[
              RowBox[{"2", "-", 
               RowBox[{"3", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"2", "-", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{
               TagBox["\[CurlyEpsilon]",
                TraditionalForm], "+", "1"}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True]}],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           HypergeometricPFQ,
           Editable->False,
           Selectable->False], ";", 
          TagBox["1",
           HypergeometricPFQ,
           Editable->True,
           Selectable->True]}], ")"}]}],
       InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
       Editable->False,
       Selectable->False],
      HypergeometricPFQ]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       TagBox["\[CurlyEpsilon]",
        TraditionalForm], "-", "1"}], ")"}], " ", 
     TemplateBox[{RowBox[{"2", "-", 
         RowBox[{"3", " ", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
      "Gamma"]}]], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["\[Pi]", 
      RowBox[{"3", "/", "2"}]], " ", 
     SuperscriptBox["2", 
      RowBox[{"4", " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}]], " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], " ", 
     RowBox[{"csc", "(", 
      RowBox[{"\[Pi]", " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}], ")"}], " ", 
     TemplateBox[{RowBox[{
         RowBox[{"-", "2"}], " ", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"], " ", 
     TagBox[
      TagBox[
       RowBox[{
        RowBox[{
         SubscriptBox["\[InvisiblePrefixScriptBase]", "3"], 
         SubscriptBox["F", "2"]}], "\[InvisibleApplication]", 
        RowBox[{"(", 
         RowBox[{
          TagBox[
           TagBox[
            RowBox[{
             TagBox[
              RowBox[{"1", "-", 
               RowBox[{"2", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"1", "-", 
               RowBox[{"2", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"1", "-", 
               RowBox[{"2", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True]}],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           HypergeometricPFQ,
           Editable->False,
           Selectable->False], ";", 
          TagBox[
           TagBox[
            RowBox[{
             TagBox[
              RowBox[{"2", "-", 
               RowBox[{"4", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"2", "-", 
               RowBox[{"2", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True]}],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           HypergeometricPFQ,
           Editable->False,
           Selectable->False], ";", 
          TagBox["1",
           HypergeometricPFQ,
           Editable->True,
           Selectable->True]}], ")"}]}],
       InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
       Editable->False,
       Selectable->False],
      HypergeometricPFQ]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}], "-", "1"}], ")"}], " ", 
     TemplateBox[{RowBox[{
         FractionBox["3", "2"], "-", 
         RowBox[{"2", " ", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
      "Gamma"]}]], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      TemplateBox[{RowBox[{"-", 
          TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
       "Gamma"], "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"4", " ", 
        SuperscriptBox[
         TagBox["\[CurlyEpsilon]",
          TraditionalForm], "2"], " ", 
        TemplateBox[{RowBox[{
            RowBox[{"-", "3"}], " ", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
         "Gamma"], " ", 
        TemplateBox[{RowBox[{"-", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
         "Gamma"], " ", 
        TemplateBox[{TagBox["\[CurlyEpsilon]", TraditionalForm]},
         "Gamma"]}], "+", 
       RowBox[{"\[Pi]", " ", 
        RowBox[{"csc", "(", 
         RowBox[{"\[Pi]", " ", 
          TagBox["\[CurlyEpsilon]",
           TraditionalForm]}], ")"}], " ", 
        TemplateBox[{RowBox[{"1", "-", 
            RowBox[{"3", " ", 
              TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
         "Gamma"]}]}], ")"}]}], 
    RowBox[{"4", " ", 
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "2"], " ", 
     TemplateBox[{RowBox[{"1", "-", 
         RowBox[{"3", " ", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
      "Gamma"], " ", 
     TemplateBox[{RowBox[{
         RowBox[{"-", "3"}], " ", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"], " ", 
     TemplateBox[{TagBox["\[CurlyEpsilon]", TraditionalForm]},
      "Gamma"]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.47290144104503*^9, 3.472901488075706*^9, 
  3.4729015257429667`*^9, 3.472901634811721*^9, 3.4729016679408293`*^9, 
  3.472901706204863*^9, 3.501051749524122*^9, 3.501052545887735*^9, 
  3.501073498479265*^9, 3.501077466572907*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"hy", "=", 
  RowBox[{"Cases2", "[", 
   RowBox[{"rint", ",", "HypergeometricPFQ"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.472901537407631*^9, 3.472901538088868*^9}, 
   3.47290163468815*^9, {3.472901689226424*^9, 3.472901690153994*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "3"], 
        SubscriptBox["F", "2"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"1", "-", 
              RowBox[{"2", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", 
              RowBox[{"2", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", 
              RowBox[{"2", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          HypergeometricPFQ,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"2", "-", 
              RowBox[{"4", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"2", "-", 
              RowBox[{"2", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          HypergeometricPFQ,
          Editable->False,
          Selectable->False], ";", 
         TagBox["1",
          HypergeometricPFQ,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     HypergeometricPFQ], ",", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "4"], 
        SubscriptBox["F", "3"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox["1",
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", 
              TagBox["\[CurlyEpsilon]",
               TraditionalForm]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", 
              TagBox["\[CurlyEpsilon]",
               TraditionalForm]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", 
              TagBox["\[CurlyEpsilon]",
               TraditionalForm]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          HypergeometricPFQ,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"2", "-", 
              RowBox[{"3", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"2", "-", 
              TagBox["\[CurlyEpsilon]",
               TraditionalForm]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{
              TagBox["\[CurlyEpsilon]",
               TraditionalForm], "+", "1"}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          HypergeometricPFQ,
          Editable->False,
          Selectable->False], ";", 
         TagBox["1",
          HypergeometricPFQ,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     HypergeometricPFQ]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{{3.4729015326003847`*^9, 3.4729015390410137`*^9}, 
   3.47290170646313*^9, 3.501051749626875*^9, 3.5010525459968233`*^9, 
   3.501073498651243*^9, 3.5010774669016*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"hn", "=", 
  RowBox[{"HypergeometricSE", "[", 
   RowBox[{"hy", ",", "\[Nu]"}], "]"}]}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    FractionBox[
     RowBox[{
      TemplateBox[{RowBox[{"2", "-", 
          RowBox[{"4", " ", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{"2", "-", 
          RowBox[{"2", " ", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
       "Gamma"], " ", 
      SuperscriptBox[
       TemplateBox[{RowBox[{
           RowBox[{
             RowBox[{"-", "2"}], " ", 
             TagBox["\[CurlyEpsilon]", TraditionalForm]}], "+", "\[Nu]", "+", 
           "1"}]},
        "Gamma"], "3"]}], 
     RowBox[{
      SuperscriptBox[
       TemplateBox[{RowBox[{"1", "-", 
           RowBox[{"2", " ", 
             TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
        "Gamma"], "3"], " ", 
      TemplateBox[{RowBox[{"\[Nu]", "+", "1"}]},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{
          RowBox[{
            RowBox[{"-", "4"}], " ", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}], "+", "\[Nu]", "+", 
          "2"}]},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{
          RowBox[{
            RowBox[{"-", "2"}], " ", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}], "+", "\[Nu]", "+", 
          "2"}]},
       "Gamma"]}]], ",", 
    FractionBox[
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        TagBox["\[CurlyEpsilon]",
         TraditionalForm], "-", "1"}], ")"}], " ", 
      TemplateBox[{RowBox[{"2", "-", 
          RowBox[{"3", " ", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{
          TagBox["\[CurlyEpsilon]", TraditionalForm], "+", "1"}]},
       "Gamma"], " ", 
      SuperscriptBox[
       TemplateBox[{RowBox[{
           RowBox[{"-", 
             TagBox["\[CurlyEpsilon]", TraditionalForm]}], "+", "\[Nu]", "+", 
           "1"}]},
        "Gamma"], "2"]}], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        TagBox["\[CurlyEpsilon]",
         TraditionalForm], "-", "\[Nu]", "-", "1"}], ")"}], " ", 
      SuperscriptBox[
       TemplateBox[{RowBox[{"1", "-", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
        "Gamma"], "2"], " ", 
      TemplateBox[{RowBox[{
          RowBox[{
            RowBox[{"-", "3"}], " ", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}], "+", "\[Nu]", "+", 
          "2"}]},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{
          TagBox["\[CurlyEpsilon]", TraditionalForm], "+", "\[Nu]", "+", 
          "1"}]},
       "Gamma"]}]]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.472901567151018*^9, 3.472901720031994*^9, 
  3.501051750104823*^9, 3.501052546674489*^9, 3.501073499255476*^9, 
  3.5010774674917507`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "HypergeometricSE"}]], "Input"],

Cell[BoxData[
 StyleBox["\<\"HypergeometricSE[exp, nu] expresses Hypergeometric functions \
by\\ntheir series expansion in terms of a sum (the Sum is omitted and\\nnu, \
running from 0 to Infinity is the summation index).\"\>", "MSG"]], "Print", \
"PrintUsage",
 CellChangeTimes->{3.5010774679036427`*^9},
 CellTags->"Info3501081067-3542981"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FunctionExpand", "[", 
    RowBox[{
     UnderoverscriptBox["\[Sum]", 
      RowBox[{"\[Nu]", "=", "0"}], "\[Infinity]"], "#"}], "]"}], "&"}], "/@", 
  "hn"}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "3"], 
        SubscriptBox["F", "2"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"1", "-", 
              RowBox[{"2", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", 
              RowBox[{"2", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", 
              RowBox[{"2", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          HypergeometricPFQ,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"2", "-", 
              RowBox[{"4", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"2", "-", 
              RowBox[{"2", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          HypergeometricPFQ,
          Editable->False,
          Selectable->False], ";", 
         TagBox["1",
          HypergeometricPFQ,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     HypergeometricPFQ], ",", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "4"], 
        SubscriptBox["F", "3"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox["1",
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", 
              TagBox["\[CurlyEpsilon]",
               TraditionalForm]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", 
              TagBox["\[CurlyEpsilon]",
               TraditionalForm]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", 
              TagBox["\[CurlyEpsilon]",
               TraditionalForm]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          HypergeometricPFQ,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"2", "-", 
              RowBox[{"3", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"2", "-", 
              TagBox["\[CurlyEpsilon]",
               TraditionalForm]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{
              TagBox["\[CurlyEpsilon]",
               TraditionalForm], "+", "1"}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          HypergeometricPFQ,
          Editable->False,
          Selectable->False], ";", 
         TagBox["1",
          HypergeometricPFQ,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     HypergeometricPFQ]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.472901735719531*^9, 3.5010517691588697`*^9, 
  3.501052565862933*^9, 3.501073518432239*^9, 3.501077487434729*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionExpand", "[", 
  RowBox[{"hn", "/.", 
   RowBox[{"\[Nu]", "\[Rule]", "0"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"1", ",", "1"}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.472901745961268*^9, 3.5010517694774942`*^9, 
  3.501052566180195*^9, 3.501073518831684*^9, 3.5010774878240643`*^9}]
}, Open  ]],

Cell[TextData[{
 "Do a series expansion up to order ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    TagBox["\[CurlyEpsilon]",
     TraditionalForm], "4"], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"hnexp", " ", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"Apart", "[", 
     RowBox[{
      RowBox[{"FunctionExpand", "[", "#", "]"}], ",", "\[Nu]"}], "]"}], "&"}],
    "/@", 
   RowBox[{"Series2", "[", 
    RowBox[{"hn", ",", "Epsilon", ",", "2"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.472901771006505*^9, 3.47290180158803*^9}, 
   3.4729018850967712`*^9, {3.4729019526335897`*^9, 3.47290197896279*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     FractionBox[
      RowBox[{
       RowBox[{"4", " ", 
        SuperscriptBox[
         TagBox["\[CurlyEpsilon]",
          TraditionalForm], "2"], " ", 
        RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
       RowBox[{"4", " ", 
        SuperscriptBox[
         TagBox["\[CurlyEpsilon]",
          TraditionalForm], "2"], " ", 
        TemplateBox[{"1",RowBox[{"\[Nu]", "+", "1"}]},
         "PolyGamma2"]}], "+", 
       RowBox[{"8", " ", 
        SuperscriptBox[
         TagBox["\[CurlyEpsilon]",
          TraditionalForm], "2"]}], "-", 
       RowBox[{"6", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}], "+", "1"}], 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"\[Nu]", "+", "1"}], ")"}], "2"]], "+", 
     FractionBox[
      RowBox[{"28", " ", 
       SuperscriptBox[
        TagBox["\[CurlyEpsilon]",
         TraditionalForm], "2"]}], 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"\[Nu]", "+", "1"}], ")"}], "4"]], "-", 
     FractionBox[
      RowBox[{"6", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"6", " ", 
          SuperscriptBox[
           TagBox["\[CurlyEpsilon]",
            TraditionalForm], "2"]}], "-", 
         TagBox["\[CurlyEpsilon]",
          TraditionalForm]}], ")"}]}], 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"\[Nu]", "+", "1"}], ")"}], "3"]]}], ",", 
    RowBox[{
     FractionBox[
      RowBox[{
       RowBox[{"4", " ", 
        SuperscriptBox[
         TagBox["\[CurlyEpsilon]",
          TraditionalForm], "2"], " ", 
        RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
       RowBox[{"4", " ", 
        SuperscriptBox[
         TagBox["\[CurlyEpsilon]",
          TraditionalForm], "2"], " ", 
        TemplateBox[{"1",RowBox[{"\[Nu]", "+", "1"}]},
         "PolyGamma2"]}], "+", 
       RowBox[{"3", " ", 
        SuperscriptBox[
         TagBox["\[CurlyEpsilon]",
          TraditionalForm], "2"]}], "-", 
       RowBox[{"4", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}], "+", "1"}], 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"\[Nu]", "+", "1"}], ")"}], "2"]], "+", 
     FractionBox[
      RowBox[{"13", " ", 
       SuperscriptBox[
        TagBox["\[CurlyEpsilon]",
         TraditionalForm], "2"]}], 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"\[Nu]", "+", "1"}], ")"}], "4"]], "-", 
     FractionBox[
      RowBox[{"4", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"4", " ", 
          SuperscriptBox[
           TagBox["\[CurlyEpsilon]",
            TraditionalForm], "2"]}], "-", 
         TagBox["\[CurlyEpsilon]",
          TraditionalForm]}], ")"}]}], 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"\[Nu]", "+", "1"}], ")"}], "3"]]}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{{3.4729017567747993`*^9, 3.472901818963334*^9}, 
   3.472901886939904*^9, {3.472901956171627*^9, 3.472901980284807*^9}, 
   3.501051770897649*^9, 3.501052567642013*^9, 3.501073520298209*^9, 
   3.501077489304187*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "Series2"}]], "Input"],

Cell[BoxData[
 StyleBox["\<\"Series2 performs a series expansion around 0.\\nSeries2 is \
similar to Series, except that it applies Normal\\non the result and that \
some Series bugs are fixed.\\nSeries2[f, e, n] is equivalent to \\nSeries2[f, \
{e, 0, n}] is equivalent to Series[f, {e, 0, n}].\"\>", "MSG"]], "Print", \
"PrintUsage",
 CellChangeTimes->{3.501077489787003*^9},
 CellTags->"Info3501081089-3542981"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"hypexp", "=", 
   RowBox[{"1", "+", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"Sum", "[", 
        RowBox[{"#", ",", 
         RowBox[{"{", 
          RowBox[{"\[Nu]", ",", "1", ",", "\[Infinity]"}], "}"}]}], "]"}], 
       "&"}], "/@", "hnexp"}], ")"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.501057323227414*^9, 3.501057325179188*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"2247.1099999999997`", ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{
        FractionBox["1", "30"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"20", " ", 
           SuperscriptBox["\[Pi]", "2"], " ", 
           SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
             TraditionalForm], "2"], " ", 
           RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
          RowBox[{"120", " ", 
           SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
             TraditionalForm], "2"], " ", 
           RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
          RowBox[{"7", " ", 
           SuperscriptBox["\[Pi]", "4"], " ", 
           SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
             TraditionalForm], "2"]}], "+", 
          RowBox[{"60", " ", 
           SuperscriptBox["\[Pi]", "2"], " ", 
           SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
             TraditionalForm], "2"]}], "-", 
          RowBox[{"30", " ", 
           SuperscriptBox["\[Pi]", "2"], " ", 
           TagBox["\[CurlyEpsilon]",
            TraditionalForm]}], "-", 
          RowBox[{"1080", " ", 
           SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
             TraditionalForm], "2"], " ", 
           TemplateBox[{"3"},
            "Zeta"]}], "+", 
          RowBox[{"180", " ", 
           TagBox["\[CurlyEpsilon]",
            TraditionalForm], " ", 
           TemplateBox[{"3"},
            "Zeta"]}], "+", 
          RowBox[{"5", " ", 
           SuperscriptBox["\[Pi]", "2"]}], "-", "30"}], ")"}]}], "+", "1"}], 
      ",", 
      RowBox[{
       RowBox[{
        FractionBox["1", "30"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"20", " ", 
           SuperscriptBox["\[Pi]", "2"], " ", 
           SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
             TraditionalForm], "2"], " ", 
           RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
          RowBox[{"120", " ", 
           SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
             TraditionalForm], "2"], " ", 
           RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
          RowBox[{"2", " ", 
           SuperscriptBox["\[Pi]", "4"], " ", 
           SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
             TraditionalForm], "2"]}], "+", 
          RowBox[{"35", " ", 
           SuperscriptBox["\[Pi]", "2"], " ", 
           SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
             TraditionalForm], "2"]}], "-", 
          RowBox[{"20", " ", 
           SuperscriptBox["\[Pi]", "2"], " ", 
           TagBox["\[CurlyEpsilon]",
            TraditionalForm]}], "-", 
          RowBox[{"480", " ", 
           SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
             TraditionalForm], "2"], " ", 
           TemplateBox[{"3"},
            "Zeta"]}], "+", 
          RowBox[{"120", " ", 
           TagBox["\[CurlyEpsilon]",
            TraditionalForm], " ", 
           TemplateBox[{"3"},
            "Zeta"]}], "+", 
          RowBox[{"5", " ", 
           SuperscriptBox["\[Pi]", "2"]}], "-", "30"}], ")"}]}], "+", "1"}]}],
      "}"}]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.472901878629587*^9, 3.472901948935339*^9, 
  3.501051878595078*^9, 3.501059532405798*^9, 3.5010757277648287`*^9, 
  3.501079697260551*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"N", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"hy", "-", "hypexp"}], ")"}], "/.", 
   RowBox[{"Epsilon", "\[Rule]", "0.01"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5010720182971907`*^9, 3.501072029003577*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"-", "2.2873248535226764`*^-6"}], ",", 
    RowBox[{"-", "3.962859360218118`*^-6"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.501071932439824*^9, 3.5010720294277287`*^9, 
  3.501075728241743*^9, 3.5010796977261744`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"N", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"hy", "-", "hypexp"}], ")"}], "/.", 
   RowBox[{"Epsilon", "\[Rule]", "0.001"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.501072040508606*^9, 3.501072043119749*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"-", "2.303723878327446`*^-9"}], ",", 
    RowBox[{"-", "4.008294696156156`*^-9"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.501072043952736*^9, 3.501075728386204*^9, 
  3.501079697869298*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"N", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"hy", "-", "hypexp"}], ")"}], "/.", 
   RowBox[{"Epsilon", "\[Rule]", "0.000001"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.501072040508606*^9, 3.501072043119749*^9}, {
  3.501072333560631*^9, 3.501072345258131*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"4.440892098500626`*^-16", ",", 
    RowBox[{"-", "1.1102230246251565`*^-16"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{{3.501072334374239*^9, 3.50107234611835*^9}, 
   3.501075728630909*^9, 3.501079697968733*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["hy"], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "3"], 
        SubscriptBox["F", "2"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"1", "-", 
              RowBox[{"2", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", 
              RowBox[{"2", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", 
              RowBox[{"2", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          HypergeometricPFQ,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"2", "-", 
              RowBox[{"4", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"2", "-", 
              RowBox[{"2", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          HypergeometricPFQ,
          Editable->False,
          Selectable->False], ";", 
         TagBox["1",
          HypergeometricPFQ,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     HypergeometricPFQ], ",", 
    TagBox[
     TagBox[
      RowBox[{
       RowBox[{
        SubscriptBox["\[InvisiblePrefixScriptBase]", "4"], 
        SubscriptBox["F", "3"]}], "\[InvisibleApplication]", 
       RowBox[{"(", 
        RowBox[{
         TagBox[
          TagBox[
           RowBox[{
            TagBox["1",
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", 
              TagBox["\[CurlyEpsilon]",
               TraditionalForm]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", 
              TagBox["\[CurlyEpsilon]",
               TraditionalForm]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"1", "-", 
              TagBox["\[CurlyEpsilon]",
               TraditionalForm]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          HypergeometricPFQ,
          Editable->False,
          Selectable->False], ";", 
         TagBox[
          TagBox[
           RowBox[{
            TagBox[
             RowBox[{"2", "-", 
              RowBox[{"3", " ", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{"2", "-", 
              TagBox["\[CurlyEpsilon]",
               TraditionalForm]}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True], ",", 
            TagBox[
             RowBox[{
              TagBox["\[CurlyEpsilon]",
               TraditionalForm], "+", "1"}],
             HypergeometricPFQ,
             Editable->True,
             Selectable->True]}],
           InterpretTemplate[{
             SlotSequence[1]}& ]],
          HypergeometricPFQ,
          Editable->False,
          Selectable->False], ";", 
         TagBox["1",
          HypergeometricPFQ,
          Editable->True,
          Selectable->True]}], ")"}]}],
      InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
      Editable->False,
      Selectable->False],
     HypergeometricPFQ]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.501072049739265*^9, 3.5010757286575193`*^9, 
  3.5010796980183*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["hypexp"], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{
      FractionBox["1", "30"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"20", " ", 
         SuperscriptBox["\[Pi]", "2"], " ", 
         SuperscriptBox[
          TagBox["\[CurlyEpsilon]",
           TraditionalForm], "2"], " ", 
         RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
        RowBox[{"120", " ", 
         SuperscriptBox[
          TagBox["\[CurlyEpsilon]",
           TraditionalForm], "2"], " ", 
         RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
        RowBox[{"7", " ", 
         SuperscriptBox["\[Pi]", "4"], " ", 
         SuperscriptBox[
          TagBox["\[CurlyEpsilon]",
           TraditionalForm], "2"]}], "+", 
        RowBox[{"60", " ", 
         SuperscriptBox["\[Pi]", "2"], " ", 
         SuperscriptBox[
          TagBox["\[CurlyEpsilon]",
           TraditionalForm], "2"]}], "-", 
        RowBox[{"30", " ", 
         SuperscriptBox["\[Pi]", "2"], " ", 
         TagBox["\[CurlyEpsilon]",
          TraditionalForm]}], "-", 
        RowBox[{"1080", " ", 
         SuperscriptBox[
          TagBox["\[CurlyEpsilon]",
           TraditionalForm], "2"], " ", 
         TemplateBox[{"3"},
          "Zeta"]}], "+", 
        RowBox[{"180", " ", 
         TagBox["\[CurlyEpsilon]",
          TraditionalForm], " ", 
         TemplateBox[{"3"},
          "Zeta"]}], "+", 
        RowBox[{"5", " ", 
         SuperscriptBox["\[Pi]", "2"]}], "-", "30"}], ")"}]}], "+", "1"}], 
    ",", 
    RowBox[{
     RowBox[{
      FractionBox["1", "30"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"20", " ", 
         SuperscriptBox["\[Pi]", "2"], " ", 
         SuperscriptBox[
          TagBox["\[CurlyEpsilon]",
           TraditionalForm], "2"], " ", 
         RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
        RowBox[{"120", " ", 
         SuperscriptBox[
          TagBox["\[CurlyEpsilon]",
           TraditionalForm], "2"], " ", 
         RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
        RowBox[{"2", " ", 
         SuperscriptBox["\[Pi]", "4"], " ", 
         SuperscriptBox[
          TagBox["\[CurlyEpsilon]",
           TraditionalForm], "2"]}], "+", 
        RowBox[{"35", " ", 
         SuperscriptBox["\[Pi]", "2"], " ", 
         SuperscriptBox[
          TagBox["\[CurlyEpsilon]",
           TraditionalForm], "2"]}], "-", 
        RowBox[{"20", " ", 
         SuperscriptBox["\[Pi]", "2"], " ", 
         TagBox["\[CurlyEpsilon]",
          TraditionalForm]}], "-", 
        RowBox[{"480", " ", 
         SuperscriptBox[
          TagBox["\[CurlyEpsilon]",
           TraditionalForm], "2"], " ", 
         TemplateBox[{"3"},
          "Zeta"]}], "+", 
        RowBox[{"120", " ", 
         TagBox["\[CurlyEpsilon]",
          TraditionalForm], " ", 
         TemplateBox[{"3"},
          "Zeta"]}], "+", 
        RowBox[{"5", " ", 
         SuperscriptBox["\[Pi]", "2"]}], "-", "30"}], ")"}]}], "+", "1"}]}], 
   "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.501072054665413*^9, 3.501075728738429*^9, 
  3.501079698087515*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[InvisibleSpace]", 
  RowBox[{"Thread", "[", 
   RowBox[{"hy", "\[Rule]", "hypexp"}], "]"}]}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     TagBox[
      TagBox[
       RowBox[{
        RowBox[{
         SubscriptBox["\[InvisiblePrefixScriptBase]", "3"], 
         SubscriptBox["F", "2"]}], "\[InvisibleApplication]", 
        RowBox[{"(", 
         RowBox[{
          TagBox[
           TagBox[
            RowBox[{
             TagBox[
              RowBox[{"1", "-", 
               RowBox[{"2", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"1", "-", 
               RowBox[{"2", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"1", "-", 
               RowBox[{"2", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True]}],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           HypergeometricPFQ,
           Editable->False,
           Selectable->False], ";", 
          TagBox[
           TagBox[
            RowBox[{
             TagBox[
              RowBox[{"2", "-", 
               RowBox[{"4", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"2", "-", 
               RowBox[{"2", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True]}],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           HypergeometricPFQ,
           Editable->False,
           Selectable->False], ";", 
          TagBox["1",
           HypergeometricPFQ,
           Editable->True,
           Selectable->True]}], ")"}]}],
       InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
       Editable->False,
       Selectable->False],
      HypergeometricPFQ], "\[Rule]", 
     RowBox[{
      RowBox[{
       FractionBox["1", "30"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"20", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          SuperscriptBox[
           TagBox["\[CurlyEpsilon]",
            TraditionalForm], "2"], " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
         RowBox[{"120", " ", 
          SuperscriptBox[
           TagBox["\[CurlyEpsilon]",
            TraditionalForm], "2"], " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
         RowBox[{"7", " ", 
          SuperscriptBox["\[Pi]", "4"], " ", 
          SuperscriptBox[
           TagBox["\[CurlyEpsilon]",
            TraditionalForm], "2"]}], "+", 
         RowBox[{"60", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          SuperscriptBox[
           TagBox["\[CurlyEpsilon]",
            TraditionalForm], "2"]}], "-", 
         RowBox[{"30", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          TagBox["\[CurlyEpsilon]",
           TraditionalForm]}], "-", 
         RowBox[{"1080", " ", 
          SuperscriptBox[
           TagBox["\[CurlyEpsilon]",
            TraditionalForm], "2"], " ", 
          TemplateBox[{"3"},
           "Zeta"]}], "+", 
         RowBox[{"180", " ", 
          TagBox["\[CurlyEpsilon]",
           TraditionalForm], " ", 
          TemplateBox[{"3"},
           "Zeta"]}], "+", 
         RowBox[{"5", " ", 
          SuperscriptBox["\[Pi]", "2"]}], "-", "30"}], ")"}]}], "+", "1"}]}], 
    ",", 
    RowBox[{
     TagBox[
      TagBox[
       RowBox[{
        RowBox[{
         SubscriptBox["\[InvisiblePrefixScriptBase]", "4"], 
         SubscriptBox["F", "3"]}], "\[InvisibleApplication]", 
        RowBox[{"(", 
         RowBox[{
          TagBox[
           TagBox[
            RowBox[{
             TagBox["1",
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"1", "-", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"1", "-", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"1", "-", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True]}],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           HypergeometricPFQ,
           Editable->False,
           Selectable->False], ";", 
          TagBox[
           TagBox[
            RowBox[{
             TagBox[
              RowBox[{"2", "-", 
               RowBox[{"3", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"2", "-", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{
               TagBox["\[CurlyEpsilon]",
                TraditionalForm], "+", "1"}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True]}],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           HypergeometricPFQ,
           Editable->False,
           Selectable->False], ";", 
          TagBox["1",
           HypergeometricPFQ,
           Editable->True,
           Selectable->True]}], ")"}]}],
       InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
       Editable->False,
       Selectable->False],
      HypergeometricPFQ], "\[Rule]", 
     RowBox[{
      RowBox[{
       FractionBox["1", "30"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"20", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          SuperscriptBox[
           TagBox["\[CurlyEpsilon]",
            TraditionalForm], "2"], " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
         RowBox[{"120", " ", 
          SuperscriptBox[
           TagBox["\[CurlyEpsilon]",
            TraditionalForm], "2"], " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
         RowBox[{"2", " ", 
          SuperscriptBox["\[Pi]", "4"], " ", 
          SuperscriptBox[
           TagBox["\[CurlyEpsilon]",
            TraditionalForm], "2"]}], "+", 
         RowBox[{"35", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          SuperscriptBox[
           TagBox["\[CurlyEpsilon]",
            TraditionalForm], "2"]}], "-", 
         RowBox[{"20", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          TagBox["\[CurlyEpsilon]",
           TraditionalForm]}], "-", 
         RowBox[{"480", " ", 
          SuperscriptBox[
           TagBox["\[CurlyEpsilon]",
            TraditionalForm], "2"], " ", 
          TemplateBox[{"3"},
           "Zeta"]}], "+", 
         RowBox[{"120", " ", 
          TagBox["\[CurlyEpsilon]",
           TraditionalForm], " ", 
          TemplateBox[{"3"},
           "Zeta"]}], "+", 
         RowBox[{"5", " ", 
          SuperscriptBox["\[Pi]", "2"]}], "-", "30"}], ")"}]}], "+", 
      "1"}]}]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.501072087440864*^9, 3.501075728840563*^9, 
  3.501079698186801*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["rint"], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     TemplateBox[{RowBox[{"-", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"], " ", 
     SuperscriptBox[
      TemplateBox[{RowBox[{"1", "-", 
          TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
       "Gamma"], "2"], " ", 
     TagBox[
      TagBox[
       RowBox[{
        RowBox[{
         SubscriptBox["\[InvisiblePrefixScriptBase]", "4"], 
         SubscriptBox["F", "3"]}], "\[InvisibleApplication]", 
        RowBox[{"(", 
         RowBox[{
          TagBox[
           TagBox[
            RowBox[{
             TagBox["1",
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"1", "-", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"1", "-", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"1", "-", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True]}],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           HypergeometricPFQ,
           Editable->False,
           Selectable->False], ";", 
          TagBox[
           TagBox[
            RowBox[{
             TagBox[
              RowBox[{"2", "-", 
               RowBox[{"3", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"2", "-", 
               TagBox["\[CurlyEpsilon]",
                TraditionalForm]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{
               TagBox["\[CurlyEpsilon]",
                TraditionalForm], "+", "1"}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True]}],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           HypergeometricPFQ,
           Editable->False,
           Selectable->False], ";", 
          TagBox["1",
           HypergeometricPFQ,
           Editable->True,
           Selectable->True]}], ")"}]}],
       InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
       Editable->False,
       Selectable->False],
      HypergeometricPFQ]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       TagBox["\[CurlyEpsilon]",
        TraditionalForm], "-", "1"}], ")"}], " ", 
     TemplateBox[{RowBox[{"2", "-", 
         RowBox[{"3", " ", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
      "Gamma"]}]], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["\[Pi]", 
      RowBox[{"3", "/", "2"}]], " ", 
     SuperscriptBox["2", 
      RowBox[{"4", " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}]], " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], " ", 
     RowBox[{"csc", "(", 
      RowBox[{"\[Pi]", " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}], ")"}], " ", 
     TemplateBox[{RowBox[{
         RowBox[{"-", "2"}], " ", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"], " ", 
     TagBox[
      TagBox[
       RowBox[{
        RowBox[{
         SubscriptBox["\[InvisiblePrefixScriptBase]", "3"], 
         SubscriptBox["F", "2"]}], "\[InvisibleApplication]", 
        RowBox[{"(", 
         RowBox[{
          TagBox[
           TagBox[
            RowBox[{
             TagBox[
              RowBox[{"1", "-", 
               RowBox[{"2", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"1", "-", 
               RowBox[{"2", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"1", "-", 
               RowBox[{"2", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True]}],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           HypergeometricPFQ,
           Editable->False,
           Selectable->False], ";", 
          TagBox[
           TagBox[
            RowBox[{
             TagBox[
              RowBox[{"2", "-", 
               RowBox[{"4", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True], ",", 
             TagBox[
              RowBox[{"2", "-", 
               RowBox[{"2", " ", 
                TagBox["\[CurlyEpsilon]",
                 TraditionalForm]}]}],
              HypergeometricPFQ,
              Editable->True,
              Selectable->True]}],
            InterpretTemplate[{
              SlotSequence[1]}& ]],
           HypergeometricPFQ,
           Editable->False,
           Selectable->False], ";", 
          TagBox["1",
           HypergeometricPFQ,
           Editable->True,
           Selectable->True]}], ")"}]}],
       InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
       Editable->False,
       Selectable->False],
      HypergeometricPFQ]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}], "-", "1"}], ")"}], " ", 
     TemplateBox[{RowBox[{
         FractionBox["3", "2"], "-", 
         RowBox[{"2", " ", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
      "Gamma"]}]], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      TemplateBox[{RowBox[{"-", 
          TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
       "Gamma"], "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"4", " ", 
        SuperscriptBox[
         TagBox["\[CurlyEpsilon]",
          TraditionalForm], "2"], " ", 
        TemplateBox[{RowBox[{
            RowBox[{"-", "3"}], " ", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
         "Gamma"], " ", 
        TemplateBox[{RowBox[{"-", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
         "Gamma"], " ", 
        TemplateBox[{TagBox["\[CurlyEpsilon]", TraditionalForm]},
         "Gamma"]}], "+", 
       RowBox[{"\[Pi]", " ", 
        RowBox[{"csc", "(", 
         RowBox[{"\[Pi]", " ", 
          TagBox["\[CurlyEpsilon]",
           TraditionalForm]}], ")"}], " ", 
        TemplateBox[{RowBox[{"1", "-", 
            RowBox[{"3", " ", 
              TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
         "Gamma"]}]}], ")"}]}], 
    RowBox[{"4", " ", 
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "2"], " ", 
     TemplateBox[{RowBox[{"1", "-", 
         RowBox[{"3", " ", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
      "Gamma"], " ", 
     TemplateBox[{RowBox[{
         RowBox[{"-", "3"}], " ", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"], " ", 
     TemplateBox[{TagBox["\[CurlyEpsilon]", TraditionalForm]},
      "Gamma"]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.501072091499978*^9, 3.501075729053212*^9, 
  3.50107969828841*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"rint", "/.", "\[InvisibleSpace]", 
  RowBox[{"Thread", "[", 
   RowBox[{"hy", "\[Rule]", "hypexp"}], "]"}]}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SuperscriptBox[
      TemplateBox[{RowBox[{"-", 
          TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
       "Gamma"], "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"4", " ", 
        SuperscriptBox[
         TagBox["\[CurlyEpsilon]",
          TraditionalForm], "2"], " ", 
        TemplateBox[{RowBox[{
            RowBox[{"-", "3"}], " ", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
         "Gamma"], " ", 
        TemplateBox[{RowBox[{"-", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
         "Gamma"], " ", 
        TemplateBox[{TagBox["\[CurlyEpsilon]", TraditionalForm]},
         "Gamma"]}], "+", 
       RowBox[{"\[Pi]", " ", 
        RowBox[{"csc", "(", 
         RowBox[{"\[Pi]", " ", 
          TagBox["\[CurlyEpsilon]",
           TraditionalForm]}], ")"}], " ", 
        TemplateBox[{RowBox[{"1", "-", 
            RowBox[{"3", " ", 
              TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
         "Gamma"]}]}], ")"}]}], 
    RowBox[{"4", " ", 
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "2"], " ", 
     TemplateBox[{RowBox[{"1", "-", 
         RowBox[{"3", " ", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
      "Gamma"], " ", 
     TemplateBox[{RowBox[{
         RowBox[{"-", "3"}], " ", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"], " ", 
     TemplateBox[{TagBox["\[CurlyEpsilon]", TraditionalForm]},
      "Gamma"]}]], "+", 
   FractionBox[
    RowBox[{
     TemplateBox[{RowBox[{"-", 
         TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
      "Gamma"], " ", 
     SuperscriptBox[
      TemplateBox[{RowBox[{"1", "-", 
          TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
       "Gamma"], "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FractionBox["1", "30"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"20", " ", 
           SuperscriptBox["\[Pi]", "2"], " ", 
           SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
             TraditionalForm], "2"], " ", 
           RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
          RowBox[{"120", " ", 
           SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
             TraditionalForm], "2"], " ", 
           RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
          RowBox[{"2", " ", 
           SuperscriptBox["\[Pi]", "4"], " ", 
           SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
             TraditionalForm], "2"]}], "+", 
          RowBox[{"35", " ", 
           SuperscriptBox["\[Pi]", "2"], " ", 
           SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
             TraditionalForm], "2"]}], "-", 
          RowBox[{"20", " ", 
           SuperscriptBox["\[Pi]", "2"], " ", 
           TagBox["\[CurlyEpsilon]",
            TraditionalForm]}], "-", 
          RowBox[{"480", " ", 
           SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
             TraditionalForm], "2"], " ", 
           TemplateBox[{"3"},
            "Zeta"]}], "+", 
          RowBox[{"120", " ", 
           TagBox["\[CurlyEpsilon]",
            TraditionalForm], " ", 
           TemplateBox[{"3"},
            "Zeta"]}], "+", 
          RowBox[{"5", " ", 
           SuperscriptBox["\[Pi]", "2"]}], "-", "30"}], ")"}]}], "+", "1"}], 
      ")"}]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       TagBox["\[CurlyEpsilon]",
        TraditionalForm], "-", "1"}], ")"}], " ", 
     TemplateBox[{RowBox[{"2", "-", 
         RowBox[{"3", " ", 
           TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
      "Gamma"]}]], "-", 
   RowBox[{
    FractionBox["1", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", " ", 
         TagBox["\[CurlyEpsilon]",
          TraditionalForm]}], "-", "1"}], ")"}], " ", 
      TemplateBox[{RowBox[{
          FractionBox["3", "2"], "-", 
          RowBox[{"2", " ", 
            TagBox["\[CurlyEpsilon]", TraditionalForm]}]}]},
       "Gamma"]}]], 
    SuperscriptBox["\[Pi]", 
     RowBox[{"3", "/", "2"}]], " ", 
    SuperscriptBox["2", 
     RowBox[{"4", " ", 
      TagBox["\[CurlyEpsilon]",
       TraditionalForm]}]], " ", 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm], " ", 
    RowBox[{"csc", "(", 
     RowBox[{"\[Pi]", " ", 
      TagBox["\[CurlyEpsilon]",
       TraditionalForm]}], ")"}], " ", 
    TemplateBox[{RowBox[{
        RowBox[{"-", "2"}], " ", 
        TagBox["\[CurlyEpsilon]", TraditionalForm]}]},
     "Gamma"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       FractionBox["1", "30"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"20", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          SuperscriptBox[
           TagBox["\[CurlyEpsilon]",
            TraditionalForm], "2"], " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
         RowBox[{"120", " ", 
          SuperscriptBox[
           TagBox["\[CurlyEpsilon]",
            TraditionalForm], "2"], " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
         RowBox[{"7", " ", 
          SuperscriptBox["\[Pi]", "4"], " ", 
          SuperscriptBox[
           TagBox["\[CurlyEpsilon]",
            TraditionalForm], "2"]}], "+", 
         RowBox[{"60", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          SuperscriptBox[
           TagBox["\[CurlyEpsilon]",
            TraditionalForm], "2"]}], "-", 
         RowBox[{"30", " ", 
          SuperscriptBox["\[Pi]", "2"], " ", 
          TagBox["\[CurlyEpsilon]",
           TraditionalForm]}], "-", 
         RowBox[{"1080", " ", 
          SuperscriptBox[
           TagBox["\[CurlyEpsilon]",
            TraditionalForm], "2"], " ", 
          TemplateBox[{"3"},
           "Zeta"]}], "+", 
         RowBox[{"180", " ", 
          TagBox["\[CurlyEpsilon]",
           TraditionalForm], " ", 
          TemplateBox[{"3"},
           "Zeta"]}], "+", 
         RowBox[{"5", " ", 
          SuperscriptBox["\[Pi]", "2"]}], "-", "30"}], ")"}]}], "+", "1"}], 
     ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50107209333585*^9, 3.501073126816475*^9, 
  3.501075729324624*^9, 3.5010796985854187`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Series2", "[", 
  RowBox[{"%", ",", "Epsilon", ",", "1"}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{"21", " ", 
     RowBox[{"\[Zeta]", "(", "2", ")"}]}], 
    RowBox[{"4", " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]}]], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["\[Pi]", "4"], " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]}], "160"], "-", 
   FractionBox["7", 
    RowBox[{"4", " ", 
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "3"]}]], "+", 
   RowBox[{"12", " ", 
    TemplateBox[{"3"},
     "Zeta"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50107209534587*^9, 3.50107312858092*^9, 
  3.5010757297802877`*^9, 3.501079699058621*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"total", "=", 
  RowBox[{"Expand", "[", 
   RowBox[{"Series2", "[", 
    RowBox[{
     RowBox[{"rint", "/.", "\[InvisibleSpace]", 
      RowBox[{"Thread", "[", 
       RowBox[{"hy", "\[Rule]", "hypexp"}], "]"}]}], ",", "Epsilon", ",", 
     "1"}], "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5010722039237213`*^9, 3.501072211868074*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{"21", " ", 
     RowBox[{"\[Zeta]", "(", "2", ")"}]}], 
    RowBox[{"4", " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]}]], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["\[Pi]", "4"], " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]}], "160"], "-", 
   FractionBox["7", 
    RowBox[{"4", " ", 
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "3"]}]], "+", 
   RowBox[{"12", " ", 
    TemplateBox[{"3"},
     "Zeta"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.501072212855509*^9, 3.501073129491898*^9, 
  3.501075730055265*^9, 3.501079699477878*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"epsdat", "=", 
   RowBox[{"N", "[", 
    RowBox[{
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"x", ",", 
         RowBox[{"rint", "/.", 
          RowBox[{"Epsilon", " ", "\[Rule]", " ", "x"}]}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", 
         RowBox[{"1", "/", "1000000"}], ",", 
         RowBox[{"1", "/", "100000"}], ",", 
         RowBox[{"1", "/", "1000000"}]}], "}"}]}], "]"}], ",", "32"}], 
    "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.501073106920843*^9, 3.50107310814318*^9}, 
   3.5010731551763887`*^9, {3.501073317639653*^9, 3.501073322318679*^9}, {
   3.501073369859943*^9, 3.501073375876134*^9}, {3.501073410201378*^9, 
   3.5010734241580887`*^9}, {3.501077299185699*^9, 3.501077300053425*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"49.05000000000018`", ",", 
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"1.0000000000000000000000000000000000000000000000000001`32.*^-6", 
        RowBox[{
        "-", "1.74999999999136408172436458215894738244404264105624214959`32.*^\
18"}]},
       {"2.`32.*^-6", 
        RowBox[{
        "-", "2.1874999999568203364984178553772731103391396431604868884`32.*^\
17"}]},
       {"3.0000000000000000000000000000000000000000000000000001`32.*^-6", 
        RowBox[{
        "-", "6.4814814811936165773149407584977384633873591521801472`32.*^16"}\
]},
       {"4.`32.*^-6", 
        RowBox[{
        "-", "2.734374999784100961258130105412440854335698232156433256`32.*^\
16"}]},
       {"5.0000000000000000000000000000000000000000000000000001`32.*^-6", 
        RowBox[{
        "-", "1.399999999827280480512956982918804103837752951309394863`32.*^\
16"}]},
       {"6.`32.*^-6", 
        RowBox[{
        "-", "8.10185185041252011868047076039786034590505252617323893`32.*^\
15"}]},
       {"7.0000000000000000000000000000000000000000000000000001`32.*^-6", 
        RowBox[{
        "-", "5.1020408150928156374301547427509048239187429049639057`32.*^15"}\
]},
       {"8.`32.*^-6", 
        RowBox[{
        "-", "3.41796874892049759395288769993677229213860469864086964`32.*^\
15"}]},
       {"9.0000000000000000000000000000000000000000000000000001`32.*^-6", 
        RowBox[{
        "-", "2.40054869588543384428395160210297256220973241055414365`32.*^\
15"}]},
       {"1.`32.*^-5", 
        RowBox[{
        "-", "1.74999999913639519022793714809334358205805327513554271`32.*^\
15"}]}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{
  3.5010722194032583`*^9, {3.501073108988888*^9, 3.5010731587986917`*^9}, 
   3.501073324377701*^9, 3.501073407419977*^9, 3.501076007765896*^9, 
   3.5010773010546227`*^9, 3.501079748685985*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Fit", "[", 
  RowBox[{"epsdat", ",", 
   RowBox[{"{", 
    RowBox[{"\[Epsilon]", ",", "1", ",", 
     RowBox[{"1", "/", "\[Epsilon]"}], ",", 
     RowBox[{"1", "/", 
      RowBox[{"\[Epsilon]", "^", "3"}]}]}], "}"}], ",", "\[Epsilon]"}], 
  "]"}]], "Input",
 CellChangeTimes->{
  3.501073205321957*^9, {3.501073245949606*^9, 3.50107324808694*^9}, {
   3.5010772767056932`*^9, 3.501077283881826*^9}, 3.5010804316385098`*^9}],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     "1.7499999999999999999999999963334505722861627562977303357609`30.\
309746329822957", 
     SuperscriptBox["\[Epsilon]", "3"]]}], "-", 
   RowBox[{
   "0.60980681771855910554652909533696758311`7.183238865242681", " ", 
    "\[Epsilon]"}], "+", 
   FractionBox["8.63590385095318099046868924665798238387`19.024435657258707", 
    "\[Epsilon]"], "+", 
   "14.42468284299939111135635708100131204821`13.650135537374583"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.501072240273787*^9, 3.501073163842142*^9, 
  3.501073206012289*^9, 3.501073248699099*^9, 3.501073326678132*^9, 
  3.501076008191772*^9, 3.501077307325715*^9, 3.5010797491046057`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"N", "[", 
  RowBox[{"total", ",", "15"}], "]"}]], "Input",
 CellChangeTimes->{{3.50107318319795*^9, 3.501073184178829*^9}, {
  3.5010771721517153`*^9, 3.5010771772064447`*^9}, {3.501077313881576*^9, 
  3.5010773857635107`*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"0.60880681896251523272775207930440694543`15.", " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]}]}], "+", 
   FractionBox["8.63590385095318879148017962489163224345`15.", 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm]], "-", 
   FractionBox["1.75`15.", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "3"]], "+", 
   "14.42468283791513141808737150099537144638`15."}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{
  3.501072243809293*^9, {3.501073177748589*^9, 3.501073184469542*^9}, 
   3.501073335221212*^9, 3.501076008273677*^9, 3.5010771774781837`*^9, {
   3.501077314150099*^9, 3.5010773863515663`*^9}, 3.501079749186626*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["total"], "Input",
 CellChangeTimes->{{3.501073186623065*^9, 3.50107318702059*^9}}],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{"21", " ", 
     RowBox[{"\[Zeta]", "(", "2", ")"}]}], 
    RowBox[{"4", " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]}]], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["\[Pi]", "4"], " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]}], "160"], "-", 
   FractionBox["7", 
    RowBox[{"4", " ", 
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "3"]}]], "+", 
   RowBox[{"12", " ", 
    TemplateBox[{"3"},
     "Zeta"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.501073187280616*^9, 3.501076008356703*^9, 
  3.501079749272987*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TimeUsed", "[", "]"}]], "Input"],

Cell[BoxData[
 FormBox["2382.87`", TraditionalForm]], "Output",
 CellChangeTimes->{3.5010760084375467`*^9, 3.501079749347176*^9}]
}, Open  ]]
}, Open  ]]
},
WindowSize->{1313, 831},
WindowMargins->{{38, Automatic}, {21, Automatic}},
PrintingPageRange->{Automatic, Automatic},
PrintingOptions->{"Magnification"->1,
"PaperOrientation"->"Portrait",
"PaperSize"->{597.562, 842.375},
"PostScriptOutputFile":>FrontEnd`FileName[{$RootDirectory, "home", "rolfm"}, 
  "denksport3.ps", CharacterEncoding -> "iso8859-1"]},
ShowSelection->True,
Magnification->1.5,
FrontEndVersion->"8.0 for Linux x86 (64-bit) (November 7, 2010)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "Info3501081023-3542981"->{
  Cell[12356, 418, 517, 8, 226, "Print",
   CellTags->"Info3501081023-3542981"]},
 "Info3501081067-3542981"->{
  Cell[39922, 1281, 342, 6, 116, "Print",
   CellTags->"Info3501081067-3542981"]},
 "Info3501081089-3542981"->{
  Cell[49618, 1606, 414, 7, 170, "Print",
   CellTags->"Info3501081089-3542981"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"Info3501081023-3542981", 95202, 3045},
 {"Info3501081067-3542981", 95313, 3048},
 {"Info3501081089-3542981", 95425, 3051}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 130, 5, 106, "Section"],
Cell[CellGroupData[{
Cell[722, 31, 100, 1, 43, "Input"],
Cell[825, 34, 155, 3, 43, "Output"]
}, Open  ]],
Cell[995, 40, 71, 1, 43, "Input"],
Cell[1069, 43, 207, 4, 22, "Text"],
Cell[1279, 49, 676, 22, 24, "Text"],
Cell[1958, 73, 373, 12, 22, "Text"],
Cell[2334, 87, 196, 4, 22, "Text"],
Cell[2533, 93, 515, 9, 176, "Text"],
Cell[CellGroupData[{
Cell[3073, 106, 1247, 37, 88, "Input"],
Cell[4323, 145, 1660, 50, 94, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[6020, 200, 141, 4, 43, "Input"],
Cell[6164, 206, 1124, 38, 94, "Output"]
}, Open  ]],
Cell[7303, 247, 231, 10, 42, "Text"],
Cell[7537, 259, 747, 20, 71, "Input"],
Cell[CellGroupData[{
Cell[8309, 283, 175, 5, 67, "Input"],
Cell[8487, 290, 1777, 54, 96, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[10301, 349, 96, 2, 43, "Input"],
Cell[10400, 353, 1860, 57, 96, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[12297, 415, 56, 1, 43, "Input"],
Cell[12356, 418, 517, 8, 226, "Print",
 CellTags->"Info3501081023-3542981"]
}, Open  ]],
Cell[CellGroupData[{
Cell[12910, 431, 359, 10, 67, "Input"],
Cell[13272, 443, 9827, 300, 366, "Output"]
}, Open  ]],
Cell[23114, 746, 44, 0, 42, "Text"],
Cell[CellGroupData[{
Cell[23183, 750, 505, 13, 67, "Input"],
Cell[23691, 765, 8029, 248, 250, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[31757, 1018, 263, 5, 43, "Input"],
Cell[32023, 1025, 4875, 157, 43, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[36935, 1187, 122, 3, 43, "Input"],
Cell[37060, 1192, 2763, 81, 85, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[39860, 1278, 59, 1, 43, "Input"],
Cell[39922, 1281, 342, 6, 116, "Print",
 CellTags->"Info3501081067-3542981"]
}, Open  ]],
Cell[CellGroupData[{
Cell[40301, 1292, 216, 7, 77, "Input"],
Cell[40520, 1301, 4823, 156, 43, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[45380, 1462, 126, 3, 43, "Input"],
Cell[45509, 1467, 240, 5, 43, "Output"]
}, Open  ]],
Cell[45764, 1475, 195, 8, 44, "Text"],
Cell[CellGroupData[{
Cell[45984, 1487, 431, 11, 43, "Input"],
Cell[46418, 1500, 3110, 98, 162, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[49565, 1603, 50, 1, 43, "Input"],
Cell[49618, 1606, 414, 7, 170, "Print",
 CellTags->"Info3501081089-3542981"]
}, Open  ]],
Cell[CellGroupData[{
Cell[50069, 1618, 412, 12, 43, "Input"],
Cell[50484, 1632, 3408, 96, 138, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[53929, 1733, 240, 6, 43, "Input"],
Cell[54172, 1741, 304, 8, 50, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[54513, 1754, 239, 6, 43, "Input"],
Cell[54755, 1762, 277, 8, 50, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[55069, 1775, 291, 7, 43, "Input"],
Cell[55363, 1784, 284, 7, 50, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[55684, 1796, 28, 0, 43, "Input"],
Cell[55715, 1798, 4777, 156, 43, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[60529, 1959, 32, 0, 43, "Input"],
Cell[60564, 1961, 3092, 93, 138, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[63693, 2059, 129, 3, 43, "Input"],
Cell[63825, 2064, 7979, 246, 209, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[71841, 2315, 30, 0, 43, "Input"],
Cell[71874, 2317, 7865, 246, 250, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[79776, 2568, 143, 3, 43, "Input"],
Cell[79922, 2573, 6237, 185, 324, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[86196, 2763, 99, 2, 43, "Input"],
Cell[86298, 2767, 687, 23, 81, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[87022, 2795, 364, 9, 43, "Input"],
Cell[87389, 2806, 687, 23, 81, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[88113, 2834, 824, 20, 43, "Input"],
Cell[88940, 2856, 2338, 60, 317, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[91315, 2921, 447, 11, 43, "Input"],
Cell[91765, 2934, 726, 17, 77, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[92528, 2956, 251, 5, 43, "Input"],
Cell[92782, 2963, 745, 19, 77, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[93564, 2987, 96, 1, 43, "Input"],
Cell[93663, 2990, 665, 23, 81, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[94365, 3018, 56, 1, 43, "Input"],
Cell[94424, 3021, 129, 2, 43, "Output"]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
