(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[    120410,       3071]
NotebookOptionsPosition[    116163,       2927]
NotebookOutlinePosition[    116807,       2953]
CellTagsIndexPosition[    116720,       2948]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["The 2-loop ghost selfenergy ", "Subsection"],

Cell[BoxData[
 RowBox[{"<<", "HighEnergyPhysics`FeynCalc`"}]], "Input"],

Cell["Loading FeynCalc from /home/rolfm/HighEnergyPhysics", "Text",
 CellMargins->{{Inherited, Inherited}, {1, 0}},
 GeneratedCell->True,
 CellAutoOverwrite->True],

Cell[TextData[{
 StyleBox["FeynCalc",
  FontWeight->"Bold"],
 " ",
 "8.1.0",
 " For help, type ?FeynCalc, open ",
 ButtonBox["FeynCalcRef8.nb",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookOpen[
   "/home/rolfm/HighEnergyPhysics/Documentation/English/FeynCalcRef8.nb"],
  ButtonData:>{"Short Overview", "intro"},
  ButtonNote->"Open FeynCalcRef8.nb"],
 " or visit ",
 ButtonBox["www.feyncalc.org",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.feyncalc.org/"], None},
  ButtonNote->"http://www.feyncalc.org/"]
}], "Text",
 CellMargins->{{Inherited, Inherited}, {1, 0}},
 GeneratedCell->True,
 CellAutoOverwrite->True],

Cell[TextData[{
 "Loading FeynArts, see ",
 ButtonBox["www.feynarts.de",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.feynarts.de/"], None},
  ButtonNote->"http://www.feynarts.de/"],
 " for documentation"
}], "Text",
 CellMargins->{{Inherited, Inherited}, {1, 0}},
 GeneratedCell->True,
 CellAutoOverwrite->True],

Cell["FeynArts 3.4 patched for use with FeynCalc", "Text",
 CellMargins->{{Inherited, Inherited}, {1, 0}},
 GeneratedCell->True,
 CellAutoOverwrite->True],

Cell[CellGroupData[{

Cell["The graphs", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Paint", "[", 
   RowBox[{
    RowBox[{"inserts", "=", 
     RowBox[{"InsertFields", "[", 
      RowBox[{
       RowBox[{"Rest", "@", 
        RowBox[{"CreateTopologies", "[", 
         RowBox[{"2", ",", 
          RowBox[{"1", "\[Rule]", "1"}], ",", 
          RowBox[{"ExcludeTopologies", "\[Rule]", 
           RowBox[{"{", "Tadpoles", "}"}]}]}], "]"}]}], ",", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"U", "[", "5", "]"}], "}"}], "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"U", "[", "5", "]"}], "}"}]}], ",", 
       RowBox[{"InsertionLevel", "\[Rule]", "Classes"}], ",", 
       RowBox[{"GenericModel", "\[Rule]", "\"\<FCQCDLorentz\>\""}], ",", 
       RowBox[{"Model", "\[Rule]", "\"\<FCQCD\>\""}]}], "]"}]}], ",", 
    RowBox[{"ColumnsXRows", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"4", ",", "2"}], "}"}]}], ",", 
    RowBox[{"SheetHeader", "\[Rule]", "False"}], ",", 
    RowBox[{"PaintLevel", "\[Rule]", 
     RowBox[{"{", "Generic", "}"}]}], ",", 
    RowBox[{"Numbering", "\[Rule]", "None"}]}], "]"}], ";"}]], "Input"],

Cell[BoxData[
 FormBox[
  GraphicsBox[{InsetBox[
     GraphicsBox[
      {Thickness[0.005], 
       {Dashing[{0.005, 0.030000000000000002`}], 
        LineBox[{{-1.0009326700810561`*^-11, 10.}, {5.999999999982448, 
         10.}}]}, PolygonBox[{{3.6, 10.}, {2.4, 9.6}, {2.4, 10.4}}], 
       InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {3., 9.0548}, {0, 1}], 
       {Dashing[{0.005, 0.030000000000000002`}], 
        LineBox[{{20.00000000000066, 10.}, {13.999999999999321`, 10.}}]}, 
       PolygonBox[{{17.599999999999994`, 10.}, {16.39999999999999, 10.4}, {
        16.39999999999999, 9.6}}], InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {16.999999999999993`, 10.9452}, {0, -1}], 
       {Dashing[{0.005, 0.030000000000000002`}], 
        CircleBox[{9.864573208290214, 9.864573207387709}, 3.866945370494878, {
         3.1065638443429586`, 4.747417796295682}]}, InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {6.81579111096687, 6.815791085716318}, \
{1, 1}], LineBox[CompressedData["
1:eJwdVXk01AsbnqSvUuMrJTNmMTOa5ff7TSIVhupJK1nCbVHJUulmX6KU5Sbp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        "]], InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {13.316594227670251`, 6.68340579846254}, \
{-1, 1}], 
       {Dashing[{0.005, 0.030000000000000002`}], 
        LineBox[{{9.99999997334453, 14.000000000000902`}, {10.00000002665547, 
         6.000000000000003}}]}, InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {10.6952, 10.0000000086634}, {-1, 0}], 
       LineBox[CompressedData["
1:eJwlkgs01YkaxSWGQolyDv/jvDjn/M//KERJop3UKDeRVyFFCSnv1I0oDTGE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        "]], InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {6.657315156232203, 13.342684817461137`}, \
{1, -1}], 
       {Dashing[{0.005, 0.030000000000000002`}], 
        CircleBox[{9.989559209705002, 9.989559209635424}, 4.010454381060245, {
         0.002603396315374207, 1.5681929371436802`}]}, InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {13.13981750122236, 13.139817526177083`}, \
{-1, -1}], 
       {PointSize[0.04], PointBox[{9.99999997334337, 13.99999999999999}], 
        PointBox[{10.00000002665663, 6.}], PointBox[{6., 10.}], 
        PointBox[{13.99999999999999, 10.}]}},
      AspectRatio->1,
      PlotRange->{{-1, 21}, {-1, 21}}], {0, 22}, {0, 0}, {22, 22}], InsetBox[
     GraphicsBox[
      {Thickness[0.005], 
       {Dashing[{0.005, 0.030000000000000002`}], 
        LineBox[{{-1.0009326700810561`*^-11, 10.}, {5.999999999982448, 
         10.}}]}, PolygonBox[{{3.6, 10.}, {2.4, 9.6}, {2.4, 10.4}}], 
       InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {3., 9.0548}, {0, 1}], 
       {Dashing[{0.005, 0.030000000000000002`}], 
        LineBox[{{20.00000000000066, 10.}, {13.999999999999321`, 10.}}]}, 
       PolygonBox[{{17.599999999999994`, 10.}, {16.39999999999999, 10.4}, {
        16.39999999999999, 9.6}}], InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {16.999999999999993`, 10.9452}, {0, -1}], 
       {Dashing[{0.005, 0.030000000000000002`}], 
        CircleBox[{9.864573208290214, 9.864573207387709}, 3.866945370494878, {
         3.1065638443429586`, 4.747417796295682}]}, InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {6.81579111096687, 6.815791085716318}, \
{1, 1}], 
       {Dashing[{0.005, 0.030000000000000002`}], 
        CircleBox[{9.989559228732924, 10.010440771197487`}, 
         4.0104543619824895`, {4.714992378597311, 6.280581915631125}]}, 
       InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {13.13981753178458, 6.860182493170143}, \
{-1, 1}], LineBox[CompressedData["
1:eJwd1nk8VWkfAPBTyC2qq/R2MYRzjoRJO1r0aCQhKdst3hDG0ggpXTVTlkZI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        "]], InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {11.645199999999999`, 10.00000001499435}, \
{-1, 0}], LineBox[CompressedData["
1:eJwlkgs01YkaxSWGQolyDv/jvDjn/M//KERJop3UKDeRVyFFCSnv1I0oDTGE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        "]], InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {6.657315156232203, 13.342684817461137`}, \
{1, -1}], LineBox[CompressedData["
1:eJwtlQlQk9cahiMXLVSkImVRkEJJ/jUKSrWISD8qXkS4rMUqglqVRZag9Gol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        "]], InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {13.316594195929966`, 13.316594222062754`}, \
{-1, -1}], 
       {PointSize[0.04], PointBox[{9.99999997334337, 13.99999999999999}], 
        PointBox[{10.00000002665663, 6.}], PointBox[{6., 10.}], 
        PointBox[{13.99999999999999, 10.}]}},
      AspectRatio->1,
      PlotRange->{{-1, 21}, {-1, 21}}], {22, 22}, {0, 0}, {22, 22}], 
    InsetBox[
     GraphicsBox[
      {Thickness[0.005], 
       {Dashing[{0.005, 0.030000000000000002`}], 
        LineBox[{{-1.0009326700810561`*^-11, 10.}, {5.999999999982448, 
         10.}}]}, PolygonBox[{{3.6, 10.}, {2.4, 9.6}, {2.4, 10.4}}], 
       InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {3., 9.0548}, {0, 1}], 
       {Dashing[{0.005, 0.030000000000000002`}], 
        LineBox[{{20.00000000000066, 10.}, {13.999999999999321`, 10.}}]}, 
       PolygonBox[{{17.599999999999994`, 10.}, {16.39999999999999, 10.4}, {
        16.39999999999999, 9.6}}], InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {16.999999999999993`, 9.0548}, {0, 1}], 
       {Dashing[{0.005, 0.030000000000000002`}], 
        CircleBox[{9.999999999999995, 10.}, 3.9999999999999947`, {
         3.141592653589793, 6.283185307179586}]}, InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {9.999999999999995, 5.3048000000000055`}, \
{0, 1}], LineBox[CompressedData["
1:eJwtVHsw1QkYRSpSS6T7+P3u9eje34MGq9hNto4IPSTKq2JLi9SaktJLyWVR
iM2jRsir9JQV8tjahMGgvPbWXpMoeWxIJYS0d7f9Zr45c/74vvnOOTOfnvd+
Zx8lBQUFV3n/i1+LC8X/UISMno3Cu9EcDOYHm2tbilA9zA0ZMOHAA5MdXZEi
PGh/9sJRgQODsiKT/g4RMuskPgemFkNA7quzNBMjZda6gXf6HDjuDr+4JlaM
E90fig8HcVDcpW2ULhOj2dPmqW6vfL67rVODoHBNx2vdjUAuygvuBTPrKYwM
PD6frcbDHcuwvDBfCknBg97Z13lIWrbubfkBCqfnZUaJ1vCRGrHjgdY+Cvqy
cHq8kY+Wwvlmz90pnFLYf8fVmoBHa2XG8x8o3Fo9U9VxmcB99QjLGgEFr+Ok
ul8ngcbLI8sbpsQwySqqFU8TsPxLOyVUfp+yNLGocJzAMo8X79sqxJjwyJHZ
tBAw21OstTlLjDIbn3TpWQKDLV4hnnJ9KYc2KJszBFYk5lmcOyWG7btP2ybz
+diStk+1+pgYTpkBK3dTfMz+w75SdFqMRTf8ti+5wMOo8YSvNFGMcqX2s+py
vfEuREZsiRi0s7F1ZiwXzHcdR7/0i/FzkR21nOLiMfs0JsCAwp5StCzs5sC0
YozcdJRCr2dCok01BxW+sk6nxxSKrFOb4to44IRJ7R9SNKZTZVfvfsMF5eet
ypygUQLdaH+53wPcFrFVDQ2brvBzfR+4aKoz7X7yhYbWVIu2ZiwPiu5vplxp
BvfSH+n2GfEhksTkBVswmHa4KZO28GES0btraAUDw2MRpr/vI/Cofa2tHcPg
9MfcfqePBGJir591U2Ww2t2dMPInofwj5ZjWTeOkhHEOrCLh8knCXi+mETyw
sWe3ogB/R6vF3j5Dw7gvT7hcT4BRu66rol00poL4Hm/lfKphh1PxKhoWfS6b
7s4SoK15pBYiGrRq1cOZWhK3ZpeOamvRqK4MaPoSSGKprcqE2XwamS6u7lVz
SGhpvhlO0KARETDjYxZHIOu3IwX6ejRU/Ka3DC0g8FI9JfqFfL/7Idai5jwf
0SZjS/fvpcENygiykOd3zaYiqjqHxrycHtPkVh4abf3H8wdo8Dh/5jzP5CFK
w4h7Ru5HMxFWWirPt3fHyGsqnoHqAadBr3IeNuuHPOe8ZpB02Fo6PJePvVRn
60ozFg81Iq3cTvBxut3mXMIxFmOGymtH1QioVuikqt1mEblK6lafL/czoaxw
UyOLmOTgV5ruJOqNdgZqSFmcXzjk0DxN4soqKwnbwCLFdoH5cKIAwweGghzz
WXAHYy92LBaitP5o2oVwFpIJx5qIMCGMGod5Lx1Z+KVNX5K1CKH+3mGNPZfF
QSV2GzVXBx7763/J7GaQ2MXbFqCng/WykF16BQw8Bs2UOHJuf81ljI5iUPDT
g+SBOTpQSekZX+LPwIsq6HduF+LR0DvNPDcGtW9uCGUxQsyxjnsZv4XB5y17
38Z9K4QkS6Jv5ckg3+vmfbsGASpzzWOaDzHo95feur5TgLSRIn9eKoOLhccX
NU2SuPH00oq8RgbFDYoa2TkkNDorrh5XYxE16d2g4kPCpEGJV7FVrl94Ulxi
R6LuifPTg1dY7JxJ3z7LmUSuQ7bmkU8sWh3Wfy+LJBEaqqebvMEAW429Md1D
Ism/LFctxQAlL221u3wFGCk0l6xqk/PP+lVrZwmhmJ7sFjpjgMSKhm8O5wsR
f9d1unyRIcJ/fZbet10Hpr0a1Cs5//pPdf9HQ/wDhJ5AaQ==
        "]], InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {5.706292966747745, 12.23006116185843}, \
{1, -1}], 
       CircleBox[{9.999999999999995, 13.049999999999999`}, 
        2.049999999999995, {0.2213144423477913, 2.9202782112420023`}], 
       InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {9.999999999999995, 15.795199999999994`}, \
{0, -1}], 
       CircleBox[{9.999999999999995, 13.950000000000001`}, 
        2.049999999999995, {3.3629070959375844`, 6.061870864831795}], InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {9.999999999999995, 11.204800000000002`}, \
{0, 1}], LineBox[CompressedData["
1:eJwtVAsw1XkUFrpMkYRK5Qr3//j9/9e2rlddXJ8mZBWxS9Fz26yoSbV5lFeL
uow0WGXX0vagh9dV6EE1ovKoLVcK1arIK2VKoe3B3po9M2fOnDPf+b45M2c+
s41hvkHqampq/qr8Uj9MfAkeal9DBMmO9eu1HcR4npbkmecogl/gAj3bn8XQ
NcttvigXoc1Ds7wmVgyfgUb1+49FSN71IDcyXowcwXCNth0FzTsVpYZbxCA7
b5TWHaBgt0ImW7FYjNvhr1defEihIdW8K0dbjMnTxU8uzaExZYHb9uxaHkFx
jjfWedCYceikZHc4j3PrDRs8g2hMSn2Xt43mMfX+vWtmYTT2jRaU2D7k0NEV
rGkaQkO2K7esO4tDWOu+Dxn+NEYs2hPaAjj0lOzJlTnQGGLixq/xHKpnHlK2
qvTo65/zFToc3vTZuBx8T6FIa7mrYJwgc/JBX7U2Ch5LLUp2qXGgxhVPLS5S
6BF2pT8y4KDsHnVPzqMQWEHrS+05vOp98Co3mcLdv9riCzZzOO9x33p0D4XE
4J5YrwIOlyXy1b9EUGicsfVCxgCH9IzOIkkshYiXP8HAiofTorTvvDIotMy/
4dQfwWPrwOeYLRUUBmh3aVslD5+6TnWbPgp4a7/mQj+PYoewfx+yNKbqe94d
miJGlI5etDKCxnM5SVU3FqNq7brLDbdpPBPe0p+rLwbzcVCiRzHwThGUB77l
8SZ3yNJ5N4P2ZB+d/ms8MhOyh0zqGLB/SBXH9/KQzVjmGv2ZwZH97osybHiQ
rG7XZyIWOR6HpZbdHJYfGp0zbM/ik8tk70sZHPTWWLuHqPouowvzbi7h8OJM
W8xtioUkOq96bIKg+WOdSZ9Ahf9BUCW4SbB5X+HYgicMroSrPej9nUBZNtvb
tpxBFF1cPLqboDOl3l4kZ+DYnnfONJRgWe9VZdQ6BonlL9I7QghKV60a/seB
QWHnmjkzVHgurXmSgxmDDcHXxiZnE9wqqmnv0WOwJ24sck4tQbtCI6tHm8Hh
iqUZN8cIWhcH1lvqqvbtp4t+s+FQudBYVmXCwPCss7ntLg5JJmxhtIrfqeTR
1NqzHKYcW5ljEMwgxymweKJP9T8nOi22HWXwuET04q4+j46gK8F+vQxiurW4
So5H/pvMDeZ2LAztRHYbF/A47mxtXJzKYt3q9G5HIQ86cyA85RmLI6YJzuVj
HHr9Zi7MtyKQPp5J3K5yaApjmjQiCFze/arpGMFh9lENNv8MwbO3Uj8BxSHR
uWgF20hwcMkHm5hmgk2Wc+MW3yOIFprOmx1HYNCSH1nbQOB93s1xk4TgnsWf
8b7FBCf9yxSK1yy2xtfLP/1KkLZe4KhXzUKa2qyR7UWADX/rZGey2Om840eb
WQSyEOOppZEsbIk5Tj1lMVif1hEQyqIsydybUrAYykpoClX1LkbCE7r7WWgx
CdmxKnzA+xS31mAWid+HWrSns2iveair78/C6+w8F0mliqdhXHHGh8X1l/lO
e7tYLJHpd2utZmEvHMxrMSIYjjS2dN+pmnctrFvpSdBl8K7DLZvF2juhV5NU
980L0TuV1chCZqs3samQYD7fky3RJhgvuJ6j00RgWPtE6eFDUON6fruo48vd
UZr9xwjY6v7Yva0Ec50ypvWPEFzh23ROVxEcHdlUKlzK4VinXGh1gGCWkeGg
MpODdHOMbotK3yL3bnRoMwf3mIAA43EW05QVTxZ94hDk9u0t5WkWI4k5ctqA
Bz9T/o2VLwtry4mnRoY8Pn71U+Z/P+XxHwXnOss=
        "]], InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {14.283684782537893`, 12.224334161450233`}, \
{-1, -1}], 
       {PointSize[0.04], PointBox[{11.99999999999999, 13.5}], 
        PointBox[{8., 13.5}], PointBox[{6., 10.}], 
        PointBox[{13.99999999999999, 10.}]}},
      AspectRatio->1,
      PlotRange->{{-1, 21}, {-1, 21}}], {44, 22}, {0, 0}, {22, 22}], 
    InsetBox[
     GraphicsBox[
      {Thickness[0.005], 
       {Dashing[{0.005, 0.030000000000000002`}], 
        LineBox[{{-1.0009326700810561`*^-11, 10.}, {5.999999999982448, 
         10.}}]}, PolygonBox[{{3.6, 10.}, {2.4, 9.6}, {2.4, 10.4}}], 
       InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {3., 9.0548}, {0, 1}], 
       {Dashing[{0.005, 0.030000000000000002`}], 
        LineBox[{{20.00000000000066, 10.}, {13.999999999999321`, 10.}}]}, 
       PolygonBox[{{17.599999999999994`, 10.}, {16.39999999999999, 10.4}, {
        16.39999999999999, 9.6}}], InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {16.999999999999993`, 9.0548}, {0, 1}], 
       {Dashing[{0.005, 0.030000000000000002`}], 
        CircleBox[{9.999999999999995, 10.}, 3.9999999999999947`, {
         3.141592653589793, 6.283185307179586}]}, InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {9.999999999999995, 5.3048000000000055`}, \
{0, 1}], LineBox[CompressedData["
1:eJwtVHsw1QkYRSpSS6T7+P3u9eje34MGq9hNto4IPSTKq2JLi9SaktJLyWVR
iM2jRsir9JQV8tjahMGgvPbWXpMoeWxIJYS0d7f9Zr45c/74vvnOOTOfnvd+
Zx8lBQUFV3n/i1+LC8X/UISMno3Cu9EcDOYHm2tbilA9zA0ZMOHAA5MdXZEi
PGh/9sJRgQODsiKT/g4RMuskPgemFkNA7quzNBMjZda6gXf6HDjuDr+4JlaM
E90fig8HcVDcpW2ULhOj2dPmqW6vfL67rVODoHBNx2vdjUAuygvuBTPrKYwM
PD6frcbDHcuwvDBfCknBg97Z13lIWrbubfkBCqfnZUaJ1vCRGrHjgdY+Cvqy
cHq8kY+Wwvlmz90pnFLYf8fVmoBHa2XG8x8o3Fo9U9VxmcB99QjLGgEFr+Ok
ul8ngcbLI8sbpsQwySqqFU8TsPxLOyVUfp+yNLGocJzAMo8X79sqxJjwyJHZ
tBAw21OstTlLjDIbn3TpWQKDLV4hnnJ9KYc2KJszBFYk5lmcOyWG7btP2ybz
+diStk+1+pgYTpkBK3dTfMz+w75SdFqMRTf8ti+5wMOo8YSvNFGMcqX2s+py
vfEuREZsiRi0s7F1ZiwXzHcdR7/0i/FzkR21nOLiMfs0JsCAwp5StCzs5sC0
YozcdJRCr2dCok01BxW+sk6nxxSKrFOb4to44IRJ7R9SNKZTZVfvfsMF5eet
ypygUQLdaH+53wPcFrFVDQ2brvBzfR+4aKoz7X7yhYbWVIu2ZiwPiu5vplxp
BvfSH+n2GfEhksTkBVswmHa4KZO28GES0btraAUDw2MRpr/vI/Cofa2tHcPg
9MfcfqePBGJir591U2Ww2t2dMPInofwj5ZjWTeOkhHEOrCLh8knCXi+mETyw
sWe3ogB/R6vF3j5Dw7gvT7hcT4BRu66rol00poL4Hm/lfKphh1PxKhoWfS6b
7s4SoK15pBYiGrRq1cOZWhK3ZpeOamvRqK4MaPoSSGKprcqE2XwamS6u7lVz
SGhpvhlO0KARETDjYxZHIOu3IwX6ejRU/Ka3DC0g8FI9JfqFfL/7Idai5jwf
0SZjS/fvpcENygiykOd3zaYiqjqHxrycHtPkVh4abf3H8wdo8Dh/5jzP5CFK
w4h7Ru5HMxFWWirPt3fHyGsqnoHqAadBr3IeNuuHPOe8ZpB02Fo6PJePvVRn
60ozFg81Iq3cTvBxut3mXMIxFmOGymtH1QioVuikqt1mEblK6lafL/czoaxw
UyOLmOTgV5ruJOqNdgZqSFmcXzjk0DxN4soqKwnbwCLFdoH5cKIAwweGghzz
WXAHYy92LBaitP5o2oVwFpIJx5qIMCGMGod5Lx1Z+KVNX5K1CKH+3mGNPZfF
QSV2GzVXBx7763/J7GaQ2MXbFqCng/WykF16BQw8Bs2UOHJuf81ljI5iUPDT
g+SBOTpQSekZX+LPwIsq6HduF+LR0DvNPDcGtW9uCGUxQsyxjnsZv4XB5y17
38Z9K4QkS6Jv5ckg3+vmfbsGASpzzWOaDzHo95feur5TgLSRIn9eKoOLhccX
NU2SuPH00oq8RgbFDYoa2TkkNDorrh5XYxE16d2g4kPCpEGJV7FVrl94Ulxi
R6LuifPTg1dY7JxJ3z7LmUSuQ7bmkU8sWh3Wfy+LJBEaqqebvMEAW429Md1D
Ism/LFctxQAlL221u3wFGCk0l6xqk/PP+lVrZwmhmJ7sFjpjgMSKhm8O5wsR
f9d1unyRIcJ/fZbet10Hpr0a1Cs5//pPdf9HQ/wDhJ5AaQ==
        "]], InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {5.706292966747745, 12.23006116185843}, \
{1, -1}], 
       {Dashing[{0.005, 0.030000000000000002`}], 
        CircleBox[{9.999999999999995, 13.049999999999999`}, 
         2.049999999999995, {0.2213144423477913, 2.9202782112420023`}]}, 
       InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {9.999999999999995, 15.795199999999994`}, \
{0, -1}], 
       {Dashing[{0.005, 0.030000000000000002`}], 
        CircleBox[{9.999999999999995, 13.950000000000001`}, 
         2.049999999999995, {3.3629070959375844`, 6.061870864831795}]}, 
       InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {9.999999999999995, 11.204800000000002`}, \
{0, 1}], LineBox[CompressedData["
1:eJwtVAsw1XkUFrpMkYRK5Qr3//j9/9e2rlddXJ8mZBWxS9Fz26yoSbV5lFeL
uow0WGXX0vagh9dV6EE1ovKoLVcK1arIK2VKoe3B3po9M2fOnDPf+b45M2c+
s41hvkHqampq/qr8Uj9MfAkeal9DBMmO9eu1HcR4npbkmecogl/gAj3bn8XQ
NcttvigXoc1Ds7wmVgyfgUb1+49FSN71IDcyXowcwXCNth0FzTsVpYZbxCA7
b5TWHaBgt0ImW7FYjNvhr1defEihIdW8K0dbjMnTxU8uzaExZYHb9uxaHkFx
jjfWedCYceikZHc4j3PrDRs8g2hMSn2Xt43mMfX+vWtmYTT2jRaU2D7k0NEV
rGkaQkO2K7esO4tDWOu+Dxn+NEYs2hPaAjj0lOzJlTnQGGLixq/xHKpnHlK2
qvTo65/zFToc3vTZuBx8T6FIa7mrYJwgc/JBX7U2Ch5LLUp2qXGgxhVPLS5S
6BF2pT8y4KDsHnVPzqMQWEHrS+05vOp98Co3mcLdv9riCzZzOO9x33p0D4XE
4J5YrwIOlyXy1b9EUGicsfVCxgCH9IzOIkkshYiXP8HAiofTorTvvDIotMy/
4dQfwWPrwOeYLRUUBmh3aVslD5+6TnWbPgp4a7/mQj+PYoewfx+yNKbqe94d
miJGlI5etDKCxnM5SVU3FqNq7brLDbdpPBPe0p+rLwbzcVCiRzHwThGUB77l
8SZ3yNJ5N4P2ZB+d/ms8MhOyh0zqGLB/SBXH9/KQzVjmGv2ZwZH97osybHiQ
rG7XZyIWOR6HpZbdHJYfGp0zbM/ik8tk70sZHPTWWLuHqPouowvzbi7h8OJM
W8xtioUkOq96bIKg+WOdSZ9Ahf9BUCW4SbB5X+HYgicMroSrPej9nUBZNtvb
tpxBFF1cPLqboDOl3l4kZ+DYnnfONJRgWe9VZdQ6BonlL9I7QghKV60a/seB
QWHnmjkzVHgurXmSgxmDDcHXxiZnE9wqqmnv0WOwJ24sck4tQbtCI6tHm8Hh
iqUZN8cIWhcH1lvqqvbtp4t+s+FQudBYVmXCwPCss7ntLg5JJmxhtIrfqeTR
1NqzHKYcW5ljEMwgxymweKJP9T8nOi22HWXwuET04q4+j46gK8F+vQxiurW4
So5H/pvMDeZ2LAztRHYbF/A47mxtXJzKYt3q9G5HIQ86cyA85RmLI6YJzuVj
HHr9Zi7MtyKQPp5J3K5yaApjmjQiCFze/arpGMFh9lENNv8MwbO3Uj8BxSHR
uWgF20hwcMkHm5hmgk2Wc+MW3yOIFprOmx1HYNCSH1nbQOB93s1xk4TgnsWf
8b7FBCf9yxSK1yy2xtfLP/1KkLZe4KhXzUKa2qyR7UWADX/rZGey2Om840eb
WQSyEOOppZEsbIk5Tj1lMVif1hEQyqIsydybUrAYykpoClX1LkbCE7r7WWgx
CdmxKnzA+xS31mAWid+HWrSns2iveair78/C6+w8F0mliqdhXHHGh8X1l/lO
e7tYLJHpd2utZmEvHMxrMSIYjjS2dN+pmnctrFvpSdBl8K7DLZvF2juhV5NU
980L0TuV1chCZqs3samQYD7fky3RJhgvuJ6j00RgWPtE6eFDUON6fruo48vd
UZr9xwjY6v7Yva0Ec50ypvWPEFzh23ROVxEcHdlUKlzK4VinXGh1gGCWkeGg
MpODdHOMbotK3yL3bnRoMwf3mIAA43EW05QVTxZ94hDk9u0t5WkWI4k5ctqA
Bz9T/o2VLwtry4mnRoY8Pn71U+Z/P+XxHwXnOss=
        "]], InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {14.283684782537893`, 12.224334161450233`}, \
{-1, -1}], 
       {PointSize[0.04], PointBox[{11.99999999999999, 13.5}], 
        PointBox[{8., 13.5}], PointBox[{6., 10.}], 
        PointBox[{13.99999999999999, 10.}]}},
      AspectRatio->1,
      PlotRange->{{-1, 21}, {-1, 21}}], {66, 22}, {0, 0}, {22, 22}], 
    InsetBox[
     GraphicsBox[
      {Thickness[0.005], 
       {Dashing[{0.005, 0.030000000000000002`}], 
        LineBox[{{-1.0009326700810561`*^-11, 10.}, {5.999999999982448, 
         10.}}]}, PolygonBox[{{3.6, 10.}, {2.4, 9.6}, {2.4, 10.4}}], 
       InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {3., 9.0548}, {0, 1}], 
       {Dashing[{0.005, 0.030000000000000002`}], 
        LineBox[{{20.00000000000066, 10.}, {13.999999999999321`, 10.}}]}, 
       PolygonBox[{{17.599999999999994`, 10.}, {16.39999999999999, 10.4}, {
        16.39999999999999, 9.6}}], InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {16.999999999999993`, 9.0548}, {0, 1}], 
       LineBox[CompressedData["
1:eJw9lws0VekbxiU5CKXiXPY+x7k6NyoqpVSPKSXyr3SjRCQZ5FYpSYhEF7lO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        "]], InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {9.999999999999995, 5.0548000000000055`}, \
{0, 1}], 
       {Dashing[{0.005, 0.030000000000000002`}], 
        CircleBox[{9.493974050732, 10.324871971010285`}, 
         3.5090449505152663`, {2.0105863474436756`, 3.2343067312428637`}]}, 
       InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {5.923353752278861, 12.10602642726922}, \
{1, -1}], 
       {Dashing[{0.005, 0.030000000000000002`}], 
        CircleBox[{9.999999999999995, 13.049999999999999`}, 
         2.049999999999995, {0.2213144423477913, 2.9202782112420023`}]}, 
       InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {9.999999999999995, 15.795199999999994`}, \
{0, -1}], LineBox[CompressedData["
1:eJw9lXlUjXkYx+tKUmJU1jG67/6+FzVDIlJfTFnqSra5pTozV0WLltGJKWmh
NFO0i5KRFMqSUmMZEsOIhhbLTRyyHVIUNRkt5o55z/s75z2/P37n9z6/53m+
389DqIOX+ch0dHSCtd9/+/9LDt3P+yT83ZW5yKzSHCV/KTUPTipg5fgqTT/H
HDHK1QEPVQqEK4/UBZ00xzu74CnKPgEN/dzdN73mmBc6eXbzQQGPur061TZy
JMwf+Xb4agGJ/lduv98mh4dqf2glI6BlPAwnN8lRm6l/o9pAwC/9aVnbLQgk
uXHrGvQEqPqrjKq2EPDTXNgfN0ZAQtuJM3FXCEzyGWNiYStglftsRaoOiYjr
LTZtgQLaHip97SxJ7F18bsHmAgH/lA7Tu+xK4rDHuMG69wWYJUXnRa4jUZ9/
8E2CkQLtx9/5bNpAYv+tsZEqKwVMTer7fH4k0Z5U9NZ8mQIH2vyu1nmTsFli
/UXdGgUcR13dvW8xiZvzbpS7+inwJrcobyetjVe0w1hfe75PdWPtri4Cpg9D
Dee7KuAxQn5gyAUCCzsStn6cpsDsUp1TUfEEEtbFzakwVKAqA4U5Swlo+Jl1
Hbe1+Vl3hGUxBJJNR8bUpgvgv3QJzDQkMDrqoVmdgwAH65hRDXoE1p/Tm7q1
k8d9Va7vgTEE7HNczyfn8mi07vdImU9gd3bDdgsnHj1HalY0a+OZxX1oYw14
DIzzyTmmITBrdMhK+yYOy9t+vtw3jYTtQscpPZc4zDXz6tBLIVHxU2J65VUO
A89Gm7S0kEhuwoaLTzhs6ryc18BSeDNh9eN6Ex5Jme+iX7hTGDy1ppRw5UH6
3hnyNIqCd2rqLXU2D4vy/KIDaRQ29BjHxj/iUWxIOZjvoRB105qmaAGdM5Rp
G7MpBGQJrw9/L2At51QyKYVC4wym2DdNwJmVy8vOxFKQj12izisXEDBDt/lw
MIV7fW5H064IyGxd7Nit0r5HU+ydflVA3t5vplfMoXCzZPDqlkoBOrqhT6sn
UrBxqr4YmS3AJK205+sBbX5Dn0Td8xOw4J5p/OZHJDSN5/tOWwqIeGzkr/yT
hGVGsb79ax4Hlc+ntZ8lcVdj5sn/yuOBh3tXcBUJxyzZeCsXHqOhm3JSQ2Ja
q/ytWofHlcqM8mUGFDLDvuq8f5pDfLPyVJczBRVz0WtRNIedif2RH/MpuBPM
pzo3Dv2f/lsU6mq1y5lD4EbZtsFetHQeRpc5GZXT0n2rRYHOx3poZIj/f141
ELKEYzBVjD+xoNAoFQy+Fd+3qmx8rYkDI70/1rwrq2wmAwsxv1N8W8RWOSPl
/yrIxeCWLiPVp3dg5wS7Flqqn+eam9cXX6Kl+vqOqjhjeIiW6p96bYD+LpWW
+jOv+KjScQsNQuxfuubUitshtNTfodfKghLX0QgU+7+0+8HZx740toj6qHX2
cvMMohEu6qdpj+LDhDgaPqK+1gcYx4wrpKEv6i92uK16ShONt6I+e8e/ap84
kcEOUb9nX+eP6A1hJH0XmobbDqplJP33Ps/xWcaxkj9kq6xL6AgWpqJ/2BpZ
QcjvLHaJ/pIf8t7j+ZKFnei/ypro1im9LAJFfzY+K80I6GYxSvRveM2IJP/7
LJJEf9sGVrn7nGBxT/T/RZdPTnHaePEiH/Z1PPL8MJeV+OFxyGq3nxEr8cXY
200d2swgT+RPRavN2h8qGIlPbqa7bCxzGYlfL9stBuXsYCS+Gbsa52xPZpAv
8u9FWIx6WzaDRpGPui7ve22PMygW+WlMlu2prmOwT+RrQeEfd4I/Mdgs8jfM
dKlj1CxW4nO+DF2IZeEv8jtIFuS/qZ5Fssj3Mc+ruhtZTuJ/aNasScMiOWk+
jDPMH56n7VO8OD9G6rZlGsh4ab7Yu5VkL+R4dIjz57GH0W99U3lEi/MpJn56
czfN44g4v3o/+42T5tu/KkTocQ==
        "]], InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {9.999999999999995, 10.954800000000002`}, \
{0, 1}], 
       {Dashing[{0.005, 0.030000000000000002`}], 
        CircleBox[{10.449854645994966`, 10.292774083425702`}, 
         3.562197172882098, {-0.08228199986392748, 1.120574228356973}]}, 
       InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {14.06662399700678, 12.100299426861024`}, \
{-1, -1}], 
       {PointSize[0.04], PointBox[{11.99999999999999, 13.5}], 
        PointBox[{8., 13.5}], PointBox[{6., 10.}], 
        PointBox[{13.99999999999999, 10.}]}},
      AspectRatio->1,
      PlotRange->{{-1, 21}, {-1, 21}}], {0, 0}, {0, 0}, {22, 22}], InsetBox[
     GraphicsBox[
      {Thickness[0.005], 
       {Dashing[{0.005, 0.030000000000000002`}], 
        LineBox[{{-1.0009326700810561`*^-11, 10.}, {5.999999999982448, 
         10.}}]}, PolygonBox[{{3.6, 10.}, {2.4, 9.6}, {2.4, 10.4}}], 
       InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {3., 9.0548}, {0, 1}], 
       {Dashing[{0.005, 0.030000000000000002`}], 
        LineBox[{{20.00000000000066, 10.}, {13.999999999999321`, 10.}}]}, 
       PolygonBox[{{17.599999999999994`, 10.}, {16.39999999999999, 10.4}, {
        16.39999999999999, 9.6}}], InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {16.999999999999993`, 9.0548}, {0, 1}], 
       {Dashing[{0.005, 0.030000000000000002`}], 
        CircleBox[{9.999999999999995, 10.}, 3.9999999999999947`, {
         3.141592653589793, 6.283185307179586}]}, InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {9.999999999999995, 5.3048000000000055`}, \
{0, 1}], LineBox[CompressedData["
1:eJwtVHsw1QkYRSpSS6T7+P3u9eje34MGq9hNto4IPSTKq2JLi9SaktJLyWVR
iM2jRsir9JQV8tjahMGgvPbWXpMoeWxIJYS0d7f9Zr45c/74vvnOOTOfnvd+
Zx8lBQUFV3n/i1+LC8X/UISMno3Cu9EcDOYHm2tbilA9zA0ZMOHAA5MdXZEi
PGh/9sJRgQODsiKT/g4RMuskPgemFkNA7quzNBMjZda6gXf6HDjuDr+4JlaM
E90fig8HcVDcpW2ULhOj2dPmqW6vfL67rVODoHBNx2vdjUAuygvuBTPrKYwM
PD6frcbDHcuwvDBfCknBg97Z13lIWrbubfkBCqfnZUaJ1vCRGrHjgdY+Cvqy
cHq8kY+Wwvlmz90pnFLYf8fVmoBHa2XG8x8o3Fo9U9VxmcB99QjLGgEFr+Ok
ul8ngcbLI8sbpsQwySqqFU8TsPxLOyVUfp+yNLGocJzAMo8X79sqxJjwyJHZ
tBAw21OstTlLjDIbn3TpWQKDLV4hnnJ9KYc2KJszBFYk5lmcOyWG7btP2ybz
+diStk+1+pgYTpkBK3dTfMz+w75SdFqMRTf8ti+5wMOo8YSvNFGMcqX2s+py
vfEuREZsiRi0s7F1ZiwXzHcdR7/0i/FzkR21nOLiMfs0JsCAwp5StCzs5sC0
YozcdJRCr2dCok01BxW+sk6nxxSKrFOb4to44IRJ7R9SNKZTZVfvfsMF5eet
ypygUQLdaH+53wPcFrFVDQ2brvBzfR+4aKoz7X7yhYbWVIu2ZiwPiu5vplxp
BvfSH+n2GfEhksTkBVswmHa4KZO28GES0btraAUDw2MRpr/vI/Cofa2tHcPg
9MfcfqePBGJir591U2Ww2t2dMPInofwj5ZjWTeOkhHEOrCLh8knCXi+mETyw
sWe3ogB/R6vF3j5Dw7gvT7hcT4BRu66rol00poL4Hm/lfKphh1PxKhoWfS6b
7s4SoK15pBYiGrRq1cOZWhK3ZpeOamvRqK4MaPoSSGKprcqE2XwamS6u7lVz
SGhpvhlO0KARETDjYxZHIOu3IwX6ejRU/Ka3DC0g8FI9JfqFfL/7Idai5jwf
0SZjS/fvpcENygiykOd3zaYiqjqHxrycHtPkVh4abf3H8wdo8Dh/5jzP5CFK
w4h7Ru5HMxFWWirPt3fHyGsqnoHqAadBr3IeNuuHPOe8ZpB02Fo6PJePvVRn
60ozFg81Iq3cTvBxut3mXMIxFmOGymtH1QioVuikqt1mEblK6lafL/czoaxw
UyOLmOTgV5ruJOqNdgZqSFmcXzjk0DxN4soqKwnbwCLFdoH5cKIAwweGghzz
WXAHYy92LBaitP5o2oVwFpIJx5qIMCGMGod5Lx1Z+KVNX5K1CKH+3mGNPZfF
QSV2GzVXBx7763/J7GaQ2MXbFqCng/WykF16BQw8Bs2UOHJuf81ljI5iUPDT
g+SBOTpQSekZX+LPwIsq6HduF+LR0DvNPDcGtW9uCGUxQsyxjnsZv4XB5y17
38Z9K4QkS6Jv5ckg3+vmfbsGASpzzWOaDzHo95feur5TgLSRIn9eKoOLhccX
NU2SuPH00oq8RgbFDYoa2TkkNDorrh5XYxE16d2g4kPCpEGJV7FVrl94Ulxi
R6LuifPTg1dY7JxJ3z7LmUSuQ7bmkU8sWh3Wfy+LJBEaqqebvMEAW429Md1D
Ism/LFctxQAlL221u3wFGCk0l6xqk/PP+lVrZwmhmJ7sFjpjgMSKhm8O5wsR
f9d1unyRIcJ/fZbet10Hpr0a1Cs5//pPdf9HQ/wDhJ5AaQ==
        "]], InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {5.706292966747745, 12.23006116185843}, \
{1, -1}], LineBox[CompressedData["
1:eJw9lXlUjWkcx1sYTWnRwmGo7rvdq/u+N0syWb8MmSjZokhMKklulpASSWEs
LadNJXNKi2Uktyx1MtdaGssoW8ooY4vKKCVc1dzpPOd9znnP88dznvf3/H6/
7/fzk/iGLPDX09HRCdF+/++9fcsGPX27HT51JLtYnreBxxiVlJ4rh4Pz28Tv
MmwQpcpNpgrk2Ox24p7yrA1MriTUqPR51HRLH73X2OBS3P3rzDIeDZ0+bb5O
tthW/t6sLZfHvqAbDz7G2CK3YEWcSx2P58NgyD+xhcPaLw5Tunjs705M2auQ
IDS/Nk3Q8PDsVhupd0iQJp22IrKJx56WM6XRNyR4mNHUWn2Nx+KlE+0SdCjE
jrOusEji0fK3W8AUewqrzs24uNubx+eigf2uzafgeez11x6Wh+WBnVkRgRTs
fZYN2tYhR2thu3/YJgorR72JKbglh7l59Tf/jRQsQr3MGk/LkdOypuKeH4XK
s1X/2h+Rw9mq4vDR2RTGXHJwLUyV431mflYco43ntbH9i/b8qOet1akdErRS
cZ3lhXJ4m9rmDPhDgoum27b3vy3HxCKdkshYCcLTIq/O7pRDnYS8jHkSyB5X
2pvKtfk5fghNYbX5t7zfOXYdD9kP7sHJhhK8jaaa7ct4zHCMsqrpJ0HSDM2d
7SYCnnhmBuQMkeCyf+H0TX4C7jt2e8f/JMGaQCGsukRA14mqRfXaeM2RAyye
dAnoGeqfcbpWgoq38ScvcwosbPn12rexFK5fKK0xmKwALH0+9IunMGfP1nUu
Tgr0vBxs/vw5hVBOfXDqCAXC2q5l1XA0zF/k2ihaBRxIbt/5eikNzR1H92eF
AiQBDwe8iKRxJCRkVFagAEVxdn5OIo2DBu07wm0FnDSkZ9qk09g9uqr+aT2P
tvFuiVvTaKQEPbJc8huP1dI5p+TxNISbdR7pSh6lHgtVpbtoNL45m+XryiNo
vG798RAaI/XzFyon8Eh5N9u501P7HqlH5jonHllHRo87N5nGmEVfc61deOjo
bnhxxZpGZcmUqTGBPMwTi7pG9Wjz+zQiWpbKw/mxRez2Bgoyfrr+rHs8whuN
gtwqKVQHe3y5bCkg1+3V2NYyCnbS5pzHKwU89V7aEaKmUBbU/epWkYDB0I0/
W0vhjlWDWVavgOvnk4oXGNAIPvCPCTtLgT31biUdrjSO103NPh+lwKF93RFf
s2nkP6vTsc9XoLvPZzRGjdWuYgWCt+rF9PdhxPNQRjXHqJgR7zu4BLue7mKw
jvz/pbpn/Vwpi7skvvWxPKMEsCgn71usGnbbfCYrvj/KpiNF9SOLGpJfsawl
fLcti5Ek/yalu8FfuqxYH01P3PApzxmxfstX3f1z9lVGrG+A1blSwwJGrH/C
zR5mSQIj9mfayd/dnHcwaCD9S6otWfRgPSP29/ubKuW+QAbJpP/zOp+WNQYw
iCb6uO3q47VcyWA/0U9tut3n4dEMMom+lGuNo4bmMfhK9LfLZJKv8ITBIKJP
zbC3rdbWLDYR/ZY2Z5tq1rOivvMstkzSv82K+te8yvBfIOVQSfyhu9jxFBPO
oZX4h6vSO7a+nEMQ8ZdtgV/68iYOV4n/zlftfCdoOCQTf95/WZS0tpNDM/Hv
5irTA0F1HLYQf08KVi/1P8PBjvj/snvvnGhtvAjCh6MfGpZ/nsaJ/PAucDi8
xogT+WLs5+W7oZ6FH+FPyTun1b+cY0U+eVmkOtlnsiK/mloV+hmHWJFvxvON
M/YeZLGC8O91aJRvTBoLBeGjjvtHzaRCFksIP40pVfqVeyx8CV+P5V1/GNLL
IobwN9RinnPkBE7kc7YeOrCLQyrht1JPGRRWzYl8H/JK3Xmfk2Ic4f+GlAny
gRFScT4MNcw2yaqQIoLMDzPdlmQDPRnUZL5M9TqV9rNUBlMyfxq9jS58GyND
NJlPUbHj6jsZGZaQ+aXp85sUvWS+/QePOgzf
        "]], InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {9.999999999999995, 16.045199999999994`}, \
{0, -1}], LineBox[CompressedData["
1:eJw9lXlUjXkYx+tKUmJU1jG67/6+FzVDIlJfTFnqSra5pTozV0WLltGJKWmh
NFO0i5KRFMqSUmMZEsOIhhbLTRyyHVIUNRkt5o55z/s75z2/P37n9z6/53m+
389DqIOX+ch0dHSCtd9/+/9LDt3P+yT83ZW5yKzSHCV/KTUPTipg5fgqTT/H
HDHK1QEPVQqEK4/UBZ00xzu74CnKPgEN/dzdN73mmBc6eXbzQQGPur061TZy
JMwf+Xb4agGJ/lduv98mh4dqf2glI6BlPAwnN8lRm6l/o9pAwC/9aVnbLQgk
uXHrGvQEqPqrjKq2EPDTXNgfN0ZAQtuJM3FXCEzyGWNiYStglftsRaoOiYjr
LTZtgQLaHip97SxJ7F18bsHmAgH/lA7Tu+xK4rDHuMG69wWYJUXnRa4jUZ9/
8E2CkQLtx9/5bNpAYv+tsZEqKwVMTer7fH4k0Z5U9NZ8mQIH2vyu1nmTsFli
/UXdGgUcR13dvW8xiZvzbpS7+inwJrcobyetjVe0w1hfe75PdWPtri4Cpg9D
Dee7KuAxQn5gyAUCCzsStn6cpsDsUp1TUfEEEtbFzakwVKAqA4U5Swlo+Jl1
Hbe1+Vl3hGUxBJJNR8bUpgvgv3QJzDQkMDrqoVmdgwAH65hRDXoE1p/Tm7q1
k8d9Va7vgTEE7HNczyfn8mi07vdImU9gd3bDdgsnHj1HalY0a+OZxX1oYw14
DIzzyTmmITBrdMhK+yYOy9t+vtw3jYTtQscpPZc4zDXz6tBLIVHxU2J65VUO
A89Gm7S0kEhuwoaLTzhs6ryc18BSeDNh9eN6Ex5Jme+iX7hTGDy1ppRw5UH6
3hnyNIqCd2rqLXU2D4vy/KIDaRQ29BjHxj/iUWxIOZjvoRB105qmaAGdM5Rp
G7MpBGQJrw9/L2At51QyKYVC4wym2DdNwJmVy8vOxFKQj12izisXEDBDt/lw
MIV7fW5H064IyGxd7Nit0r5HU+ydflVA3t5vplfMoXCzZPDqlkoBOrqhT6sn
UrBxqr4YmS3AJK205+sBbX5Dn0Td8xOw4J5p/OZHJDSN5/tOWwqIeGzkr/yT
hGVGsb79ax4Hlc+ntZ8lcVdj5sn/yuOBh3tXcBUJxyzZeCsXHqOhm3JSQ2Ja
q/ytWofHlcqM8mUGFDLDvuq8f5pDfLPyVJczBRVz0WtRNIedif2RH/MpuBPM
pzo3Dv2f/lsU6mq1y5lD4EbZtsFetHQeRpc5GZXT0n2rRYHOx3poZIj/f141
ELKEYzBVjD+xoNAoFQy+Fd+3qmx8rYkDI70/1rwrq2wmAwsxv1N8W8RWOSPl
/yrIxeCWLiPVp3dg5wS7Flqqn+eam9cXX6Kl+vqOqjhjeIiW6p96bYD+LpWW
+jOv+KjScQsNQuxfuubUitshtNTfodfKghLX0QgU+7+0+8HZx740toj6qHX2
cvMMohEu6qdpj+LDhDgaPqK+1gcYx4wrpKEv6i92uK16ShONt6I+e8e/ap84
kcEOUb9nX+eP6A1hJH0XmobbDqplJP33Ps/xWcaxkj9kq6xL6AgWpqJ/2BpZ
QcjvLHaJ/pIf8t7j+ZKFnei/ypro1im9LAJFfzY+K80I6GYxSvRveM2IJP/7
LJJEf9sGVrn7nGBxT/T/RZdPTnHaePEiH/Z1PPL8MJeV+OFxyGq3nxEr8cXY
200d2swgT+RPRavN2h8qGIlPbqa7bCxzGYlfL9stBuXsYCS+Gbsa52xPZpAv
8u9FWIx6WzaDRpGPui7ve22PMygW+WlMlu2prmOwT+RrQeEfd4I/Mdgs8jfM
dKlj1CxW4nO+DF2IZeEv8jtIFuS/qZ5Fssj3Mc+ruhtZTuJ/aNasScMiOWk+
jDPMH56n7VO8OD9G6rZlGsh4ab7Yu5VkL+R4dIjz57GH0W99U3lEi/MpJn56
czfN44g4v3o/+42T5tu/KkTocQ==
        "]], InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {9.999999999999995, 10.954800000000002`}, \
{0, 1}], LineBox[CompressedData["
1:eJwtVAsw1XkUFrpMkYRK5Qr3//j9/9e2rlddXJ8mZBWxS9Fz26yoSbV5lFeL
uow0WGXX0vagh9dV6EE1ovKoLVcK1arIK2VKoe3B3po9M2fOnDPf+b45M2c+
s41hvkHqampq/qr8Uj9MfAkeal9DBMmO9eu1HcR4npbkmecogl/gAj3bn8XQ
NcttvigXoc1Ds7wmVgyfgUb1+49FSN71IDcyXowcwXCNth0FzTsVpYZbxCA7
b5TWHaBgt0ImW7FYjNvhr1defEihIdW8K0dbjMnTxU8uzaExZYHb9uxaHkFx
jjfWedCYceikZHc4j3PrDRs8g2hMSn2Xt43mMfX+vWtmYTT2jRaU2D7k0NEV
rGkaQkO2K7esO4tDWOu+Dxn+NEYs2hPaAjj0lOzJlTnQGGLixq/xHKpnHlK2
qvTo65/zFToc3vTZuBx8T6FIa7mrYJwgc/JBX7U2Ch5LLUp2qXGgxhVPLS5S
6BF2pT8y4KDsHnVPzqMQWEHrS+05vOp98Co3mcLdv9riCzZzOO9x33p0D4XE
4J5YrwIOlyXy1b9EUGicsfVCxgCH9IzOIkkshYiXP8HAiofTorTvvDIotMy/
4dQfwWPrwOeYLRUUBmh3aVslD5+6TnWbPgp4a7/mQj+PYoewfx+yNKbqe94d
miJGlI5etDKCxnM5SVU3FqNq7brLDbdpPBPe0p+rLwbzcVCiRzHwThGUB77l
8SZ3yNJ5N4P2ZB+d/ms8MhOyh0zqGLB/SBXH9/KQzVjmGv2ZwZH97osybHiQ
rG7XZyIWOR6HpZbdHJYfGp0zbM/ik8tk70sZHPTWWLuHqPouowvzbi7h8OJM
W8xtioUkOq96bIKg+WOdSZ9Ahf9BUCW4SbB5X+HYgicMroSrPej9nUBZNtvb
tpxBFF1cPLqboDOl3l4kZ+DYnnfONJRgWe9VZdQ6BonlL9I7QghKV60a/seB
QWHnmjkzVHgurXmSgxmDDcHXxiZnE9wqqmnv0WOwJ24sck4tQbtCI6tHm8Hh
iqUZN8cIWhcH1lvqqvbtp4t+s+FQudBYVmXCwPCss7ntLg5JJmxhtIrfqeTR
1NqzHKYcW5ljEMwgxymweKJP9T8nOi22HWXwuET04q4+j46gK8F+vQxiurW4
So5H/pvMDeZ2LAztRHYbF/A47mxtXJzKYt3q9G5HIQ86cyA85RmLI6YJzuVj
HHr9Zi7MtyKQPp5J3K5yaApjmjQiCFze/arpGMFh9lENNv8MwbO3Uj8BxSHR
uWgF20hwcMkHm5hmgk2Wc+MW3yOIFprOmx1HYNCSH1nbQOB93s1xk4TgnsWf
8b7FBCf9yxSK1yy2xtfLP/1KkLZe4KhXzUKa2qyR7UWADX/rZGey2Om840eb
WQSyEOOppZEsbIk5Tj1lMVif1hEQyqIsydybUrAYykpoClX1LkbCE7r7WWgx
CdmxKnzA+xS31mAWid+HWrSns2iveair78/C6+w8F0mliqdhXHHGh8X1l/lO
e7tYLJHpd2utZmEvHMxrMSIYjjS2dN+pmnctrFvpSdBl8K7DLZvF2juhV5NU
980L0TuV1chCZqs3samQYD7fky3RJhgvuJ6j00RgWPtE6eFDUON6fruo48vd
UZr9xwjY6v7Yva0Ec50ypvWPEFzh23ROVxEcHdlUKlzK4VinXGh1gGCWkeGg
MpODdHOMbotK3yL3bnRoMwf3mIAA43EW05QVTxZ94hDk9u0t5WkWI4k5ctqA
Bz9T/o2VLwtry4mnRoY8Pn71U+Z/P+XxHwXnOss=
        "]], InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {14.283684782537893`, 12.224334161450233`}, \
{-1, -1}], 
       {PointSize[0.04], PointBox[{11.99999999999999, 13.5}], 
        PointBox[{8., 13.5}], PointBox[{6., 10.}], 
        PointBox[{13.99999999999999, 10.}]}},
      AspectRatio->1,
      PlotRange->{{-1, 21}, {-1, 21}}], {22, 0}, {0, 0}, {22, 22}], InsetBox[
     GraphicsBox[
      {Thickness[0.005], 
       {Dashing[{0.005, 0.030000000000000002`}], 
        LineBox[{{1.525446435834965*^-12, 10.}, {3.000000000006441, 10.}}]}, 
       PolygonBox[{{2.1, 10.}, {0.9, 9.6}, {0.9, 10.4}}], InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {1.5, 10.9452}, {0, -1}], 
       {Dashing[{0.005, 0.030000000000000002`}], 
        LineBox[{{19.999999999998018`, 10.}, {16.99999999999754, 10.}}]}, 
       PolygonBox[{{19.1, 10.}, {17.9, 10.4}, {17.9, 9.6}}], InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {18.5, 9.0548}, {0, 1}], 
       {Dashing[{0.005, 0.030000000000000002`}], 
        LineBox[{{12.000000000001542`, 10.}, {8.00000000000287, 10.}}]}, 
       InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {9.999999999999986, 9.3048}, {0, 1}], 
       {Dashing[{0.005, 0.030000000000000002`}], 
        CircleBox[{14.499999999999986`, 10.}, 2.500000000000015, {
         5.773159728050814*^-15, 3.141592653589787}]}, InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {14.499999999999986`, 13.1952}, {0, -1}], 
       LineBox[CompressedData["
1:eJw9Vgs0Veke76XS0FDT43qdfU7FOfs89maqK6SfotKDhIiEhNSERMJFRHlc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        "]], InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {14.499999999999986`, 6.5548}, {0, 1}], 
       {Dashing[{0.005, 0.030000000000000002`}], 
        CircleBox[{5.5, 10.}, 2.5, {-3.141592653589793, 0.}]}, InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {5.499999999999999, 6.8048}, {0, 1}], 
       LineBox[CompressedData["
1:eJw9Vgs0VWkbFhHHoXMO53CuHZdzUao/g5HIE6VQLuWW+IkUlammqFxzKfSn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        "]], InsetBox[
        TagBox[
         StyleBox[
          TagBox["\<\"\"\>",
           DisplayForm],
          FontFamily->"Times",
          FontSize->8.247272727272728],
         StyleForm[#, FontFamily -> "Times", FontSize -> 
          8.247272727272728]& ], {5.5, 13.4452}, {0, -1}], 
       {PointSize[0.04], PointBox[{11.99999999999997, 10.}], 
        PointBox[{8., 10.}], PointBox[{3., 10.}], PointBox[{17., 10.}]}},
      AspectRatio->1,
      PlotRange->{{-1, 21}, {-1, 21}}], {44, 0}, {0, 0}, {22, 22}]},
   AspectRatio->NCache[
     Rational[1, 2], 0.5],
   PlotRange->{{0, 88}, {0, 44}}], TraditionalForm]], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Calculating the bare 2-loop off-shell ghost selfenergy diagrams\
\>", "Subsubsection"],

Cell[BoxData[
 RowBox[{
  RowBox[{"QuarkMass", "=", "0"}], ";"}]], "Input"],

Cell["We need to put the AmplitudeLevel setting to Classes here.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"amps", "=", 
    RowBox[{"CalcColorFactor", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"CreateFeynAmp", "[", 
          RowBox[{"inserts", ",", 
           RowBox[{"Truncated", "\[Rule]", "True"}], ",", 
           RowBox[{"PreFactor", "\[Rule]", "I"}], ",", 
           RowBox[{"AmplitudeLevel", "\[Rule]", 
            RowBox[{"{", "Classes", "}"}]}]}], "]"}], "/.", 
         RowBox[{"p1", "\[RuleDelayed]", "p"}]}], "/.", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"li1", "\[RuleDelayed]", "\[Mu]"}], ",", 
          RowBox[{"li2", "\[RuleDelayed]", "\[Nu]"}]}], "}"}]}], "/.", 
       RowBox[{
        RowBox[{"FeynAmpList", "[", "__", "]"}], "\[RuleDelayed]", "List"}]}],
       " ", "/.", " ", 
      RowBox[{
       RowBox[{"FeynAmp", "[", 
        RowBox[{"_", ",", "_", ",", "x_"}], "]"}], "\[RuleDelayed]", "x"}]}], 
     "]"}]}], ";"}], "//", "Timing"}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"0.5480339999999999`", ",", "Null"}], "}"}], 
  TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"amps", "//", "TableForm"}]], "Input"],

Cell[BoxData[
 FormBox[
  TagBox[
   TagBox[GridBox[{
      {
       RowBox[{
        FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
        SubsuperscriptBox["C", "A", "2"], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["ci1",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["ci2",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        RowBox[{
         SubscriptBox["\[CapitalPi]", "u"], "(", 
         FormBox[
          RowBox[{"-", "q1"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SuperscriptBox[
          OverscriptBox["\[CapitalLambda]", "~"], "li4"], "(", 
         FormBox[
          RowBox[{"-", "q1"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SuperscriptBox[
          OverscriptBox["\[CapitalLambda]", "~"], "\[Nu]"], "(", 
         FormBox[
          RowBox[{"-", "p"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SubscriptBox["\[CapitalPi]", "u"], "(", 
         FormBox[
          RowBox[{"q2", "-", "p"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SubscriptBox["\[CapitalPi]", "u"], "(", 
         FormBox[
          RowBox[{"q2", "-", "q1"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SuperscriptBox[
          OverscriptBox["\[CapitalLambda]", "~"], "li3"], "(", 
         FormBox[
          RowBox[{"q2", "-", "p"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SuperscriptBox[
          OverscriptBox["\[CapitalLambda]", "~"], "\[Mu]"], "(", 
         FormBox[
          RowBox[{"q2", "-", "q1"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SubsuperscriptBox["\[CapitalPi]", "g", 
          RowBox[{"\[Mu]", "\[Nu]"}]], "(", "q2", ")"}], " ", 
        RowBox[{
         SubsuperscriptBox["\[CapitalPi]", "g", 
          RowBox[{"li3", "li4"}]], "(", 
         FormBox[
          RowBox[{"q1", "-", "p"}],
          TraditionalForm], ")"}]}]},
      {
       RowBox[{
        FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
        SubsuperscriptBox["C", "A", "2"], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["ci1",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["ci2",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        RowBox[{
         SubscriptBox["\[CapitalPi]", "u"], "(", 
         FormBox[
          RowBox[{"-", "q1"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SubscriptBox["\[CapitalPi]", "u"], "(", 
         FormBox[
          RowBox[{"-", "q2"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SuperscriptBox[
          OverscriptBox["\[CapitalLambda]", "~"], "li4"], "(", 
         FormBox[
          RowBox[{"-", "q1"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SuperscriptBox[
          OverscriptBox["\[CapitalLambda]", "~"], "li6"], "(", 
         FormBox[
          RowBox[{"-", "p"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SuperscriptBox[
          OverscriptBox["\[CapitalLambda]", "~"], "\[Nu]"], "(", 
         FormBox[
          RowBox[{"-", "q2"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SubsuperscriptBox["\[CapitalPi]", "g", 
          RowBox[{"li3", "li4"}]], "(", 
         FormBox[
          RowBox[{"q1", "-", "p"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SubsuperscriptBox["\[CapitalPi]", "g", 
          RowBox[{"li5", "li6"}]], "(", 
         FormBox[
          RowBox[{"p", "-", "q2"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SubsuperscriptBox["\[CapitalPi]", "g", 
          RowBox[{"\[Mu]", "\[Nu]"}]], "(", 
         FormBox[
          RowBox[{"q2", "-", "q1"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SuperscriptBox["V", 
          RowBox[{"\[Mu]", "li3", "li5"}]], "(", 
         RowBox[{
          FormBox[
           RowBox[{"q1", "-", "q2"}],
           TraditionalForm], 
          RowBox[{",", " "}], 
          FormBox[
           RowBox[{"p", "-", "q1"}],
           TraditionalForm], 
          RowBox[{",", " "}], 
          FormBox[
           RowBox[{"q2", "-", "p"}],
           TraditionalForm]}], ")"}]}]},
      {
       RowBox[{"\[ImaginaryI]", " ", "Tf", " ", 
        SubscriptBox["C", "A"], " ", 
        SubscriptBox["N", "f"], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["ci1",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["ci2",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        RowBox[{
         SubscriptBox["\[CapitalPi]", "u"], "(", 
         FormBox[
          RowBox[{"-", "q1"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SuperscriptBox[
          OverscriptBox["\[CapitalLambda]", "~"], "li4"], "(", 
         FormBox[
          RowBox[{"-", "p"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SuperscriptBox[
          OverscriptBox["\[CapitalLambda]", "~"], "\[Nu]"], "(", 
         FormBox[
          RowBox[{"-", "q1"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SubsuperscriptBox["\[CapitalPi]", "g", 
          RowBox[{"li3", "li4"}]], "(", 
         FormBox[
          RowBox[{"p", "-", "q1"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SubsuperscriptBox["\[CapitalPi]", "g", 
          RowBox[{"\[Mu]", "\[Nu]"}]], "(", 
         FormBox[
          RowBox[{"q1", "-", "p"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{"tr", "(", 
         FormBox[
          RowBox[{
           RowBox[{
            SubscriptBox["\[CapitalPi]", "q"], "(", 
            FormBox[
             RowBox[{"q2", "-", "p"}],
             TraditionalForm], ")"}], ".", 
           SuperscriptBox["Q", "\[Mu]"], ".", 
           RowBox[{
            SubscriptBox["\[CapitalPi]", "q"], "(", 
            FormBox[
             RowBox[{"q2", "-", "q1"}],
             TraditionalForm], ")"}], ".", 
           SuperscriptBox["Q", "li3"]}],
          TraditionalForm], ")"}]}]},
      {
       RowBox[{
        RowBox[{"-", "\[ImaginaryI]"}], " ", 
        SubsuperscriptBox["C", "A", "2"], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["ci1",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["ci2",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        RowBox[{
         SubscriptBox["\[CapitalPi]", "u"], "(", 
         FormBox[
          RowBox[{"-", "q1"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SuperscriptBox[
          OverscriptBox["\[CapitalLambda]", "~"], "li4"], "(", 
         FormBox[
          RowBox[{"-", "p"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SuperscriptBox[
          OverscriptBox["\[CapitalLambda]", "~"], "\[Nu]"], "(", 
         FormBox[
          RowBox[{"-", "q1"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SubscriptBox["\[CapitalPi]", "u"], "(", 
         FormBox[
          RowBox[{"q2", "-", "p"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SubscriptBox["\[CapitalPi]", "u"], "(", 
         FormBox[
          RowBox[{"q2", "-", "q1"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SuperscriptBox[
          OverscriptBox["\[CapitalLambda]", "~"], "li3"], "(", 
         FormBox[
          RowBox[{"q2", "-", "q1"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SuperscriptBox[
          OverscriptBox["\[CapitalLambda]", "~"], "\[Mu]"], "(", 
         FormBox[
          RowBox[{"q2", "-", "p"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SubsuperscriptBox["\[CapitalPi]", "g", 
          RowBox[{"li3", "li4"}]], "(", 
         FormBox[
          RowBox[{"p", "-", "q1"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SubsuperscriptBox["\[CapitalPi]", "g", 
          RowBox[{"\[Mu]", "\[Nu]"}]], "(", 
         FormBox[
          RowBox[{"q1", "-", "p"}],
          TraditionalForm], ")"}]}]},
      {
       RowBox[{"\[ImaginaryI]", " ", 
        SubsuperscriptBox["C", "A", "2"], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["ci1",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["ci2",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        RowBox[{
         SuperscriptBox[
          OverscriptBox["\[CapitalLambda]", "~"], "\[Nu]"], "(", 
         FormBox[
          RowBox[{"-", "p"}],
          TraditionalForm], ")"}], " ", 
        SuperscriptBox[
         RowBox[{
          SubscriptBox["\[CapitalPi]", "u"], "(", 
          FormBox[
           RowBox[{"q1", "-", "p"}],
           TraditionalForm], ")"}], "2"], " ", 
        RowBox[{
         SubscriptBox["\[CapitalPi]", "u"], "(", 
         FormBox[
          RowBox[{"q2", "-", "p"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SuperscriptBox[
          OverscriptBox["\[CapitalLambda]", "~"], "li3"], "(", 
         FormBox[
          RowBox[{"q1", "-", "p"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SuperscriptBox[
          OverscriptBox["\[CapitalLambda]", "~"], "li4"], "(", 
         FormBox[
          RowBox[{"q2", "-", "p"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SuperscriptBox[
          OverscriptBox["\[CapitalLambda]", "~"], "\[Mu]"], "(", 
         FormBox[
          RowBox[{"q1", "-", "p"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SubsuperscriptBox["\[CapitalPi]", "g", 
          RowBox[{"\[Mu]", "\[Nu]"}]], "(", "q1", ")"}], " ", 
        RowBox[{
         SubsuperscriptBox["\[CapitalPi]", "g", 
          RowBox[{"li3", "li4"}]], "(", 
         FormBox[
          RowBox[{"q1", "-", "q2"}],
          TraditionalForm], ")"}]}]},
      {
       RowBox[{
        RowBox[{"-", 
         FractionBox["1", "2"]}], " ", "\[ImaginaryI]", " ", 
        SubsuperscriptBox["C", "A", "2"], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["ci1",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["ci2",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        RowBox[{
         SubscriptBox["\[CapitalPi]", "u"], "(", 
         FormBox[
          RowBox[{"-", "q1"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SuperscriptBox[
          OverscriptBox["\[CapitalLambda]", "~"], "li8"], "(", 
         FormBox[
          RowBox[{"-", "p"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SuperscriptBox[
          OverscriptBox["\[CapitalLambda]", "~"], "\[Nu]"], "(", 
         FormBox[
          RowBox[{"-", "q1"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SubsuperscriptBox["\[CapitalPi]", "g", 
          RowBox[{"li3", "li4"}]], "(", 
         FormBox[
          RowBox[{"q1", "-", "q2"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SubsuperscriptBox["\[CapitalPi]", "g", 
          RowBox[{"li5", "li6"}]], "(", 
         FormBox[
          RowBox[{"q2", "-", "p"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SubsuperscriptBox["\[CapitalPi]", "g", 
          RowBox[{"li7", "li8"}]], "(", 
         FormBox[
          RowBox[{"p", "-", "q1"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SubsuperscriptBox["\[CapitalPi]", "g", 
          RowBox[{"\[Mu]", "\[Nu]"}]], "(", 
         FormBox[
          RowBox[{"q1", "-", "p"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SuperscriptBox["V", 
          RowBox[{"li3", "li5", "li7"}]], "(", 
         RowBox[{
          FormBox[
           RowBox[{"q2", "-", "q1"}],
           TraditionalForm], 
          RowBox[{",", " "}], 
          FormBox[
           RowBox[{"p", "-", "q2"}],
           TraditionalForm], 
          RowBox[{",", " "}], 
          FormBox[
           RowBox[{"q1", "-", "p"}],
           TraditionalForm]}], ")"}], " ", 
        RowBox[{
         SuperscriptBox["V", 
          RowBox[{"\[Mu]", "li4", "li6"}]], "(", 
         RowBox[{
          FormBox[
           RowBox[{"p", "-", "q1"}],
           TraditionalForm], 
          RowBox[{",", " "}], 
          FormBox[
           RowBox[{"q1", "-", "q2"}],
           TraditionalForm], 
          RowBox[{",", " "}], 
          FormBox[
           RowBox[{"q2", "-", "p"}],
           TraditionalForm]}], ")"}]}]},
      {
       RowBox[{"\[ImaginaryI]", " ", 
        SubsuperscriptBox["C", "A", "2"], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["ci1",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["ci2",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        RowBox[{
         SubscriptBox["\[CapitalPi]", "u"], "(", 
         FormBox[
          RowBox[{"-", "p"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SubscriptBox["\[CapitalPi]", "u"], "(", 
         FormBox[
          RowBox[{"-", "q1"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SubscriptBox["\[CapitalPi]", "u"], "(", 
         FormBox[
          RowBox[{"-", "q2"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SuperscriptBox[
          OverscriptBox["\[CapitalLambda]", "~"], "li3"], "(", 
         FormBox[
          RowBox[{"-", "p"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SuperscriptBox[
          OverscriptBox["\[CapitalLambda]", "~"], "li4"], "(", 
         FormBox[
          RowBox[{"-", "q2"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SuperscriptBox[
          OverscriptBox["\[CapitalLambda]", "~"], "\[Nu]"], "(", 
         FormBox[
          RowBox[{"-", "p"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SuperscriptBox[
          OverscriptBox["\[CapitalLambda]", "~"], "\[Mu]"], "(", 
         FormBox[
          RowBox[{"-", "q1"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SubsuperscriptBox["\[CapitalPi]", "g", 
          RowBox[{"li3", "li4"}]], "(", 
         FormBox[
          RowBox[{"q2", "-", "p"}],
          TraditionalForm], ")"}], " ", 
        RowBox[{
         SubsuperscriptBox["\[CapitalPi]", "g", 
          RowBox[{"\[Mu]", "\[Nu]"}]], "(", 
         FormBox[
          RowBox[{"p", "-", "q1"}],
          TraditionalForm], ")"}]}]}
     },
     GridBoxAlignment->{
      "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
       "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}],
    Column],
   Function[BoxForm`e$, 
    TableForm[BoxForm`e$]]], TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["A function to do self energy diagrams", "Subsection"],

Cell["This is a substitution rule for a tensor integral of rank 1.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"tsub1", "=", 
  RowBox[{
   RowBox[{"FCI", "[", 
    RowBox[{
     RowBox[{"FVD", "[", 
      RowBox[{
       RowBox[{"qu1", ":", 
        RowBox[{"(", 
         RowBox[{"q1", "|", "q2"}], ")"}]}], ",", "mu"}], "]"}], 
     RowBox[{"FVD", "[", 
      RowBox[{
       RowBox[{"qu2", ":", 
        RowBox[{"(", 
         RowBox[{"q1", "|", "q2"}], ")"}]}], ",", "nu"}], "]"}]}], "]"}], ":>", 
   RowBox[{"TIDL", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"qu1", ",", "mu"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"qu2", ",", "nu"}], "}"}]}], "}"}], ",", 
     RowBox[{"{", "p", "}"}]}], "]"}]}]}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     SuperscriptBox[
      FormBox[
       RowBox[{"qu1", ":", 
        RowBox[{"q1", "|", "q2"}]}],
       TraditionalForm], 
      FormBox[
       FormBox["mu",
        TraditionalForm],
       TraditionalForm]], ")"}], " ", 
    RowBox[{"(", 
     SuperscriptBox[
      FormBox[
       RowBox[{"qu2", ":", 
        RowBox[{"q1", "|", "q2"}]}],
       TraditionalForm], 
      FormBox[
       FormBox["nu",
        TraditionalForm],
       TraditionalForm]], ")"}]}], "\[RuleDelayed]", 
   RowBox[{"TIDL", "(", 
    RowBox[{
     RowBox[{"(", "\[NoBreak]", GridBox[{
        {"qu1", "mu"},
        {"qu2", "nu"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
         "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.4]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}], ",", 
     RowBox[{"{", "p", "}"}]}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell["This is a substitution rule for a tensor integral of rank 2.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"tsub2", "=", 
  RowBox[{
   RowBox[{"FCI", "[", 
    RowBox[{"FVD", "[", 
     RowBox[{
      RowBox[{"qu", ":", 
       RowBox[{"(", 
        RowBox[{"q1", "|", "q2"}], ")"}]}], ",", "al_"}], "]"}], "]"}], ":>", 
   
   RowBox[{"TIDL", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"qu", ",", "al"}], "}"}], "}"}], ",", 
     RowBox[{"{", "p", "}"}]}], "]"}]}]}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox[
     RowBox[{"qu", ":", 
      RowBox[{"q1", "|", "q2"}]}],
     TraditionalForm], 
    FormBox[
     FormBox["al_",
      TraditionalForm],
     TraditionalForm]], "\[RuleDelayed]", 
   RowBox[{"TIDL", "(", 
    RowBox[{
     RowBox[{"(", "\[NoBreak]", GridBox[{
        {"qu", "al"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
         "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.7]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.4]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}], ",", 
     RowBox[{"{", "p", "}"}]}], ")"}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ScalarProduct", "[", 
    RowBox[{"p", ",", "p"}], "]"}], "=", "pp"}], ";", 
  RowBox[{
   RowBox[{"FAD", "[", "p", "]"}], "=", 
   RowBox[{"1", "/", "pp"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"do2self1", "[", "z_", "]"}], ":=", 
  RowBox[{"FCE", "[", 
   RowBox[{"ToFI", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"ch1", "[", "z", "]"}], "=", 
         RowBox[{"Expand", "[", 
          RowBox[{
           RowBox[{"Contract", "[", 
            RowBox[{
             RowBox[{"SUNSimplify", "[", 
              RowBox[{
               RowBox[{"Explicit", "[", 
                RowBox[{"z", ",", 
                 RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
                 RowBox[{"Gauge", "\[Rule]", 
                  RowBox[{"1", "-", "\[Xi]"}]}]}], "]"}], ",", 
               RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}], "/.", 
             "\[InvisibleSpace]", 
             RowBox[{"DiracTrace", "\[Rule]", "TR"}]}], "]"}], ",", 
           RowBox[{"q1", "|", "q2"}]}], "]"}]}], ")"}], "/.", 
       "\[InvisibleSpace]", "tsub1"}], "/.", "\[InvisibleSpace]", "tsub2"}], 
     ",", 
     RowBox[{"{", 
      RowBox[{"q1", ",", "q2"}], "}"}], ",", 
     RowBox[{"{", "p", "}"}]}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"do2self2", "[", "z_", "]"}], ":=", 
  RowBox[{
   RowBox[{"TarcerRecurse", "[", 
    RowBox[{
     RowBox[{"Collect3", "[", 
      RowBox[{
       RowBox[{"z", "/.", "inttable"}], ",", 
       RowBox[{"{", 
        RowBox[{"TAI", ",", "TBI", ",", "TVI", ",", "TFI"}], "}"}], ",", 
       RowBox[{"Factoring", "\[Rule]", "Factor2"}]}], "]"}], ";", 
     RowBox[{"If", "[", 
      RowBox[{"comment", ",", 
       RowBox[{"Print", "[", 
        RowBox[{"\"\<starting recursion on \>\"", ",", 
         RowBox[{"Length", "[", 
          RowBox[{"ch2", "[", "z", "]"}], "]"}], ",", "\"\< integrals\>\""}], 
        "]"}]}], "]"}], ";", 
     RowBox[{"ch2", "[", "z", "]"}]}], "]"}], "/.", "\[InvisibleSpace]", 
   RowBox[{"x_Plus", "\[RuleDelayed]", 
    RowBox[{
     RowBox[{"Collect2", "[", 
      RowBox[{"x", ",", "\[Xi]"}], "]"}], "/;", 
     RowBox[{
      RowBox[{"Variables", "[", "x", "]"}], "\[Equal]", 
      RowBox[{"{", 
       RowBox[{"D", ",", "\[Xi]"}], "}"}]}]}]}]}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Calculate the graphs one by one", "Subsection"],

Cell[BoxData[
 RowBox[{
  RowBox[{"comment", "=", "True"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SPD", "[", 
    RowBox[{"p", ",", "p"}], "]"}], "=", "pp"}], ";"}]], "Input"],

Cell["Do all the algebra but no integrals yet.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{
   RowBox[{"res1", "=", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"Print", "[", 
        RowBox[{
        "\"\<calculating \>\"", ",", "i", ",", " ", "\"\<  time = \>\"", ",", 
         
         RowBox[{
          RowBox[{"Timing", "[", 
           RowBox[{
            RowBox[{"re", "[", "i", "]"}], "=", 
            RowBox[{"do2self1", "[", 
             RowBox[{"(", 
              RowBox[{"amps", "[", 
               RowBox[{"[", "i", "]"}], "]"}], ")"}], "]"}]}], "]"}], "[", 
          RowBox[{"[", "1", "]"}], "]"}], ",", 
         "\"\<, number of integrals to calculate = \>\"", ",", 
         RowBox[{"Length", "[", 
          RowBox[{"re", "[", "i", "]"}], "]"}]}], "]"}], ";", 
       RowBox[{"re", "[", "i", "]"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", " ", 
        RowBox[{"Length", "[", "amps", "]"}]}], "}"}]}], "]"}]}], ";"}], 
  "]"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"calculating \"\>", "\[InvisibleSpace]", "1", 
    "\[InvisibleSpace]", "\<\"  time = \"\>", "\[InvisibleSpace]", 
    "0.512032`", 
    "\[InvisibleSpace]", "\<\", number of integrals to calculate = \"\>", 
    "\[InvisibleSpace]", "36"}],
   SequenceForm[
   "calculating ", 1, "  time = ", 0.512032, 
    ", number of integrals to calculate = ", 36],
   Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"calculating \"\>", "\[InvisibleSpace]", "2", 
    "\[InvisibleSpace]", "\<\"  time = \"\>", "\[InvisibleSpace]", 
    "1.5760980000000004`", 
    "\[InvisibleSpace]", "\<\", number of integrals to calculate = \"\>", 
    "\[InvisibleSpace]", "108"}],
   SequenceForm[
   "calculating ", 2, "  time = ", 1.5760980000000004`, 
    ", number of integrals to calculate = ", 108],
   Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"calculating \"\>", "\[InvisibleSpace]", "3", 
    "\[InvisibleSpace]", "\<\"  time = \"\>", "\[InvisibleSpace]", 
    "1.4240890000000004`", 
    "\[InvisibleSpace]", "\<\", number of integrals to calculate = \"\>", 
    "\[InvisibleSpace]", "75"}],
   SequenceForm[
   "calculating ", 3, "  time = ", 1.4240890000000004`, 
    ", number of integrals to calculate = ", 75],
   Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"calculating \"\>", "\[InvisibleSpace]", "4", 
    "\[InvisibleSpace]", "\<\"  time = \"\>", "\[InvisibleSpace]", 
    "0.40802599999999956`", 
    "\[InvisibleSpace]", "\<\", number of integrals to calculate = \"\>", 
    "\[InvisibleSpace]", "54"}],
   SequenceForm[
   "calculating ", 4, "  time = ", 0.40802599999999956`, 
    ", number of integrals to calculate = ", 54],
   Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"calculating \"\>", "\[InvisibleSpace]", "5", 
    "\[InvisibleSpace]", "\<\"  time = \"\>", "\[InvisibleSpace]", 
    "0.7800479999999999`", 
    "\[InvisibleSpace]", "\<\", number of integrals to calculate = \"\>", 
    "\[InvisibleSpace]", "56"}],
   SequenceForm[
   "calculating ", 5, "  time = ", 0.7800479999999999, 
    ", number of integrals to calculate = ", 56],
   Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"calculating \"\>", "\[InvisibleSpace]", "6", 
    "\[InvisibleSpace]", "\<\"  time = \"\>", "\[InvisibleSpace]", 
    "8.964561`", 
    "\[InvisibleSpace]", "\<\", number of integrals to calculate = \"\>", 
    "\[InvisibleSpace]", "514"}],
   SequenceForm[
   "calculating ", 6, "  time = ", 8.964561, 
    ", number of integrals to calculate = ", 514],
   Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"calculating \"\>", "\[InvisibleSpace]", "7", 
    "\[InvisibleSpace]", "\<\"  time = \"\>", "\[InvisibleSpace]", 
    "0.3440209999999979`", 
    "\[InvisibleSpace]", "\<\", number of integrals to calculate = \"\>", 
    "\[InvisibleSpace]", "25"}],
   SequenceForm[
   "calculating ", 7, "  time = ", 0.3440209999999979, 
    ", number of integrals to calculate = ", 25],
   Editable->False], TraditionalForm]], "Print"]
}, Open  ]],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"14.008875`", ",", "Null"}], "}"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"allints", " ", "=", " ", 
   RowBox[{"Cases2", "[", 
    RowBox[{"res1", ",", "TFI"}], "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"allints", "//", "Length"}]], "Input"],

Cell[BoxData[
 FormBox["651", TraditionalForm]], "Output"]
}, Open  ]],

Cell["\<\
There are 651 integrals to be done for the ghost self energy. There are \
several possibilities how to proceed. One possibility is to calculate the \
integrals one by one and save them to a file. This can be conveniently done \
using the CheckDB function. If the file \"tfioffshellmassless.s\" does not \
exist the first argument of CheckDB is evaluated, otherwise the list is \
loaded and assigned to inttable:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{
   RowBox[{"inttable", "=", 
    RowBox[{"CheckDB", "[", 
     RowBox[{
      RowBox[{"Dispatch", "[", 
       RowBox[{"Thread", "[", 
        RowBox[{"allints", "\[Rule]", 
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"WriteString", "[", 
             RowBox[{"\"\<stdout\>\"", ",", "\"\<.\>\""}], "]"}], ";", 
            RowBox[{"TarcerRecurse", "[", 
             RowBox[{
             "allints", "\[LeftDoubleBracket]", "i", 
              "\[RightDoubleBracket]"}], "]"}]}], ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", 
             RowBox[{"Length", "[", "allints", "]"}]}], "}"}]}], "]"}]}], 
        "]"}], "]"}], ",", "\"\<tfioffshellmassless.s\>\""}], "]"}]}], ";"}], 
  "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"0.04400200000000254`", ",", "Null"}], "}"}], 
  TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{
   RowBox[{"result", "=", 
    RowBox[{
     RowBox[{
      RowBox[{"Collect2", "[", 
       RowBox[{"#", ",", 
        RowBox[{"{", 
         RowBox[{"TAI", ",", "TBI", ",", "TJI"}], "}"}], ",", 
        RowBox[{"Factoring", "\[Rule]", "Factor2"}]}], "]"}], "&"}], "/@", 
     RowBox[{"(", 
      RowBox[{"res1", "/.", "inttable"}], ")"}]}]}], ";"}], "]"}]], "Input",
 CellChangeTimes->{3.472902291302947*^9}],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"0.7280450000000016`", ",", "Null"}], "}"}], 
  TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"result", "[", 
  RowBox[{"[", "1", "]"}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{"\[Xi]", " ", "pp", " ", 
     SubsuperscriptBox["C", "A", "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox["D", "3"], " ", 
        RowBox[{"(", 
         RowBox[{"-", "\[Xi]"}], ")"}]}], "+", 
       RowBox[{"8", " ", 
        SuperscriptBox["D", "2"], " ", "\[Xi]"}], "-", 
       RowBox[{"2", " ", 
        SuperscriptBox["D", "2"]}], "-", 
       RowBox[{"17", " ", "D", " ", "\[Xi]"}], "+", 
       RowBox[{"6", " ", "D"}], "+", 
       RowBox[{"8", " ", "\[Xi]"}]}], ")"}], " ", 
     SubsuperscriptBox["g", "s", "4"], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{"ci1", "ci2"}]], " ", 
     SuperscriptBox[
      RowBox[{"TBI", "(", 
       RowBox[{"D", ",", "pp", ",", 
        RowBox[{"(", "\[NoBreak]", GridBox[{
           {"1", "0"},
           {"1", "0"}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.7]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}]}], 
       ")"}], "2"]}], 
    RowBox[{"64", " ", 
     RowBox[{"(", 
      RowBox[{"4", "-", "D"}], ")"}]}]], "-", 
   RowBox[{
    FractionBox["1", 
     RowBox[{"16", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"4", "-", "D"}], ")"}], "2"]}]], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"3", "-", "D"}], ")"}], " ", 
     SubsuperscriptBox["C", "A", "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        RowBox[{"2", " ", 
         SuperscriptBox["D", "3"], " ", 
         SuperscriptBox["\[Xi]", "2"]}]}], "+", 
       RowBox[{"21", " ", 
        SuperscriptBox["D", "2"], " ", 
        SuperscriptBox["\[Xi]", "2"]}], "-", 
       RowBox[{"10", " ", 
        SuperscriptBox["D", "2"], " ", "\[Xi]"}], "-", 
       RowBox[{"68", " ", "D", " ", 
        SuperscriptBox["\[Xi]", "2"]}], "+", 
       RowBox[{"60", " ", "D", " ", "\[Xi]"}], "-", 
       RowBox[{"8", " ", "D"}], "+", 
       RowBox[{"72", " ", 
        SuperscriptBox["\[Xi]", "2"]}], "-", 
       RowBox[{"96", " ", "\[Xi]"}], "+", "32"}], ")"}], " ", 
     SubsuperscriptBox["g", "s", "4"], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{"ci1", "ci2"}]], " ", 
     RowBox[{"TJI", "(", 
      RowBox[{"D", ",", "pp", ",", 
       RowBox[{"(", "\[NoBreak]", GridBox[{
          {"1", "0"},
          {"1", "0"},
          {"1", "0"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.7]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}]}], 
      ")"}]}]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Variables", "[", "result", "]"}], "//", "InputForm"}]], "Input"],

Cell["\<\
{CA, D, Gstrong, Nf, pp, Tf, \[Xi], SD[ci1, ci2], 
 TBI[D, pp, {{1, 0}, {1, 0}}], 
 TJI[D, pp, {{1, 0}, {1, 0}, {1, 0}}]}\
\>", "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Context", "[", "TBI", "]"}]], "Input"],

Cell[BoxData[
 FormBox["\<\"Global`\"\>", TraditionalForm]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Do the expansion of the master integrals in \[CurlyEpsilon] and compare with \
the literature\
\>", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"??", "Tf"}]], "Input",
 CellChangeTimes->{{3.472902385691998*^9, 3.472902386092847*^9}, 
   3.47290247120977*^9}],

Cell["HighEnergyPhysics`qcd`Tf`Tf", "Print", "PrintUsage",
 CellTags->"Info3535454416-8481426"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "Nf"}]], "Input",
 CellChangeTimes->{{3.472902394185211*^9, 3.472902394672224*^9}}],

Cell[BoxData[
 StyleBox["\<\"Nf denotes the number of flavors.\"\>", "MSG"]], "Print", \
"PrintUsage",
 CellTags->"Info3535454416-8481426"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ghselftf", " ", "=", 
  RowBox[{"result", "[", 
   RowBox[{"[", "3", "]"}], "]"}]}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{"2", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"2", "-", "D"}], ")"}], "2"], " ", "Tf", " ", 
     SubscriptBox["C", "A"], " ", 
     SubscriptBox["N", "f"], " ", 
     SubsuperscriptBox["g", "s", "4"], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{"ci1", "ci2"}]], " ", 
     RowBox[{"TJI", "(", 
      RowBox[{"D", ",", "pp", ",", 
       RowBox[{"(", "\[NoBreak]", GridBox[{
          {"1", "0"},
          {"1", "0"},
          {"1", "0"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.7]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}]}], 
      ")"}]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"4", "-", "D"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"6", "-", "D"}], ")"}]}]]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"eta", "=", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"Gamma", "[", 
       RowBox[{
        FractionBox["D", "2"], "-", "1"}], "]"}], "2"], " ", 
     RowBox[{"Gamma", "[", 
      RowBox[{"3", "-", 
       FractionBox["D", "2"]}], "]"}]}], 
    RowBox[{"Gamma", "[", 
     RowBox[{"D", "-", "3"}], "]"}]]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TarcerExpand", "[", 
  RowBox[{
   FractionBox["ghselftf", 
    SuperscriptBox["eta", "2"]], ",", 
   RowBox[{"D", "\[Rule]", 
    RowBox[{"4", "-", 
     RowBox[{"2", " ", "Epsilon"}]}]}], ",", "0"}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"TarcerExpand", "(", 
   RowBox[{
    RowBox[{"-", 
     FractionBox[
      RowBox[{"2", " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"2", "-", "D"}], ")"}], "2"], " ", "Tf", " ", 
       SubscriptBox["C", "A"], " ", 
       SuperscriptBox[
        TemplateBox[{RowBox[{"D", "-", "3"}]},
         "Gamma"], "2"], " ", 
       SubscriptBox["N", "f"], " ", 
       SubsuperscriptBox["g", "s", "4"], " ", 
       SubscriptBox["\[Delta]", 
        RowBox[{"ci1", "ci2"}]], " ", 
       RowBox[{"TJI", "(", 
        RowBox[{"D", ",", "pp", ",", 
         RowBox[{"(", "\[NoBreak]", GridBox[{
            {"1", "0"},
            {"1", "0"},
            {"1", "0"}
           },
           
           GridBoxAlignment->{
            "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
             "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
           GridBoxSpacings->{"Columns" -> {
               Offset[0.27999999999999997`], {
                Offset[0.7]}, 
               Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
               Offset[0.2], {
                Offset[0.4]}, 
               Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}]}], 
        ")"}]}], 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"4", "-", "D"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{"6", "-", "D"}], ")"}], " ", 
       SuperscriptBox[
        TemplateBox[{RowBox[{"3", "-", 
            FractionBox["D", "2"]}]},
         "Gamma"], "2"], " ", 
       SuperscriptBox[
        TemplateBox[{RowBox[{
            FractionBox["D", "2"], "-", "1"}]},
         "Gamma"], "4"]}]]}], ",", 
    RowBox[{"D", "\[Rule]", 
     RowBox[{"4", "-", 
      RowBox[{"2", " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}]}]}], ",", "0"}], ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The above is exactly what is given in eq. (6.13) of \"A.I. Davydychev, \
P.Osland, O.V. Tarasov, Phy. Rev. D ",
 StyleBox["58",
  FontWeight->"Bold"],
 ", 036007 (1998)\"."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ghselfca", " ", "=", 
  RowBox[{"Collect2", "[", 
   RowBox[{
    RowBox[{"Plus", "@@", 
     RowBox[{"Drop", "[", 
      RowBox[{"result", ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "3"}], "}"}]}], "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"TBI", ",", "TJI"}], "}"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{"pp", " ", 
     SubsuperscriptBox["C", "A", "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", " ", 
        SuperscriptBox["D", "2"], " ", 
        SuperscriptBox["\[Xi]", "2"]}], "-", 
       RowBox[{"8", " ", 
        SuperscriptBox["D", "2"], " ", "\[Xi]"}], "-", 
       RowBox[{"15", " ", "D", " ", 
        SuperscriptBox["\[Xi]", "2"]}], "+", 
       RowBox[{"58", " ", "D", " ", "\[Xi]"}], "-", 
       RowBox[{"16", " ", "D"}], "+", 
       RowBox[{"26", " ", 
        SuperscriptBox["\[Xi]", "2"]}], "-", 
       RowBox[{"104", " ", "\[Xi]"}], "+", "56"}], ")"}], " ", 
     SubsuperscriptBox["g", "s", "4"], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{"ci1", "ci2"}]], " ", 
     SuperscriptBox[
      RowBox[{"TBI", "(", 
       RowBox[{"D", ",", "pp", ",", 
        RowBox[{"(", "\[NoBreak]", GridBox[{
           {"1", "0"},
           {"1", "0"}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.7]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}]}], 
       ")"}], "2"]}], 
    RowBox[{"32", " ", 
     RowBox[{"(", 
      RowBox[{"D", "-", "4"}], ")"}]}]], "-", 
   RowBox[{
    FractionBox["1", 
     RowBox[{"16", " ", 
      RowBox[{"(", 
       RowBox[{"D", "-", "6"}], ")"}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"D", "-", "4"}], ")"}], "2"]}]], 
    RowBox[{
     SubsuperscriptBox["C", "A", "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"6", " ", 
        SuperscriptBox["D", "5"], " ", 
        SuperscriptBox["\[Xi]", "2"]}], "-", 
       RowBox[{"115", " ", 
        SuperscriptBox["D", "4"], " ", 
        SuperscriptBox["\[Xi]", "2"]}], "+", 
       RowBox[{"2", " ", 
        SuperscriptBox["D", "4"], " ", "\[Xi]"}], "+", 
       RowBox[{"867", " ", 
        SuperscriptBox["D", "3"], " ", 
        SuperscriptBox["\[Xi]", "2"]}], "-", 
       RowBox[{"130", " ", 
        SuperscriptBox["D", "3"], " ", "\[Xi]"}], "-", 
       RowBox[{"16", " ", 
        SuperscriptBox["D", "3"]}], "-", 
       RowBox[{"3216", " ", 
        SuperscriptBox["D", "2"], " ", 
        SuperscriptBox["\[Xi]", "2"]}], "+", 
       RowBox[{"1188", " ", 
        SuperscriptBox["D", "2"], " ", "\[Xi]"}], "+", 
       RowBox[{"24", " ", 
        SuperscriptBox["D", "2"]}], "+", 
       RowBox[{"5884", " ", "D", " ", 
        SuperscriptBox["\[Xi]", "2"]}], "-", 
       RowBox[{"3840", " ", "D", " ", "\[Xi]"}], "+", 
       RowBox[{"432", " ", "D"}], "-", 
       RowBox[{"4256", " ", 
        SuperscriptBox["\[Xi]", "2"]}], "+", 
       RowBox[{"4160", " ", "\[Xi]"}], "-", "1024"}], ")"}], " ", 
     SubsuperscriptBox["g", "s", "4"], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{"ci1", "ci2"}]], " ", 
     RowBox[{"TJI", "(", 
      RowBox[{"D", ",", "pp", ",", 
       RowBox[{"(", "\[NoBreak]", GridBox[{
          {"1", "0"},
          {"1", "0"},
          {"1", "0"}
         },
         GridBoxAlignment->{
          "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.7]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}]}], 
      ")"}]}]}]}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TarcerExpand", "[", 
  RowBox[{
   FractionBox["ghselfca", 
    SuperscriptBox["eta", "2"]], ",", 
   RowBox[{"D", "\[Rule]", 
    RowBox[{"4", "-", 
     RowBox[{"2", " ", "Epsilon"}]}]}], ",", "1"}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  RowBox[{"TarcerExpand", "(", 
   RowBox[{
    RowBox[{
     FractionBox["1", 
      RowBox[{
       SuperscriptBox[
        TemplateBox[{RowBox[{"3", "-", 
            FractionBox["D", "2"]}]},
         "Gamma"], "2"], " ", 
       SuperscriptBox[
        TemplateBox[{RowBox[{
            FractionBox["D", "2"], "-", "1"}]},
         "Gamma"], "4"]}]], 
     RowBox[{
      SuperscriptBox[
       TemplateBox[{RowBox[{"D", "-", "3"}]},
        "Gamma"], "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        FractionBox[
         RowBox[{"pp", " ", 
          SubsuperscriptBox["C", "A", "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", 
             SuperscriptBox["D", "2"], " ", 
             SuperscriptBox["\[Xi]", "2"]}], "-", 
            RowBox[{"8", " ", 
             SuperscriptBox["D", "2"], " ", "\[Xi]"}], "-", 
            RowBox[{"15", " ", "D", " ", 
             SuperscriptBox["\[Xi]", "2"]}], "+", 
            RowBox[{"58", " ", "D", " ", "\[Xi]"}], "-", 
            RowBox[{"16", " ", "D"}], "+", 
            RowBox[{"26", " ", 
             SuperscriptBox["\[Xi]", "2"]}], "-", 
            RowBox[{"104", " ", "\[Xi]"}], "+", "56"}], ")"}], " ", 
          SubsuperscriptBox["g", "s", "4"], " ", 
          SubscriptBox["\[Delta]", 
           RowBox[{"ci1", "ci2"}]], " ", 
          SuperscriptBox[
           RowBox[{"TBI", "(", 
            RowBox[{"D", ",", "pp", ",", 
             RowBox[{"(", "\[NoBreak]", GridBox[{
                {"1", "0"},
                {"1", "0"}
               },
               
               GridBoxAlignment->{
                "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
                 "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
               GridBoxSpacings->{"Columns" -> {
                   Offset[0.27999999999999997`], {
                    Offset[0.7]}, 
                   Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
                 "Rows" -> {
                   Offset[0.2], {
                    Offset[0.4]}, 
                   Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}]}],
             ")"}], "2"]}], 
         RowBox[{"32", " ", 
          RowBox[{"(", 
           RowBox[{"D", "-", "4"}], ")"}]}]], "-", 
        RowBox[{
         RowBox[{"1", "/", 
          RowBox[{"(", 
           RowBox[{"16", " ", 
            RowBox[{"(", 
             RowBox[{"D", "-", "6"}], ")"}], " ", 
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{"D", "-", "4"}], ")"}], "2"]}], ")"}]}], 
         RowBox[{
          SubsuperscriptBox["C", "A", "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"6", " ", 
             SuperscriptBox["D", "5"], " ", 
             SuperscriptBox["\[Xi]", "2"]}], "-", 
            RowBox[{"115", " ", 
             SuperscriptBox["D", "4"], " ", 
             SuperscriptBox["\[Xi]", "2"]}], "+", 
            RowBox[{"2", " ", 
             SuperscriptBox["D", "4"], " ", "\[Xi]"}], "+", 
            RowBox[{"867", " ", 
             SuperscriptBox["D", "3"], " ", 
             SuperscriptBox["\[Xi]", "2"]}], "-", 
            RowBox[{"130", " ", 
             SuperscriptBox["D", "3"], " ", "\[Xi]"}], "-", 
            RowBox[{"16", " ", 
             SuperscriptBox["D", "3"]}], "-", 
            RowBox[{"3216", " ", 
             SuperscriptBox["D", "2"], " ", 
             SuperscriptBox["\[Xi]", "2"]}], "+", 
            RowBox[{"1188", " ", 
             SuperscriptBox["D", "2"], " ", "\[Xi]"}], "+", 
            RowBox[{"24", " ", 
             SuperscriptBox["D", "2"]}], "+", 
            RowBox[{"5884", " ", "D", " ", 
             SuperscriptBox["\[Xi]", "2"]}], "-", 
            RowBox[{"3840", " ", "D", " ", "\[Xi]"}], "+", 
            RowBox[{"432", " ", "D"}], "-", 
            RowBox[{"4256", " ", 
             SuperscriptBox["\[Xi]", "2"]}], "+", 
            RowBox[{"4160", " ", "\[Xi]"}], "-", "1024"}], ")"}], " ", 
          SubsuperscriptBox["g", "s", "4"], " ", 
          SubscriptBox["\[Delta]", 
           RowBox[{"ci1", "ci2"}]], " ", 
          RowBox[{"TJI", "(", 
           RowBox[{"D", ",", "pp", ",", 
            RowBox[{"(", "\[NoBreak]", GridBox[{
               {"1", "0"},
               {"1", "0"},
               {"1", "0"}
              },
              
              GridBoxAlignment->{
               "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
                "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
              GridBoxSpacings->{"Columns" -> {
                  Offset[0.27999999999999997`], {
                   Offset[0.7]}, 
                  Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, 
                "Rows" -> {
                  Offset[0.2], {
                   Offset[0.4]}, 
                  Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}]}], 
           ")"}]}]}]}], ")"}]}]}], ",", 
    RowBox[{"D", "\[Rule]", 
     RowBox[{"4", "-", 
      RowBox[{"2", " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}]}]}], ",", "1"}], ")"}], TraditionalForm]], "Output"]
}, Open  ]],

Cell[TextData[{
 "This verifies eq. (6.12) of \"A.I. Davydychev, P.Osland, O.V. Tarasov, Phy. \
Rev. D ",
 StyleBox["58",
  FontWeight->"Bold"],
 ", 036007 (1998)\"."
}], "Text"]
}, Open  ]]
},
WindowToolbars->"EditBar",
WindowSize->{1074, 801},
WindowMargins->{{Automatic, 111}, {Automatic, 38}},
ShowSelection->True,
Magnification->1.5,
FrontEndVersion->"8.0 for Linux x86 (64-bit) (October 10, 2011)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "Info3535454416-8481426"->{
  Cell[101827, 2496, 95, 1, 61, "Print",
   CellTags->"Info3535454416-8481426"],
  Cell[102073, 2506, 139, 3, 59, "Print",
   CellTags->"Info3535454416-8481426"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"Info3535454416-8481426", 116533, 2940}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 50, 0, 57, "Subsection"],
Cell[620, 24, 71, 1, 43, "Input"],
Cell[694, 27, 163, 3, 22, "Text"],
Cell[860, 32, 641, 21, 24, "Text"],
Cell[1504, 55, 329, 11, 22, "Text"],
Cell[1836, 68, 154, 3, 22, "Text"],
Cell[CellGroupData[{
Cell[2015, 75, 35, 0, 40, "Subsubsection"],
Cell[CellGroupData[{
Cell[2075, 79, 1092, 27, 179, "Input"],
Cell[3170, 108, 64289, 1276, 282, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[67508, 1390, 96, 2, 40, "Subsubsection"],
Cell[67607, 1394, 75, 2, 43, "Input"],
Cell[67685, 1398, 74, 0, 42, "Text"],
Cell[CellGroupData[{
Cell[67784, 1402, 981, 26, 125, "Input"],
Cell[68768, 1430, 127, 4, 43, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[68932, 1439, 61, 1, 43, "Input"],
Cell[68996, 1442, 15492, 485, 293, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[84549, 1934, 59, 0, 57, "Subsection"],
Cell[84611, 1936, 76, 0, 42, "Text"],
Cell[CellGroupData[{
Cell[84712, 1940, 688, 23, 71, "Input"],
Cell[85403, 1965, 1214, 40, 67, "Output"]
}, Open  ]],
Cell[86632, 2008, 76, 0, 42, "Text"],
Cell[CellGroupData[{
Cell[86733, 2012, 425, 15, 43, "Input"],
Cell[87161, 2029, 887, 27, 49, "Output"]
}, Open  ]],
Cell[88063, 2059, 223, 7, 43, "Input"],
Cell[88289, 2068, 1119, 30, 206, "Input"],
Cell[89411, 2100, 1027, 27, 125, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[90475, 2132, 53, 0, 57, "Subsection"],
Cell[90531, 2134, 76, 2, 43, "Input"],
Cell[90610, 2138, 124, 4, 43, "Input"],
Cell[90737, 2144, 56, 0, 42, "Text"],
Cell[CellGroupData[{
Cell[90818, 2148, 962, 27, 152, "Input"],
Cell[CellGroupData[{
Cell[91805, 2179, 462, 11, 33, "Print"],
Cell[92270, 2192, 485, 11, 33, "Print"],
Cell[92758, 2205, 483, 11, 33, "Print"],
Cell[93244, 2218, 485, 11, 33, "Print"],
Cell[93732, 2231, 482, 11, 33, "Print"],
Cell[94217, 2244, 464, 11, 33, "Print"],
Cell[94684, 2257, 482, 11, 33, "Print"]
}, Open  ]],
Cell[95181, 2271, 115, 3, 43, "Output"]
}, Open  ]],
Cell[95311, 2277, 147, 4, 43, "Input"],
Cell[CellGroupData[{
Cell[95483, 2285, 61, 1, 43, "Input"],
Cell[95547, 2288, 58, 1, 43, "Output"]
}, Open  ]],
Cell[95620, 2292, 435, 7, 119, "Text"],
Cell[CellGroupData[{
Cell[96080, 2303, 811, 22, 179, "Input"],
Cell[96894, 2327, 128, 4, 43, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[97059, 2336, 460, 13, 71, "Input"],
Cell[97522, 2351, 127, 4, 43, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[97686, 2360, 82, 2, 43, "Input"],
Cell[97771, 2364, 3261, 92, 190, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[101069, 2461, 99, 2, 43, "Input"],
Cell[101171, 2465, 147, 4, 107, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[101355, 2474, 62, 1, 43, "Input"],
Cell[101420, 2477, 70, 1, 43, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[101539, 2484, 123, 3, 57, "Subsection"],
Cell[CellGroupData[{
Cell[101687, 2491, 137, 3, 43, "Input"],
Cell[101827, 2496, 95, 1, 61, "Print",
 CellTags->"Info3535454416-8481426"]
}, Open  ]],
Cell[CellGroupData[{
Cell[101959, 2502, 111, 2, 43, "Input"],
Cell[102073, 2506, 139, 3, 59, "Print",
 CellTags->"Info3535454416-8481426"]
}, Open  ]],
Cell[CellGroupData[{
Cell[102249, 2514, 118, 3, 43, "Input"],
Cell[102370, 2519, 1203, 35, 125, "Output"]
}, Open  ]],
Cell[103588, 2557, 382, 13, 86, "Input"],
Cell[CellGroupData[{
Cell[103995, 2574, 242, 7, 70, "Input"],
Cell[104240, 2583, 1860, 54, 171, "Output"]
}, Open  ]],
Cell[106115, 2640, 202, 6, 69, "Text"],
Cell[CellGroupData[{
Cell[106342, 2650, 327, 10, 43, "Input"],
Cell[106672, 2662, 3859, 106, 285, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[110568, 2773, 242, 7, 70, "Input"],
Cell[110813, 2782, 5141, 133, 429, "Output"]
}, Open  ]],
Cell[115969, 2918, 178, 6, 43, "Text"]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
